@@ -15,9 +15,14 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
+
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <stdio.h>
 
-#include "config.h"
+#include "../lib/color.h"
 #include "intl.h"
 
 #include "color_area.h"
@@ -37,9 +42,13 @@ int active_color = 0;
 /*  Static variables  */
 GtkWidget *color_area;
 static GdkGC *color_area_gc = NULL;
+static GdkGC *mask_gc       = NULL;
 static GdkPixmap *color_area_pixmap = NULL;
+static GdkBitmap *color_area_mask   = NULL;
 static GdkPixmap *default_pixmap = NULL;
+static GdkBitmap *default_mask      = NULL;
 static GdkPixmap *swap_pixmap = NULL;
+static GdkBitmap *swap_mask         = NULL;
 
 static GtkWidget *color_select = NULL;
 static int color_select_active = 0;
@@ -61,6 +70,9 @@ static gint
 color_selection_delete (GtkWidget               *w,
 			GdkEvent                *event,
 			GtkColorSelectionDialog *cs);
+static gint
+color_selection_destroy (GtkWidget               *w,
+			 GtkColorSelectionDialog *cs);
 static void
 color_selection_changed (GtkWidget *w,
                          GtkColorSelectionDialog *cs);
@@ -70,10 +82,10 @@ static int
 color_area_target (int x,
                    int y)
 {
-  int rect_w, rect_h;
-  int width, height;
+  gint rect_w, rect_h;
+  gint width, height;
 
-  gdk_window_get_size (color_area_pixmap, &width, &height);
+  gdk_drawable_get_size (color_area_pixmap, &width, &height);
 
   rect_w = width * 0.65;
   rect_h = height * 0.65;
@@ -101,19 +113,19 @@ color_area_draw ()
   Color col;
   GdkColor *win_bg;
   GdkColor fg, bg, bd;
-  int rect_w, rect_h;
-  int width, height;
-  int def_width, def_height;
-  int swap_width, swap_height;
+  gint rect_w, rect_h;
+  gint width, height;
+  gint def_width, def_height;
+  gint swap_width, swap_height;
+  GdkColor  mask_pattern;
 
-  if (!color_area_pixmap)     /* we haven't gotten initial expose yet,
-                               * no point in drawing anything */
-     return;
+  /* Check we haven't gotten initial expose yet,
+   * no point in drawing anything
+   */
+  if (!color_area_pixmap || !color_area_gc)
+    return;
 
-  if (!color_area_gc)
-    color_area_gc = gdk_gc_new (color_area_pixmap);
-
-  gdk_window_get_size (color_area_pixmap, &width, &height);
+  gdk_drawable_get_size (color_area_pixmap, &width, &height);
 
   win_bg = &(color_area->style->bg[GTK_STATE_NORMAL]);
   col = attributes_get_foreground();
@@ -125,6 +137,15 @@ color_area_draw ()
   rect_w = width * 0.65;
   rect_h = height * 0.65;
 
+  /*  initialize the mask to transparent  */
+  mask_pattern.pixel = 0;
+  gdk_gc_set_foreground (mask_gc, &mask_pattern);
+  gdk_draw_rectangle (color_area_mask, mask_gc, TRUE, 0, 0, -1, -1);
+
+  /*  set the mask's gc to opaque  */
+  mask_pattern.pixel = 1;
+  gdk_gc_set_foreground (mask_gc, &mask_pattern);
+
   gdk_gc_set_foreground (color_area_gc, win_bg);
   gdk_draw_rectangle (color_area_pixmap, color_area_gc, 1,
 		      0, 0, width, height);
@@ -132,36 +153,63 @@ color_area_draw ()
   gdk_gc_set_foreground (color_area_gc, &bg);
   gdk_draw_rectangle (color_area_pixmap, color_area_gc, 1,
 		      (width - rect_w), (height - rect_h), rect_w, rect_h);
+  gdk_draw_rectangle (color_area_mask, mask_gc, TRUE,
+		      (width - rect_w), (height - rect_h), rect_w, rect_h);
 
   if (active_color == FOREGROUND)
-    gtk_draw_shadow (color_area->style, color_area_pixmap, GTK_STATE_NORMAL, GTK_SHADOW_OUT,
-		     (width - rect_w), (height - rect_h), rect_w, rect_h);
+    gtk_paint_shadow (color_area->style, color_area_pixmap, GTK_STATE_NORMAL,
+                      GTK_SHADOW_OUT,
+                      NULL, color_area, NULL,
+		      (width - rect_w), (height - rect_h),
+                      rect_w, rect_h);
   else
-    gtk_draw_shadow (color_area->style, color_area_pixmap, GTK_STATE_NORMAL, GTK_SHADOW_IN,
-		     (width - rect_w), (height - rect_h), rect_w, rect_h);
+    gtk_paint_shadow (color_area->style, color_area_pixmap, GTK_STATE_NORMAL,
+                      GTK_SHADOW_IN,
+                      NULL, color_area, NULL,
+		      (width - rect_w), (height - rect_h),
+                      rect_w, rect_h);
 
   gdk_gc_set_foreground (color_area_gc, &fg);
   gdk_draw_rectangle (color_area_pixmap, color_area_gc, 1,
 		      0, 0, rect_w, rect_h);
+  gdk_draw_rectangle (color_area_mask, mask_gc, TRUE,
+		      0, 0, rect_w, rect_h);
 
   if (active_color == FOREGROUND)
-    gtk_draw_shadow (color_area->style, color_area_pixmap, GTK_STATE_NORMAL, GTK_SHADOW_IN,
-		     0, 0, rect_w, rect_h);
+    gtk_paint_shadow (color_area->style, color_area_pixmap, GTK_STATE_NORMAL,
+                      GTK_SHADOW_IN,
+                      NULL, color_area, NULL,
+                      0, 0,
+                      rect_w, rect_h);
   else
-    gtk_draw_shadow (color_area->style, color_area_pixmap, GTK_STATE_NORMAL, GTK_SHADOW_OUT,
-		     0, 0, rect_w, rect_h);
+    gtk_paint_shadow (color_area->style, color_area_pixmap, GTK_STATE_NORMAL,
+                      GTK_SHADOW_OUT,
+                      NULL, color_area, NULL,
+                      0, 0,
+                      rect_w, rect_h);
 
-
-  gdk_window_get_size (default_pixmap, &def_width, &def_height);
-  gdk_draw_pixmap (color_area_pixmap, color_area_gc, default_pixmap,
+  /*  draw the default pixmap  */
+  gdk_drawable_get_size (default_pixmap, &def_width, &def_height);
+  gdk_draw_drawable (color_area_pixmap, color_area_gc, default_pixmap,
+		     0, 0, 0, height - def_height, def_width, def_height);
+  gdk_draw_drawable (color_area_mask, mask_gc, default_mask,
 		   0, 0, 0, height - def_height, def_width, def_height);
 
-  gdk_window_get_size (swap_pixmap, &swap_width, &swap_height);
-  gdk_draw_pixmap (color_area_pixmap, color_area_gc, swap_pixmap,
+  /*  draw the swap pixmap  */
+  gdk_drawable_get_size (swap_pixmap, &swap_width, &swap_height);
+  gdk_draw_drawable (color_area_pixmap, color_area_gc, swap_pixmap,
+		     0, 0, width - swap_width, 0, swap_width, swap_height);
+  gdk_draw_drawable (color_area_mask, mask_gc, swap_mask,
 		   0, 0, width - swap_width, 0, swap_width, swap_height);
 
-  gdk_draw_pixmap (color_area->window, color_area_gc, color_area_pixmap,
+  /*  draw the widget  */
+  gdk_gc_set_clip_mask (color_area_gc, color_area_mask);
+  gdk_gc_set_clip_origin (color_area_gc, 0, 0);
+  gdk_draw_drawable (color_area->window, color_area_gc, color_area_pixmap,
 		   0, 0, 0, 0, width, height);
+
+  /*  reset the clip mask  */
+  gdk_gc_set_clip_mask (color_area_gc, NULL);
 }
 
 static void
@@ -169,15 +217,13 @@ color_selection_ok (GtkWidget               *w,
                     GtkColorSelectionDialog *cs)
 {
   GtkColorSelection *colorsel;
-  gdouble color[3];
+  GdkColor color;
   Color col;
 
   colorsel=GTK_COLOR_SELECTION(cs->colorsel);
 
-  gtk_color_selection_get_color(colorsel,color);
-  col.red = color[0];
-  col.green= color[1];
-  col.blue = color[2];
+  gtk_color_selection_get_current_color(colorsel,&color);
+  GDK_COLOR_TO_DIA(color, col);
 
   if (edit_color == FOREGROUND) {
     attributes_set_foreground(&col);
@@ -186,7 +232,7 @@ color_selection_ok (GtkWidget               *w,
   }
   color_area_draw ();
 
-  gtk_color_selection_set_color(colorsel,color);
+  /*  gtk_color_selection_set_currentcolor(colorsel,&color);*/
 
   gtk_widget_hide(color_select);
   color_select_active = 0;
@@ -196,7 +242,8 @@ static void
 color_selection_cancel (GtkWidget               *w,
 			GtkColorSelectionDialog *cs)
 {
-  gtk_widget_hide(color_select);
+  if (color_select != NULL)
+    gtk_widget_hide(color_select);
   color_select_active = 0;
   attributes_set_foreground(&stored_foreground);
   attributes_set_background(&stored_background);
@@ -213,20 +260,27 @@ color_selection_delete (GtkWidget               *w,
   return TRUE;
 }
 
+static gint
+color_selection_destroy (GtkWidget               *w,
+			 GtkColorSelectionDialog *cs)
+{
+  color_select = NULL;
+  color_selection_cancel(w,cs);
+  return TRUE;
+}
+
 static void
 color_selection_changed (GtkWidget *w,
                          GtkColorSelectionDialog *cs)
 {
   GtkColorSelection *colorsel;
-  gdouble color[3];
+  GdkColor color;
   Color col;
 
   colorsel=GTK_COLOR_SELECTION(cs->colorsel);
 
-  gtk_color_selection_get_color(colorsel,color);
-  col.red = color[0];
-  col.green= color[1];
-  col.blue = color[2];
+  gtk_color_selection_get_current_color(colorsel,&color);
+  GDK_COLOR_TO_DIA(color, col);
 
   if (edit_color == FOREGROUND) {
     attributes_set_foreground(&col);
@@ -240,8 +294,8 @@ static void
 color_area_edit (void)
 {
   Color col;
-  gdouble color[3];
   GtkWidget *window;
+  GdkColor color;
 
   if (!color_select_active) {
     stored_foreground  = attributes_get_foreground();
@@ -257,66 +311,67 @@ color_area_edit (void)
   }
 
   if (! color_select) {
-    window = color_select = gtk_color_selection_dialog_new(_("Select color"));
+    window = color_select = 
+      gtk_color_selection_dialog_new(edit_color==FOREGROUND?
+				     _("Select foreground color"):
+				     _("Select background color"));
     color_select_active = 1;
-    
+
+    gtk_color_selection_set_has_palette (
+	GTK_COLOR_SELECTION (GTK_COLOR_SELECTION_DIALOG (window)->colorsel),
+	TRUE);
+
     gtk_color_selection_set_update_policy(
         GTK_COLOR_SELECTION (GTK_COLOR_SELECTION_DIALOG (window)->colorsel),
 	GTK_UPDATE_CONTINUOUS);
     
     gtk_window_set_position (GTK_WINDOW (window), GTK_WIN_POS_MOUSE);
     
-    gtk_signal_connect (GTK_OBJECT (window), "delete_event",
-			GTK_SIGNAL_FUNC(color_selection_delete),
+    g_signal_connect (G_OBJECT (window), "delete_event",
+		      G_CALLBACK(color_selection_delete),
 			window);
     
-    gtk_signal_connect (
-	GTK_OBJECT (GTK_COLOR_SELECTION_DIALOG (window)->colorsel),
+    g_signal_connect (GTK_OBJECT (window), "destroy",
+		      G_CALLBACK(color_selection_destroy),
+			window);
+    
+    g_signal_connect (
+	G_OBJECT (GTK_COLOR_SELECTION_DIALOG (window)->colorsel),
 	"color_changed",
-	GTK_SIGNAL_FUNC(color_selection_changed),
+	G_CALLBACK(color_selection_changed),
 	window);
     
-    gtk_signal_connect (
-	GTK_OBJECT (GTK_COLOR_SELECTION_DIALOG (window)->ok_button),
+    g_signal_connect (
+	G_OBJECT (GTK_COLOR_SELECTION_DIALOG (window)->ok_button),
 	"clicked",
-	GTK_SIGNAL_FUNC(color_selection_ok),
+	G_CALLBACK(color_selection_ok),
 	window);
 
-    gtk_signal_connect (
-	GTK_OBJECT (GTK_COLOR_SELECTION_DIALOG (window)->cancel_button),
+    g_signal_connect (
+	G_OBJECT (GTK_COLOR_SELECTION_DIALOG (window)->cancel_button),
 	"clicked",
-	GTK_SIGNAL_FUNC(color_selection_cancel),
+	G_CALLBACK(color_selection_cancel),
 	window);
 
 
     /* Make sure window is shown before setting its colors: */
-    gtk_widget_show_now (window);
+    gtk_widget_show_now (color_select);
       
-    color[0] = col.red;
-    color[1] = col.green;
-    color[2] = col.blue;
-
-    gtk_color_selection_set_color(
-         GTK_COLOR_SELECTION (GTK_COLOR_SELECTION_DIALOG (window)->colorsel),
-   	 color);
-    gtk_color_selection_set_color(
-         GTK_COLOR_SELECTION (GTK_COLOR_SELECTION_DIALOG (window)->colorsel),
-	 color);
   } else {
+    gtk_window_set_title(GTK_WINDOW(color_select),
+			 edit_color==FOREGROUND?
+			 _("Select foreground color"):
+			 _("Select background color"));
     if (! color_select_active) {
       gtk_widget_show (color_select);
     }
-    color[0] = col.red;
-    color[1] = col.green;
-    color[2] = col.blue;
-
-    gtk_color_selection_set_color(
-         GTK_COLOR_SELECTION (GTK_COLOR_SELECTION_DIALOG (color_select)->colorsel),
-   	 color);
-    gtk_color_selection_set_color(
-         GTK_COLOR_SELECTION (GTK_COLOR_SELECTION_DIALOG (color_select)->colorsel),
-	 color);
   }
+  DIA_COLOR_TO_GDK(col, color);
+
+  gtk_color_selection_set_current_color(
+	GTK_COLOR_SELECTION (GTK_COLOR_SELECTION_DIALOG (color_select)->colorsel),
+	&color);
+
 }
 
 static gint
@@ -324,24 +379,35 @@ color_area_events (GtkWidget *widget,
 		   GdkEvent  *event)
 {
   GdkEventButton *bevent;
-  GdkEventConfigure *cevent;
   int target;
-
-  
   
   switch (event->type)
     {
     case GDK_CONFIGURE:
-      cevent = (GdkEventConfigure *)  event;
-      if (cevent->width > 1) {
-	color_area_pixmap = gdk_pixmap_new (widget->window,
-					    cevent->width,
-					    cevent->height, -1);
-      }
+      if (color_area_pixmap)
+	{
+	  g_object_unref (color_area_pixmap);
+	  g_object_unref (color_area_mask);
+	}
+
+	color_area_pixmap = gdk_pixmap_new (color_area->window,
+					  color_area->allocation.width,
+					  color_area->allocation.height, -1);
+        color_area_mask = gdk_pixmap_new (color_area->window,
+					color_area->allocation.width,
+					color_area->allocation.height, 1);
       break;
     case GDK_EXPOSE:
-      if (color_area_pixmap)
-	color_area_draw ();
+      if (GTK_WIDGET_DRAWABLE (color_area))
+	{
+	  if (!color_area_gc)
+	    {
+	      color_area_gc = gdk_gc_new (color_area->window);
+	      mask_gc = gdk_gc_new (color_area_mask);
+	    }
+
+	  color_area_draw ();
+	}
       break;
     case GDK_BUTTON_PRESS:
       bevent = (GdkEventButton *) event;
@@ -381,17 +447,26 @@ GtkWidget *
 color_area_create (int        width,
 		   int        height,
 		   GdkPixmap *default_pmap,
-		   GdkPixmap *swap_pmap)
+		   GdkBitmap *default_msk,
+		   GdkPixmap *swap_pmap,
+		   GdkBitmap *swap_msk)
 {
-  color_area = gtk_drawing_area_new ();
-  gtk_drawing_area_size (GTK_DRAWING_AREA (color_area), width, height);
-  gtk_widget_set_events (color_area, GDK_EXPOSURE_MASK | GDK_BUTTON_PRESS_MASK);
-  gtk_signal_connect (GTK_OBJECT (color_area), "event",
-		      (GtkSignalFunc) color_area_events,
-		      NULL);
-  default_pixmap = default_pmap;
-  swap_pixmap    = swap_pmap;
+  GtkWidget *event_box;
 
-  return color_area;
+  event_box = gtk_event_box_new();
+  color_area = gtk_drawing_area_new ();
+  gtk_widget_set_size_request (color_area, width, height);
+  gtk_widget_set_events (color_area, GDK_EXPOSURE_MASK | GDK_BUTTON_PRESS_MASK);
+  g_signal_connect (G_OBJECT (color_area), "event",
+		       G_CALLBACK(color_area_events),
+		       NULL);
+  default_pixmap = default_pmap;
+  default_mask   = default_msk;
+  swap_pixmap    = swap_pmap;
+  swap_mask      = swap_msk;
+
+  gtk_widget_show(color_area);
+  gtk_container_add(GTK_CONTAINER(event_box), color_area);
+  return event_box;
 }
 
