@@ -33,23 +33,25 @@ connectionpoint_draw(ConnectionPoint *conpoint,
 {
   int x,y;
   Point *point = &conpoint->pos;
-  Renderer *renderer = ddisp->renderer;
+  DiaRenderer *renderer = ddisp->renderer;
+  DiaRendererClass *renderer_ops = DIA_RENDERER_GET_CLASS (ddisp->renderer);
+  DiaInteractiveRendererInterface *irenderer =
+    DIA_GET_INTERACTIVE_RENDERER_INTERFACE (ddisp->renderer);
   
   ddisplay_transform_coords(ddisp, point->x, point->y, &x, &y);
 
-  (renderer->ops->set_linewidth)(renderer, 0.0);
-  (renderer->ops->set_linestyle)(renderer, LINESTYLE_SOLID);
+  renderer_ops->set_linewidth (renderer, 0.0);
+  renderer_ops->set_linestyle (renderer, LINESTYLE_SOLID);
 
-  (renderer->interactive_ops->draw_pixel_line)(renderer,
-					       x-CP_SZ,y-CP_SZ,
-					       x+CP_SZ,y+CP_SZ,
-					       &connectionpoint_color);
-
-  (renderer->interactive_ops->draw_pixel_line)(renderer,
-					       x+CP_SZ,y-CP_SZ,
-					       x-CP_SZ,y+CP_SZ,
-					       &connectionpoint_color);
+  irenderer->draw_pixel_line (renderer,
+			x-CP_SZ,y-CP_SZ,
+			x+CP_SZ,y+CP_SZ,
+			&connectionpoint_color);
 
+  irenderer->draw_pixel_line (renderer,
+			x+CP_SZ,y-CP_SZ,
+			x-CP_SZ,y+CP_SZ,
+			&connectionpoint_color);
 }
 
 void
@@ -102,25 +104,29 @@ diagram_update_connections_object(Diagram *dia, Object *obj,
       while (list!=NULL) {
 	connected_obj = (Object *) list->data;
 
+	object_add_updates(connected_obj, dia);
 	handle = NULL;
 	for (j=0;j<connected_obj->num_handles;j++) {
-	  if (connected_obj->handles[j]->connected_to == cp)
+	  if (connected_obj->handles[j]->connected_to == cp) {
 	    handle = connected_obj->handles[j];
+	    connected_obj->ops->move_handle(connected_obj, handle, &cp->pos,
+					    cp, HANDLE_MOVE_CONNECTED,0);
+	  }
 	}
-	assert(handle!=NULL);
-	
-	object_add_updates(connected_obj, dia);
-	connected_obj->ops->move_handle(connected_obj, handle ,
-					&cp->pos, HANDLE_MOVE_CONNECTED,0);
 	object_add_updates(connected_obj, dia);
 
 	diagram_update_connections_object(dia, connected_obj, FALSE);
 	
 	list = g_list_next(list);
       }
-      
     }
-    
+  }
+  if (obj->children) {
+    GList *child;
+    for (child = obj->children; child != NULL; child = child->next) {
+      Object *child_obj = (Object *)child->data;
+      diagram_update_connections_object(dia, child_obj, update_nonmoved);
+    }    
   }
 }
 
