@@ -23,6 +23,7 @@
 #include "object_ops.h"
 #include "preferences.h"
 #include "undo.h"
+#include "cursor.h"
 
 static void create_object_button_press(CreateObjectTool *tool, GdkEventButton *event,
 				     DDisplay *ddisp);
@@ -42,6 +43,9 @@ create_object_button_press(CreateObjectTool *tool, GdkEventButton *event,
   Handle *handle1;
   Handle *handle2;
   Object *obj;
+  Object *parent_obj;
+  real click_distance;
+  GList *avoid = NULL;
 
   ddisplay_untransform_coords(ddisp,
 			      (int)event->x, (int)event->y,
@@ -50,22 +54,48 @@ create_object_button_press(CreateObjectTool *tool, GdkEventButton *event,
   origpoint = clickedpoint;
 
   snap_to_grid(ddisp, &clickedpoint.x, &clickedpoint.y);
-  
-  obj = tool->objtype->ops->create(&clickedpoint, tool->user_data,
-				   &handle1, &handle2);
+
+  click_distance = ddisplay_untransform_length(ddisp, 3.0);
+
+  obj = dia_object_default_create (tool->objtype, &clickedpoint,
+                                   tool->user_data,
+                                   &handle1, &handle2);
+
   diagram_add_object(ddisp->diagram, obj);
 
+  /* Make sure to not parent to itself (!) */
+  avoid = g_list_prepend(avoid, obj);
   /* Try a connect */
   if (handle1 != NULL &&
       handle1->connect_type != HANDLE_NONCONNECTABLE) {
     ConnectionPoint *connectionpoint;
     connectionpoint =
-      object_find_connectpoint_display(ddisp, &origpoint);
+      object_find_connectpoint_display(ddisp, &origpoint, obj);
     if (connectionpoint != NULL) {
       (obj->ops->move)(obj, &origpoint);
+      /* Make sure to not parent to the object connected to */
+      avoid = g_list_prepend(avoid, connectionpoint->object);
     }
   }
   
+
+  parent_obj = diagram_find_clicked_object_except(ddisp->diagram, 
+						  &clickedpoint,
+						  click_distance,
+						  avoid);
+
+  g_list_free(avoid);
+
+  if (parent_obj && parent_obj->can_parent) /* starting point is within another object */
+  {
+    Change *change = undo_parenting(ddisp->diagram, parent_obj, obj, TRUE);
+    (change->apply)(change, ddisp->diagram);
+    /*
+    obj->parent = parent_obj;
+    parent_obj->children = g_list_append(parent_obj->children, obj);
+    */
+  }
+
   if (!(event->state & GDK_SHIFT_MASK)) {
     /* Not Multi-select => remove current selection */
     diagram_remove_all_selected(ddisp->diagram, TRUE);
@@ -73,7 +103,7 @@ create_object_button_press(CreateObjectTool *tool, GdkEventButton *event,
   diagram_select(ddisp->diagram, obj);
 
   obj->ops->selectf(obj, &clickedpoint,
-		   (Renderer *)ddisp->renderer);
+		ddisp->renderer);
 
   tool->obj = obj;
 
@@ -84,6 +114,7 @@ create_object_button_press(CreateObjectTool *tool, GdkEventButton *event,
   }
 
   object_add_updates(obj, ddisp->diagram);
+  ddisplay_do_update_menu_sensitivity(ddisp);
   diagram_flush(ddisp->diagram);
   
   if (handle2 != NULL) {
@@ -94,10 +125,12 @@ create_object_button_press(CreateObjectTool *tool, GdkEventButton *event,
     gdk_pointer_grab (ddisp->canvas->window, FALSE,
 		      GDK_POINTER_MOTION_HINT_MASK | GDK_BUTTON1_MOTION_MASK | GDK_BUTTON_RELEASE_MASK,
 		      NULL, NULL, event->time);
+    ddisplay_set_all_cursor(get_cursor(CURSOR_SCROLL));
   } else {
     diagram_update_extents(ddisp->diagram);
     tool->moving = FALSE;
   }
+
 }
 
 static void
@@ -116,7 +149,7 @@ create_object_button_release(CreateObjectTool *tool, GdkEventButton *event,
 
     object_add_updates(tool->obj, ddisp->diagram);
     tool->obj->ops->move_handle(tool->obj, tool->handle, &tool->last_to,
-				HANDLE_MOVE_CREATE_FINAL,0);
+				NULL, HANDLE_MOVE_CREATE_FINAL, 0);
     object_add_updates(tool->obj, ddisp->diagram);
 
   }
@@ -141,8 +174,9 @@ create_object_button_release(CreateObjectTool *tool, GdkEventButton *event,
 
   undo_set_transactionpoint(ddisp->diagram->undo);
   
-  if (prefs.reset_tools_after_create)
+  if (prefs.reset_tools_after_create != tool->invert_persistence)
       tool_reset();
+  ddisplay_set_all_cursor(default_cursor);
 }
 static void
 create_object_motion(CreateObjectTool *tool, GdkEventMotion *event,
@@ -156,19 +190,24 @@ create_object_motion(CreateObjectTool *tool, GdkEventMotion *event,
   
   ddisplay_untransform_coords(ddisp, event->x, event->y, &to.x, &to.y);
 
+  /* make sure the new object is restricted to its parent */
+  parent_handle_move_out_check(tool->obj, &to);
+
   /* Move to ConnectionPoint if near: */
   if ((tool->handle != NULL &&
        tool->handle->connect_type != HANDLE_NONCONNECTABLE) &&
       ((connectionpoint =
-	object_find_connectpoint_display(ddisp, &to)) != NULL)) {
+	object_find_connectpoint_display(ddisp, &to, tool->obj)) != NULL)) {
     to = connectionpoint->pos;
+    ddisplay_set_all_cursor(get_cursor(CURSOR_CONNECT));
   } else {
     /* No connectionopoint near, then snap to grid (if enabled) */
     snap_to_grid(ddisp, &to.x, &to.y);
+    ddisplay_set_all_cursor(get_cursor(CURSOR_SCROLL));
   }
 
   object_add_updates(tool->obj, ddisp->diagram);
-  tool->obj->ops->move_handle(tool->obj, tool->handle, &to,
+  tool->obj->ops->move_handle(tool->obj, tool->handle, &to, connectionpoint,
 			      HANDLE_MOVE_CREATE, 0);
   object_add_updates(tool->obj, ddisp->diagram);
   
@@ -181,11 +220,13 @@ create_object_motion(CreateObjectTool *tool, GdkEventMotion *event,
 
 
 
-Tool *create_create_object_tool(ObjectType *objtype, void *user_data)
+Tool *
+create_create_object_tool(ObjectType *objtype, void *user_data,
+			  int invert_persistence)
 {
   CreateObjectTool *tool;
 
-  tool = g_new(CreateObjectTool, 1);
+  tool = g_new0(CreateObjectTool, 1);
   tool->tool.type = CREATE_OBJECT_TOOL;
   tool->tool.button_press_func = (ButtonPressFunc) &create_object_button_press;
   tool->tool.button_release_func = (ButtonReleaseFunc) &create_object_button_release;
@@ -195,6 +236,7 @@ Tool *create_create_object_tool(ObjectType *objtype, void *user_data)
   tool->objtype = objtype;
   tool->user_data = user_data;
   tool->moving = FALSE;
+  tool->invert_persistence = invert_persistence;
   
   return (Tool *) tool;
 }
