@@ -1,4 +1,4 @@
-/* Dia -- an diagram creation/manipulation program
+/* Dia -- an diagram creation/manipulation program -*- c -*-
  * Copyright (C) 1998 Alexander Larsson
  *
  * This program is free software; you can redistribute it and/or modify
@@ -30,11 +30,15 @@ typedef struct _Diagram Diagram;
 #include "diagramdata.h"
 #include "undo.h"
 #include "filter.h"
+#include "menus.h"
 
 struct _Diagram {
   char *filename;
   int unsaved;            /* True if diagram is created but not saved.*/
-  int modified;
+  int mollified;
+  gboolean autosaved;     /* True if the diagram is autosaved since last mod */
+  char *autosavefilename;     /* Holds the name of the current autosave file
+			       * for this diagram, or NULL.  */
   
   DiagramData *data;
   
@@ -44,11 +48,21 @@ struct _Diagram {
   UndoStack *undo;
 };
 
-extern GList *open_diagrams; /* Read only! */
+struct _object_extent
+{
+  Object *object;
+  Rectangle *extent;
+};
+
+typedef struct _object_extent object_extent;
+
+GList *dia_open_diagrams(void); /* Read only! */
 
 Diagram *diagram_load(const char *filename, DiaImportFilter *ifilter);
+int diagram_load_into (Diagram *dest, const char *filename, DiaImportFilter *ifilter);
 Diagram *new_diagram(const char *filename); /*Note: filename is copied*/
 void diagram_destroy(Diagram *dia);
+gboolean diagram_is_modified(Diagram *dia);
 void diagram_modified(Diagram *dia);
 void diagram_set_modified(Diagram *dia, int modified);
 void diagram_add_ddisplay(Diagram *dia, DDisplay *ddisp);
@@ -74,27 +88,41 @@ void diagram_flush(Diagram *dia);
 Object *diagram_find_clicked_object(Diagram *dia,
 				    Point *pos,
 				    real maxdist);
+Object *diagram_find_clicked_object_except(Diagram *dia,
+					   Point *pos,
+					   real maxdist,
+					   GList *avoid);
 real diagram_find_closest_handle(Diagram *dia, Handle **handle,
 				 Object **obj, Point *pos);
 real diagram_find_closest_connectionpoint(Diagram *dia,
 					  ConnectionPoint **cp,
-					  Point *pos);
+					  Point *pos,
+					  Object *notthis);
 void diagram_update_extents(Diagram *dia);
+gint diagram_parent_sort_cb(object_extent ** a, object_extent **b);
 
-void diagram_update_menu_sensitivity(Diagram *dia);
+void diagram_update_menu_sensitivity (Diagram *dia, UpdatableMenuItems *items);
+void diagram_update_menubar_sensitivity(Diagram *dia, UpdatableMenuItems *items);
+void diagram_update_popupmenu_sensitivity(Diagram *dia);
 
 void diagram_place_under_selected(Diagram *dia);
 void diagram_place_over_selected(Diagram *dia);
+void diagram_place_down_selected(Diagram *dia);
+void diagram_place_up_selected(Diagram *dia);
 void diagram_group_selected(Diagram *dia);
 void diagram_ungroup_selected(Diagram *dia);
+void diagram_parent_selected(Diagram *dia);
+void diagram_unparent_selected(Diagram *dia);
+void diagram_unparent_children_selected(Diagram *dia);
 
 void diagram_set_filename(Diagram *dia, char *filename);
-
-void diagram_import_from_xfig(Diagram *dia, char *filename);
+gchar *diagram_get_name(Diagram *dia);
 
 int diagram_modified_exists(void);
 
-void diagram_redraw_all();
+void diagram_redraw_all(void);
+
+void diagram_object_modified(Diagram *dia, Object *object);
 
 #endif /* DIAGRAM_H */
 
