@@ -19,10 +19,15 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+#include <config.h>
+
 #include "filedlg.h"
+#include <sys/types.h>
 #include <sys/stat.h>
 #include <string.h>
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
+#endif
 #include <gtk/gtk.h>
 #include "intl.h"
 #include "filter.h"
@@ -30,10 +35,13 @@
 #include "message.h"
 #include "layer_dialog.h"
 #include "load_save.h"
+#include "preferences.h"
+#include "interface.h"
 
 static GtkWidget *opendlg = NULL, *open_options = NULL, *open_omenu = NULL;
 static GtkWidget *savedlg = NULL;
 static GtkWidget *exportdlg = NULL, *export_options = NULL, *export_omenu=NULL;
+static GtkWidget *compressbutton = NULL;
 
 static int
 file_dialog_hide (GtkWidget *filesel)
@@ -60,14 +68,25 @@ set_true_callback(GtkWidget *w, int *data)
   *data = TRUE;
 }
 
+static void
+toggle_compress_callback(GtkWidget *w, gpointer data)
+{
+  // Changes prefs exactly when the user toggles the setting, i.e.
+  // the setting really remembers what the user chose last time, but
+  // lets diagrams of the opposite kind stay that way unless the user
+  // intervenes.
+  prefs.new_diagram.compress_save =
+    gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(compressbutton));
+}
+
 static void
 open_set_extension(GtkObject *item)
 {
   DiaImportFilter *ifilter = gtk_object_get_user_data(item);
   GString *s;
-  gchar *text = gtk_entry_get_text(GTK_ENTRY(GTK_FILE_SELECTION(opendlg)
-                                             ->selection_entry));
-  gchar *last_dot = strrchr(text, '.');
+  const gchar *text = gtk_entry_get_text(GTK_ENTRY(GTK_FILE_SELECTION(opendlg)
+                                                   ->selection_entry));
+  const gchar *last_dot = strrchr(text, '.');
 
   if (!ifilter || last_dot == text || text[0] == '\0' ||
       ifilter->extensions[0] == NULL)
@@ -89,6 +108,7 @@ create_open_menu(void)
   GtkWidget *menu;
   GtkWidget *item;
   GList *tmp;
+  
 
   menu = gtk_menu_new();
   item = gtk_menu_item_new_with_label(_("By extension"));
@@ -101,13 +121,15 @@ create_open_menu(void)
   
   for (tmp = filter_get_import_filters(); tmp != NULL; tmp = tmp->next) {
     DiaImportFilter *ifilter = tmp->data;
+    gchar *filter_label;
 
     if (!ifilter)
       continue;
-    item = gtk_menu_item_new_with_label(
-		filter_get_import_filter_label(ifilter));
+    filter_label = filter_get_import_filter_label(ifilter);
+    item = gtk_menu_item_new_with_label(filter_label);
+    g_free(filter_label);
     gtk_object_set_user_data(GTK_OBJECT(item), ifilter);
-    gtk_signal_connect(GTK_OBJECT(item), "activate",
+    g_signal_connect(GTK_OBJECT(item), "activate",
                        GTK_SIGNAL_FUNC(open_set_extension), NULL);
     gtk_container_add(GTK_CONTAINER(menu), item);
     gtk_widget_show(item);
@@ -118,7 +140,7 @@ create_open_menu(void)
 static void
 file_open_ok_callback(GtkWidget *w, GtkFileSelection *fs)
 {
-  char *filename;
+  const char *filename;
   Diagram *diagram = NULL;
   DiaImportFilter *ifilter;
   DDisplay *ddisp;
@@ -140,7 +162,7 @@ file_open_ok_callback(GtkWidget *w, GtkFileSelection *fs)
 }
 
 void
-file_open_callback(GtkWidget *widget, gpointer user_data)
+file_open_callback(gpointer data, guint action, GtkWidget *widget)
 {
   if (!opendlg) {
     DDisplay *ddisp;
@@ -151,19 +173,21 @@ file_open_callback(GtkWidget *widget, gpointer user_data)
       dia = ddisp->diagram;
     }
     opendlg = gtk_file_selection_new(_("Open Diagram"));
-    gtk_window_set_wmclass(GTK_WINDOW(opendlg), "open_diagram", "Dia");
+    gtk_window_set_role(GTK_WINDOW(opendlg), "open_diagram");
     gtk_window_set_position(GTK_WINDOW(opendlg), GTK_WIN_POS_MOUSE);
     gtk_file_selection_set_filename(GTK_FILE_SELECTION(opendlg),
 				    dia && dia->filename ? dia->filename
 				    : "." G_DIR_SEPARATOR_S);
-    gtk_signal_connect_object(
+    g_signal_connect_swapped(
 		GTK_OBJECT(GTK_FILE_SELECTION(opendlg)->cancel_button),
-		"clicked", GTK_SIGNAL_FUNC(file_dialog_hide),
+	    "clicked", G_CALLBACK(file_dialog_hide),
 		GTK_OBJECT(opendlg));
-    gtk_signal_connect(GTK_OBJECT(opendlg), "delete_event",
-		       GTK_SIGNAL_FUNC(file_dialog_hide), NULL);
-    gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(opendlg)->ok_button),
-		       "clicked", GTK_SIGNAL_FUNC(file_open_ok_callback),
+    g_signal_connect(GTK_OBJECT(opendlg), "delete_event",
+		     G_CALLBACK(file_dialog_hide), NULL);
+    g_signal_connect(GTK_OBJECT(opendlg), "destroy",
+		       G_CALLBACK(gtk_widget_destroyed), &opendlg);
+    g_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(opendlg)->ok_button),
+		       "clicked", G_CALLBACK(file_open_ok_callback),
 		       opendlg);
     gtk_quit_add_destroy(1, GTK_OBJECT(opendlg));
   } else {
@@ -205,7 +229,7 @@ file_open_callback(GtkWidget *widget, gpointer user_data)
 static void
 file_save_as_ok_callback(GtkWidget *w, GtkFileSelection *fs)
 {
-  char *filename;
+  const char *filename;
   Diagram *dia;
   struct stat stat_struct;
 
@@ -215,64 +239,39 @@ file_save_as_ok_callback(GtkWidget *w, GtkFileSelection *fs)
 
   if (stat(filename, &stat_struct) == 0) {
     GtkWidget *dialog = NULL;
-    GtkWidget *button, *label;
     char buffer[300];
-    int result;
+    char *utf8filename = NULL;
 
-    dialog = gtk_dialog_new();
-  
-    gtk_signal_connect (GTK_OBJECT (dialog), "destroy", 
-			GTK_SIGNAL_FUNC(gtk_main_quit), NULL);
-    
-    gtk_window_set_title (GTK_WINDOW (dialog), _("File already exists"));
-    gtk_container_set_border_width (GTK_CONTAINER (dialog), 0);
+    if (!g_utf8_validate(filename, -1, NULL)) {
+      utf8filename = g_locale_to_utf8(filename, -1, NULL, NULL, NULL);
+      if (utf8filename == NULL) {
+	message_warning(_("Some characters in the filename are neither UTF-8 nor your local encoding.\nSome things will break."));
+      }
+    }
+    if (utf8filename == NULL) utf8filename = g_strdup(filename);
 
     g_snprintf(buffer, 300,
 	       _("The file '%s' already exists.\n"
-		 "Do you want to overwrite it?"), filename);
-    label = gtk_label_new (buffer);
-    gtk_misc_set_padding (GTK_MISC (label), 10, 10);
-    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), 
-			label, TRUE, TRUE, 0);
-    gtk_widget_show (label);
+		 "Do you want to overwrite it?"), utf8filename);
+    g_free(utf8filename);
 
-    result = FALSE;
-    
-    button = gtk_button_new_with_label (_("Yes"));
-    GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
-    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->action_area), 
-			button, TRUE, TRUE, 0);
-    gtk_widget_grab_default (button);
-    gtk_signal_connect (GTK_OBJECT (button), "clicked",
-			GTK_SIGNAL_FUNC(set_true_callback),
-			&result);
-    gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
-			       GTK_SIGNAL_FUNC (gtk_widget_destroy),
-			       GTK_OBJECT (dialog));
-    gtk_widget_show (button);
-    
-    button = gtk_button_new_with_label (_("No"));
-    GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
-    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->action_area),
-			button, TRUE, TRUE, 0);
-    gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
-			       GTK_SIGNAL_FUNC (gtk_widget_destroy),
-			       GTK_OBJECT (dialog));
-    gtk_widget_show (button);
+    dialog = gtk_message_dialog_new (GTK_WINDOW(fs),
+                                     GTK_DIALOG_MODAL, GTK_MESSAGE_QUESTION,
+                                     GTK_BUTTONS_YES_NO,
+                                     buffer);
+    gtk_window_set_title (GTK_WINDOW (dialog), _("File already exists"));
+    gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_YES);
 
-    gtk_widget_show(dialog);
-
-    gtk_widget_grab_focus(dialog);
-    gtk_grab_add(dialog);
-
-    gtk_main();
-
-    if (result == FALSE) {
+    if (gtk_dialog_run (GTK_DIALOG (dialog)) != GTK_RESPONSE_YES) {
       gtk_widget_hide(savedlg);
+      gtk_widget_destroy(dialog);
       return;
     }
+    gtk_widget_destroy(dialog);
   }
 
+  dia->data->is_compressed = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(compressbutton));
+
   diagram_update_extents(dia);
 
   diagram_set_filename(dia, filename);
@@ -282,7 +281,7 @@ file_save_as_ok_callback(GtkWidget *w, GtkFileSelection *fs)
 }
 
 void
-file_save_as_callback(GtkWidget *widget, gpointer user_data)
+file_save_as_callback(gpointer data, guint action, GtkWidget *widget)
 {
   DDisplay *ddisp;
   Diagram *dia;
@@ -293,42 +292,57 @@ file_save_as_callback(GtkWidget *widget, gpointer user_data)
 
   if (!savedlg) {
     savedlg = gtk_file_selection_new(_("Save Diagram"));
-    gtk_window_set_wmclass(GTK_WINDOW(savedlg), "save_diagram", "Dia");
+    gtk_window_set_role(GTK_WINDOW(savedlg), "save_diagram");
     gtk_window_set_position(GTK_WINDOW(savedlg), GTK_WIN_POS_MOUSE);
-    gtk_file_selection_set_filename(GTK_FILE_SELECTION(savedlg),
-				    dia->filename ? dia->filename
-				    : "." G_DIR_SEPARATOR_S);
-    gtk_signal_connect_object(
+    /* Need better way to make it a reasonable size.  Isn't there some*/
+    /* standard look for them (or is that just Gnome?)*/
+    compressbutton = gtk_check_button_new_with_label(_("Compress diagram files"));
+    gtk_box_pack_start_defaults(GTK_BOX(GTK_FILE_SELECTION(savedlg)->main_vbox),
+				compressbutton);
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(compressbutton),
+				 dia->data->is_compressed);
+    g_signal_connect(G_OBJECT(compressbutton), "toggled",
+		     toggle_compress_callback, NULL);
+    gtk_widget_show(compressbutton);
+    gtk_tooltips_set_tip(tool_tips, compressbutton,
+			 _("Compression reduces file size to less than 1/10th size and speeds up loading and saving.  Some text programs cannot manipulate compressed files."), NULL);
+    g_signal_connect_swapped(
 		GTK_OBJECT(GTK_FILE_SELECTION(savedlg)->cancel_button),
-		"clicked", GTK_SIGNAL_FUNC(file_dialog_hide),
+		"clicked", G_CALLBACK(file_dialog_hide),
 		GTK_OBJECT(savedlg));
-    gtk_signal_connect(GTK_OBJECT(savedlg), "delete_event",
-		       GTK_SIGNAL_FUNC(file_dialog_hide), NULL);
-    gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(savedlg)->ok_button),
-		       "clicked", GTK_SIGNAL_FUNC(file_save_as_ok_callback),
+    g_signal_connect(GTK_OBJECT(savedlg), "delete_event",
+		     G_CALLBACK(file_dialog_hide), NULL);
+    g_signal_connect(GTK_OBJECT(savedlg), "destroy",
+		     G_CALLBACK(gtk_widget_destroyed), &savedlg);
+    g_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(savedlg)->ok_button),
+		     "clicked", G_CALLBACK(file_save_as_ok_callback),
 		       savedlg);
     gtk_quit_add_destroy(1, GTK_OBJECT(savedlg));
   } else {
     gtk_widget_set_sensitive(savedlg, TRUE);
+    g_signal_handlers_block_by_func(G_OBJECT(compressbutton), toggle_compress_callback, NULL);
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(compressbutton),
+				 dia->data->is_compressed);
+    g_signal_handlers_unblock_by_func(G_OBJECT(compressbutton), toggle_compress_callback, NULL);
     if (GTK_WIDGET_VISIBLE(savedlg))
       return;
-    gtk_file_selection_set_filename(GTK_FILE_SELECTION(savedlg),
-				    dia->filename ? dia->filename
-				    : "." G_DIR_SEPARATOR_S);
   }
+  gtk_file_selection_set_filename(GTK_FILE_SELECTION(savedlg),
+				  dia->filename ? dia->filename
+				  : "." G_DIR_SEPARATOR_S);
   gtk_object_set_user_data(GTK_OBJECT(savedlg), dia);
   gtk_widget_show(savedlg);
 }
 
 void
-file_save_callback(GtkWidget *widget, gpointer user_data)
+file_save_callback(gpointer data, guint action, GtkWidget *widget)
 {
   DDisplay *ddisp;
 
   ddisp = ddisplay_active();
 
   if (ddisp->diagram->unsaved) {
-    file_save_as_callback(widget, user_data);
+    file_save_as_callback(data, action, widget);
   } else {
     diagram_update_extents(ddisp->diagram);
     diagram_save(ddisp->diagram, ddisp->diagram->filename);
@@ -340,9 +354,9 @@ export_set_extension(GtkObject *item)
 {
   DiaExportFilter *ef = gtk_object_get_user_data(item);
   GString *s;
-  gchar *text = gtk_entry_get_text(GTK_ENTRY(GTK_FILE_SELECTION(exportdlg)
-					     ->selection_entry));
-  gchar *last_dot = strrchr(text, '.');
+  const gchar *text = gtk_entry_get_text(GTK_ENTRY(GTK_FILE_SELECTION(exportdlg)
+                                                   ->selection_entry));
+  const gchar *last_dot = strrchr(text, '.');
 
   if (!ef || last_dot == text || text[0] == '\0' || ef->extensions[0] == NULL)
     return;
@@ -375,13 +389,16 @@ create_export_menu(void)
   
   for (tmp = filter_get_export_filters(); tmp != NULL; tmp = tmp->next) {
     DiaExportFilter *ef = tmp->data;
+    gchar *filter_label;
 
     if (!ef)
       continue;
-    item = gtk_menu_item_new_with_label(filter_get_export_filter_label(ef));
+    filter_label = filter_get_export_filter_label(ef);
+    item = gtk_menu_item_new_with_label(filter_label);
+    g_free(filter_label);
     gtk_object_set_user_data(GTK_OBJECT(item), ef);
-    gtk_signal_connect(GTK_OBJECT(item), "activate",
-		       GTK_SIGNAL_FUNC(export_set_extension), NULL);
+    g_signal_connect(GTK_OBJECT(item), "activate",
+		     G_CALLBACK(export_set_extension), NULL);
     gtk_container_add(GTK_CONTAINER(menu), item);
     gtk_widget_show(item);
   }
@@ -391,7 +408,7 @@ create_export_menu(void)
 static void
 file_export_ok_callback(GtkWidget *w, GtkFileSelection *fs)
 {
-  char *filename;
+  const char *filename;
   Diagram *dia;
   DiaExportFilter *ef;
   struct stat statbuf;
@@ -403,66 +420,26 @@ file_export_ok_callback(GtkWidget *w, GtkFileSelection *fs)
 
   if (stat(filename, &statbuf) == 0) {
     GtkWidget *dialog = NULL;
-    GtkWidget *button, *label;
     char buffer[300];
     int result;
 
-    dialog = gtk_dialog_new();
-  
-    gtk_signal_connect (GTK_OBJECT (dialog), "destroy", 
-					 GTK_SIGNAL_FUNC(gtk_main_quit), NULL);
-    
-    gtk_window_set_title (GTK_WINDOW (dialog), _("File already exists"));
-    gtk_container_set_border_width (GTK_CONTAINER (dialog), 0);
     g_snprintf(buffer, 300,
 	       _("The file '%s' already exists.\n"
 		 "Do you want to overwrite it?"), filename);
-    label = gtk_label_new (buffer);
-  
-    gtk_misc_set_padding (GTK_MISC (label), 10, 10);
-    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), 
-			label, TRUE, TRUE, 0);
-  
-    gtk_widget_show (label);
+    dialog = gtk_message_dialog_new (GTK_WINDOW(fs),
+                                     GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT, 
+				     GTK_MESSAGE_QUESTION,
+                                     GTK_BUTTONS_YES_NO,
+                                     buffer);
+    gtk_window_set_title (GTK_WINDOW (dialog), _("File already exists"));
+    gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_YES);
 
-    result = FALSE;
-    
-    button = gtk_button_new_with_label (_("Yes"));
-    GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
-    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->action_area), 
-			button, TRUE, TRUE, 0);
-    gtk_widget_grab_default (button);
-    gtk_signal_connect (GTK_OBJECT (button), "clicked",
-			GTK_SIGNAL_FUNC(set_true_callback),
-			&result);
-    gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
-			       GTK_SIGNAL_FUNC (gtk_widget_destroy),
-			       GTK_OBJECT (dialog));
-    gtk_widget_show (button);
-    
-    button = gtk_button_new_with_label (_("No"));
-    GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
-    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->action_area),
-			button, TRUE, TRUE, 0);
-    
-    gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
-			       GTK_SIGNAL_FUNC (gtk_widget_destroy),
-			       GTK_OBJECT (dialog));
-    
-    gtk_widget_show (button);
-
-    gtk_widget_show (dialog);
-
-    /* Make dialog modal: */
-    gtk_widget_grab_focus(dialog);
-    gtk_grab_add(dialog);
-
-    gtk_main();
-
-    if (result==FALSE) {
+    if (gtk_dialog_run (GTK_DIALOG (dialog)) != GTK_RESPONSE_YES) {
       gtk_widget_hide(exportdlg);
+      gtk_widget_destroy(dialog);
       return;
     }
+    gtk_widget_destroy(dialog);
   }
 
   ef = gtk_object_get_user_data(GTK_OBJECT(GTK_OPTION_MENU(export_omenu)
@@ -470,7 +447,7 @@ file_export_ok_callback(GtkWidget *w, GtkFileSelection *fs)
   if (!ef)
     ef = filter_guess_export_filter(filename);
   if (ef)
-      ef->export(dia->data, filename, dia->filename);
+      ef->export(dia->data, filename, dia->filename, ef->user_data);
   else
       message_error(_("Could not determine which export filter\n"
 		      "to use to save '%s'"), filename);
@@ -479,10 +456,11 @@ file_export_ok_callback(GtkWidget *w, GtkFileSelection *fs)
 }
 
 void
-file_export_callback(GtkWidget *widget, gpointer user_data)
+file_export_callback(gpointer data, guint action, GtkWidget *widget)
 {
   DDisplay *ddisp;
   Diagram *dia;
+  GtkWidget *export_menu, *export_item;
 
   ddisp = ddisplay_active();
   if (!ddisp) return;
@@ -490,28 +468,23 @@ file_export_callback(GtkWidget *widget, gpointer user_data)
 
   if (!exportdlg) {
     exportdlg = gtk_file_selection_new(_("Export Diagram"));
-    gtk_window_set_wmclass(GTK_WINDOW(exportdlg), "export_diagram", "Dia");
+    gtk_window_set_role(GTK_WINDOW(exportdlg), "export_diagram");
     gtk_window_set_position(GTK_WINDOW(exportdlg), GTK_WIN_POS_MOUSE);
     gtk_file_selection_set_filename(GTK_FILE_SELECTION(exportdlg),
 				    dia->filename ? dia->filename
 				    : "." G_DIR_SEPARATOR_S);
-    gtk_signal_connect_object(
+    g_signal_connect_swapped(
 		GTK_OBJECT(GTK_FILE_SELECTION(exportdlg)->cancel_button),
-		"clicked", GTK_SIGNAL_FUNC(file_dialog_hide),
+	    "clicked", G_CALLBACK(file_dialog_hide),
 		GTK_OBJECT(exportdlg));
-    gtk_signal_connect(GTK_OBJECT(exportdlg), "delete_event",
-		       GTK_SIGNAL_FUNC(file_dialog_hide), NULL);
-    gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(exportdlg)->ok_button),
-		       "clicked", GTK_SIGNAL_FUNC(file_export_ok_callback),
+    g_signal_connect(GTK_OBJECT(exportdlg), "delete_event",
+		     G_CALLBACK(file_dialog_hide), NULL);
+    g_signal_connect(GTK_OBJECT(exportdlg), "destroy",
+		     G_CALLBACK(gtk_widget_destroyed), &exportdlg);
+    g_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(exportdlg)->ok_button),
+		     "clicked", G_CALLBACK(file_export_ok_callback),
 		       exportdlg);
     gtk_quit_add_destroy(1, GTK_OBJECT(exportdlg));
-  } else {
-    gtk_widget_set_sensitive(exportdlg, TRUE);
-    if (GTK_WIDGET_VISIBLE(exportdlg))
-      return;
-    gtk_file_selection_set_filename(GTK_FILE_SELECTION(exportdlg),
-				    dia->filename ? dia->filename
-				    : "." G_DIR_SEPARATOR_S);
   }
   if (!export_options) {
     GtkWidget *hbox, *label, *omenu;
@@ -537,7 +510,18 @@ file_export_callback(GtkWidget *widget, gpointer user_data)
     gtk_box_pack_end(GTK_BOX (GTK_FILE_SELECTION(exportdlg)->main_vbox),
 		      export_options, FALSE, FALSE, 5);
   }
+ 
   gtk_object_set_user_data(GTK_OBJECT(exportdlg), dia);
+  gtk_widget_set_sensitive(exportdlg, TRUE);
+  if (GTK_WIDGET_VISIBLE(exportdlg))
+    return;
+  gtk_file_selection_set_filename(GTK_FILE_SELECTION(exportdlg),
+				  dia->filename ? dia->filename
+				  : "." G_DIR_SEPARATOR_S);
+  export_menu = gtk_option_menu_get_menu(GTK_OPTION_MENU(export_omenu));
+  export_item = gtk_menu_get_active(GTK_MENU(export_menu));
+  if (export_item)
+    export_set_extension(GTK_OBJECT(export_item));
   gtk_widget_show(export_options);
   gtk_widget_show(exportdlg);
 }
