@@ -24,9 +24,9 @@
 
 #include <gtk/gtk.h>
 
-void file_open_callback(GtkWidget *widget, gpointer user_data);
-void file_save_callback(GtkWidget *widget, gpointer user_data);
-void file_save_as_callback(GtkWidget *widget, gpointer user_data);
-void file_export_callback(GtkWidget *widget, gpointer user_data);
+void file_open_callback(gpointer data, guint action, GtkWidget *widget);
+void file_save_callback(gpointer data, guint action, GtkWidget *widget);
+void file_save_as_callback(gpointer data, guint action, GtkWidget *widget);
+void file_export_callback(gpointer data, guint action, GtkWidget *widget);
 
 #endif
