@@ -16,201 +16,97 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+#include <config.h>
+
 #include <math.h>
+#include <string.h> /* strlen */
 #include <gdk/gdk.h>
 
 #include "render_libart.h"
-#include "message.h"
 
 #ifdef HAVE_LIBART
 
-#include <libart_lgpl/art_point.h>
-#include <libart_lgpl/art_misc.h>
-#include <libart_lgpl/art_affine.h>
-#include <libart_lgpl/art_svp_vpath.h>
-#include <libart_lgpl/art_bpath.h>
-#include <libart_lgpl/art_vpath_bpath.h>
+#include "dialibartrenderer.h"
 #include <libart_lgpl/art_rgb.h>
-#include <libart_lgpl/art_rgb_svp.h>
-#include <libart_lgpl/art_rgb_affine.h>
-#include "libart_lgpl/art_rgb_bitmap_affine.h"
-#include <libart_lgpl/art_filterlevel.h>
+#include "font.h"
+#include "color.h"
 
-static void begin_render(RendererLibart *renderer, DiagramData *data);
-static void end_render(RendererLibart *renderer);
-static void set_linewidth(RendererLibart *renderer, real linewidth);
-static void set_linecaps(RendererLibart *renderer, LineCaps mode);
-static void set_linejoin(RendererLibart *renderer, LineJoin mode);
-static void set_linestyle(RendererLibart *renderer, LineStyle mode);
-static void set_dashlength(RendererLibart *renderer, real length);
-static void set_fillstyle(RendererLibart *renderer, FillStyle mode);
-static void set_font(RendererLibart *renderer, Font *font, real height);
-static void draw_line(RendererLibart *renderer, 
-		      Point *start, Point *end, 
-		      Color *line_color);
-static void draw_polyline(RendererLibart *renderer, 
-			  Point *points, int num_points, 
-			  Color *line_color);
-static void draw_polygon(RendererLibart *renderer, 
-			 Point *points, int num_points, 
-			 Color *line_color);
-static void fill_polygon(RendererLibart *renderer, 
-			 Point *points, int num_points, 
-			 Color *line_color);
-static void draw_rect(RendererLibart *renderer, 
-		      Point *ul_corner, Point *lr_corner,
-		      Color *color);
-static void fill_rect(RendererLibart *renderer, 
-		      Point *ul_corner, Point *lr_corner,
-		      Color *color);
-static void draw_arc(RendererLibart *renderer, 
-		     Point *center,
-		     real width, real height,
-		     real angle1, real angle2,
-		     Color *color);
-static void fill_arc(RendererLibart *renderer, 
-		     Point *center,
-		     real width, real height,
-		     real angle1, real angle2,
-		     Color *color);
-static void draw_ellipse(RendererLibart *renderer, 
-			 Point *center,
-			 real width, real height,
-			 Color *color);
-static void fill_ellipse(RendererLibart *renderer, 
-			 Point *center,
-			 real width, real height,
-			 Color *color);
-static void draw_bezier(RendererLibart *renderer, 
-			BezPoint *points,
-			int numpoints,
-			Color *color);
-static void fill_bezier(RendererLibart *renderer, 
-			BezPoint *points, /* Last point must be same as first point */
-			int numpoints,
-			Color *color);
-static void draw_string(RendererLibart *renderer,
-			const char *text,
-			Point *pos, Alignment alignment,
-			Color *color);
-static void draw_image(RendererLibart *renderer,
-		       Point *point,
-		       real width, real height,
-		       DiaImage image);
-
-static real get_text_width(RendererLibart *renderer,
+static real get_text_width(DiaRenderer *self,
 			   const char *text, int length);
 
-static void clip_region_clear(RendererLibart *renderer);
-static void clip_region_add_rect(RendererLibart *renderer,
+static void clip_region_clear(DiaRenderer *self);
+static void clip_region_add_rect(DiaRenderer *self,
 				 Rectangle *rect);
 
-static void draw_pixel_line(RendererLibart *renderer,
+static void draw_pixel_line(DiaRenderer *self,
 			    int x1, int y1,
 			    int x2, int y2,
 			    Color *color);
-static void draw_pixel_rect(RendererLibart *renderer,
+static void draw_pixel_rect(DiaRenderer *self,
 				 int x, int y,
 				 int width, int height,
 				 Color *color);
-static void fill_pixel_rect(RendererLibart *renderer,
+static void fill_pixel_rect(DiaRenderer *self,
 				 int x, int y,
 				 int width, int height,
 				 Color *color);
+static void set_size(DiaRenderer *self, GdkWindow *window,
+                     int width, int height);
+static void copy_to_window (DiaRenderer *self, GdkWindow *window,
+                            int x, int y, int width, int height);
 
-static RenderOps LibartRenderOps = {
-  (BeginRenderFunc) begin_render,
-  (EndRenderFunc) end_render,
 
-  (SetLineWidthFunc) set_linewidth,
-  (SetLineCapsFunc) set_linecaps,
-  (SetLineJoinFunc) set_linejoin,
-  (SetLineStyleFunc) set_linestyle,
-  (SetDashLengthFunc) set_dashlength,
-  (SetFillStyleFunc) set_fillstyle,
-  (SetFontFunc) set_font,
-  
-  (DrawLineFunc) draw_line,
-  (DrawPolyLineFunc) draw_polyline,
-  
-  (DrawPolygonFunc) draw_polygon,
-  (FillPolygonFunc) fill_polygon,
-
-  (DrawRectangleFunc) draw_rect,
-  (FillRectangleFunc) fill_rect,
-
-  (DrawArcFunc) draw_arc,
-  (FillArcFunc) fill_arc,
-
-  (DrawEllipseFunc) draw_ellipse,
-  (FillEllipseFunc) fill_ellipse,
-
-  (DrawBezierFunc) draw_bezier,
-  (FillBezierFunc) fill_bezier,
-
-  (DrawStringFunc) draw_string,
-
-  (DrawImageFunc) draw_image,
-};
-
-static InteractiveRenderOps LibartInteractiveRenderOps = {
-  (GetTextWidthFunc) get_text_width,
-
-  (ClipRegionClearFunc) clip_region_clear,
-  (ClipRegionAddRectangleFunc) clip_region_add_rect,
-
-  (DrawPixelLineFunc) draw_pixel_line,
-  (DrawPixelRectangleFunc) draw_pixel_rect,
-  (FillPixelRectangleFunc) fill_pixel_rect,
-};
-
-RendererLibart *
-new_libart_renderer(DDisplay *ddisp, int interactive)
+static void 
+dia_libart_renderer_iface_init (DiaInteractiveRendererInterface* iface)
 {
-  RendererLibart *renderer;
-
-  renderer = g_new(RendererLibart, 1);
-  renderer->renderer.ops = &LibartRenderOps;
-  renderer->renderer.is_interactive = interactive;
-  renderer->renderer.interactive_ops = &LibartInteractiveRenderOps;
-  renderer->ddisp = ddisp;
-
-  renderer->rgb_buffer = NULL;
-  renderer->renderer.pixel_width = 0;
-  renderer->renderer.pixel_height = 0;
-
-  renderer->line_width = 1.0;
-  renderer->cap_style = GDK_CAP_BUTT;
-  renderer->join_style = GDK_JOIN_MITER;
-  
-  renderer->saved_line_style = LINESTYLE_SOLID;
-  renderer->dash_enabled = 0;
-  renderer->dash_length = 10;
-  renderer->dot_length = 1;
-
-  renderer->gdk_font = NULL;
-  renderer->suck_font = NULL;
-
-  return renderer;
+  iface->clip_region_clear = clip_region_clear;
+  iface->clip_region_add_rect = clip_region_add_rect;
+  iface->draw_pixel_line = draw_pixel_line;
+  iface->draw_pixel_rect = draw_pixel_rect;
+  iface->fill_pixel_rect = fill_pixel_rect;
+  iface->copy_to_window = copy_to_window;
+  iface->set_size = set_size;
 }
 
-void
-destroy_libart_renderer(RendererLibart *renderer)
-{
-  if (renderer->rgb_buffer != NULL)
-    g_free(renderer->rgb_buffer);
 
-  g_free(renderer);
+DiaRenderer *
+new_libart_renderer(DiaTransform *trans, int interactive)
+{
+  DiaLibartRenderer *renderer;
+  GType renderer_type = 0;
+
+  renderer = g_object_new(DIA_TYPE_LIBART_RENDERER, NULL);
+  renderer->transform = trans;
+
+  if (!DIA_GET_INTERACTIVE_RENDERER_INTERFACE (renderer))
+    {
+      static const GInterfaceInfo irenderer_iface_info = 
+      {
+        (GInterfaceInitFunc) dia_libart_renderer_iface_init,
+        NULL,           /* iface_finalize */
+        NULL            /* iface_data     */
+      };
+
+      renderer_type = DIA_TYPE_LIBART_RENDERER;
+      /* register the interactive renderer interface */
+      g_type_add_interface_static (renderer_type,
+                                   DIA_TYPE_INTERACTIVE_RENDERER_INTERFACE,
+                                   &irenderer_iface_info);
+    }
+  renderer->parent_instance.is_interactive = interactive;
+
+  return DIA_RENDERER (renderer);
 }
 
-void
-libart_renderer_set_size(RendererLibart *renderer, GdkWindow *window,
-			 int width, int height)
+static void
+set_size(DiaRenderer *self, GdkWindow *window,
+         int width, int height)
 {
+  DiaLibartRenderer *renderer = DIA_LIBART_RENDERER (self);
   int i;
   
-  if ( (renderer->renderer.pixel_width==width) &&
-       (renderer->renderer.pixel_height==height) )
+  if ( (renderer->pixel_width==width) &&
+       (renderer->pixel_height==height) )
     return;
   
   if (renderer->rgb_buffer != NULL) {
@@ -220,22 +116,23 @@ libart_renderer_set_size(RendererLibart *renderer, GdkWindow *window,
   renderer->rgb_buffer = g_new (guint8, width * height * 3);
   for (i=0;i<width * height * 3;i++)
     renderer->rgb_buffer[i] = 0xff;
-  renderer->renderer.pixel_width = width;
-  renderer->renderer.pixel_height = height;
+  renderer->pixel_width = width;
+  renderer->pixel_height = height;
 }
 
-extern void
-renderer_libart_copy_to_window(RendererLibart *renderer, GdkWindow *window,
-			       int x, int y, int width, int height)
+static void
+copy_to_window (DiaRenderer *self, GdkWindow *window,
+                int x, int y, int width, int height)
 {
   static GdkGC *copy_gc = NULL;
+  DiaLibartRenderer *renderer = DIA_LIBART_RENDERER (self);
   int w;
 
   if (copy_gc == NULL) {
     copy_gc = gdk_gc_new(window);
   }
 
-  w = renderer->renderer.pixel_width;
+  w = renderer->pixel_width;
   
   gdk_draw_rgb_image(window,
 		     copy_gc,
@@ -246,6 +143,19 @@ renderer_libart_copy_to_window(RendererLibart *renderer, GdkWindow *window,
 		     w*3);
 }
 
+#if 0
+static guint32 color_to_abgr(Color *col)
+{
+  int rgba;
+
+  rgba = 0x0;
+  rgba |= (guint)(0xFF*col->blue) << 16;
+  rgba |= (guint)(0xFF*col->green) << 8;
+  rgba |= (guint)(0xFF*col->red);
+  
+  return rgba;
+}
+
 static guint32 color_to_rgba(Color *col)
 {
   int rgba;
@@ -261,11 +171,17 @@ static guint32 color_to_rgba(Color *col)
 static void
 begin_render(RendererLibart *renderer, DiagramData *data)
 {
+#ifdef HAVE_FREETYPE
+  renderer->old_context = dia_font_init(pango_ft2_get_context(75, 75));
+#endif
 }
 
 static void
 end_render(RendererLibart *renderer)
 {
+#ifdef HAVE_FREETYPE
+  dia_font_init(renderer->old_context);
+#endif
 }
 
 
@@ -399,18 +315,18 @@ set_fillstyle(RendererLibart *renderer, FillStyle mode)
   case FILLSTYLE_SOLID:
     break;
   default:
-    message_error("gdk_renderer: Unsupported fill mode specified!\n");
+    message_error(_("gdk_renderer: Unsupported fill mode specified!\n"));
   }
 }
 
 static void
-set_font(RendererLibart *renderer, Font *font, real height)
+set_font(RendererLibart *renderer, DiaFont *font, real height)
 {
-  renderer->font_height =
-    ddisplay_transform_length(renderer->ddisp, height);
-
-  renderer->gdk_font = font_get_gdkfont(font, renderer->font_height);
-  renderer->suck_font = font_get_suckfont(font, renderer->font_height);
+  renderer->font_height = height;
+  
+  if (renderer->font)
+    dia_font_unref(renderer->font);
+  renderer->font = dia_font_ref(font);
 }
 
 static void
@@ -1107,68 +1023,157 @@ fill_bezier(RendererLibart *renderer,
   art_svp_free( svp );
 }
 
+
 static void
-draw_string(RendererLibart *renderer,
-	    const char *text,
-	    Point *pos, Alignment alignment,
-	    Color *color)
+draw_string (RendererLibart *renderer,
+	     const gchar *text,
+	     Point *pos, Alignment alignment,
+	     Color *color)
 {
+  /* Not working with Pango */
   DDisplay *ddisp = renderer->ddisp;
-  int x, y, i, dx;
+  guint8 *bitmap = NULL;
+  double x,y;
+  Point start_pos;
+  PangoLayout* layout;
+  int width, height;
+  int i, j;
   int iwidth;
   int len;
-  double affine[6];
-  SuckFont *suckfont;
+  double affine[6], tmpaffine[6];
   double xpos, ypos;
   guint32 rgba;
+  int rowstride;
+  double old_zoom;
+  double font_height;
+  GdkPixmap *pixmap;
+  GdkGC *gc;
 
-  ddisplay_transform_coords(ddisp, pos->x, pos->y,
-			    &x, &y);
-  
-  iwidth = gdk_string_width(renderer->gdk_font, text);
+  double fudge = 10.0;
+
+  point_copy(&start_pos,pos);
+
+  /* Something's screwed up with the zooming for fonts.
+     It's like it zooms double.  Yet removing some of the zooms
+     don't seem to work, either.
+  */
+  /*
+  old_zoom = ddisp->zoom_factor;
+  ddisp->zoom_factor = ddisp->zoom_factor;
+  */
   switch (alignment) {
   case ALIGN_LEFT:
     break;
   case ALIGN_CENTER:
-    x -= iwidth/2;
+    start_pos.x -= dia_font_scaled_string_width(text, renderer->font,
+						renderer->font_height,
+						ddisp->zoom_factor)/2;
     break;
   case ALIGN_RIGHT:
-    x -= iwidth;
+    start_pos.x -= dia_font_scaled_string_width(text, renderer->font,
+						renderer->font_height,
+						ddisp->zoom_factor);
     break;
   }
+ 
+  //  font_height = ddisplay_transform_length(ddisp, renderer->font_height);
 
-  suckfont = renderer->suck_font;
+  start_pos.y -= dia_font_ascent(text, renderer->font, renderer->font_height);
+  ddisplay_transform_coords_double(ddisp, start_pos.x, start_pos.y, &x, &y);
 
-  xpos = (double) x + 1; 
-  ypos = (double) (y - suckfont->ascent);
-  
-  rgba = color_to_rgba(color);
-  
-  len = strlen(text);
-  for (i = 0; i < len; i++) {
-    SuckChar *ch;
-    
-    ch = &suckfont->chars[(unsigned char)text[i]];
-    art_affine_translate(affine, xpos, ypos);
-    art_rgb_bitmap_affine(renderer->rgb_buffer,
-			  0, 0,
-			  renderer->renderer.pixel_width,
-			  renderer->renderer.pixel_height,
-			  renderer->renderer.pixel_width*3,
-			   suckfont->bitmap + (ch->bitmap_offset >> 3),
-			   ch->width,
-			   suckfont->bitmap_height,
-			   suckfont->bitmap_width >> 3,
-			   rgba,
-			   affine,
-			   ART_FILTER_NEAREST, NULL);
-    
-    dx = ch->left_sb + ch->width + ch->right_sb;
-    xpos += dx;
+  layout = dia_font_scaled_build_layout(text, renderer->font, 
+					renderer->font_height,
+					ddisp->zoom_factor);
+  /*
+  ddisp->zoom_factor = old_zoom;
+  */
+  pango_layout_get_pixel_size(layout, &width, &height);
+  /* Pango doesn't have a 'render to raw bits' function, so we have
+   * to render based on what other engines are available.
+   */
+#define DEPTH 4
+#ifdef HAVE_FREETYPE
+  /* Freetype version */
+ {
+   FT_Bitmap ftbitmap;
+   guint8 *graybitmap;
+   rowstride = 32*((width+31)/31);
+   
+   graybitmap = (guint8*)g_new0(guint8, height*rowstride);
+
+   ftbitmap.rows = height;
+   ftbitmap.width = width;
+   ftbitmap.pitch = rowstride;
+   ftbitmap.buffer = graybitmap;
+   ftbitmap.num_grays = 256;
+   ftbitmap.pixel_mode = ft_pixel_mode_grays;
+   ftbitmap.palette_mode = 0;
+   ftbitmap.palette = 0;
+   printf("Rendering %s onto bitmap of size %dx%d row %d\n", text, width, height, rowstride);
+   pango_ft2_render_layout(&ftbitmap, layout, 0, 0);
+   bitmap = (guint8*)g_new0(guint8, height*rowstride*DEPTH);
+   for (i = 0; i < height; i++) {
+     for (j = 0; j < width; j++) {
+       bitmap[DEPTH*(i*rowstride+j)] = color->red;
+       bitmap[DEPTH*(i*rowstride+j)+1] = color->green;
+       bitmap[DEPTH*(i*rowstride+j)+2] = color->blue;
+       bitmap[DEPTH*(i*rowstride+j)+3] = graybitmap[i*rowstride+j];
+     }
+     //     bitmap[4*i+3] = graybitmap[i];
+   }
+   g_free(graybitmap);
+ }
+#else
+  /* Gdk version -- but we shouldn't need Gdk, dammit! */
+  /* IMHO using Gdk for text rendering is appreciated,
+   * at least if we use it every where else too, as 
+   * is on win32 ...     --hb
+   */
+#if 0
+  {
+    static gboolean warned = FALSE;
+
+    if (!warned) {
+      g_warning ("No AA text rendering yet ..");
+      warned = TRUE;
+    }
+    g_object_unref(G_OBJECT(layout));
+    return;
   }
+#endif
+#define DEPTH 4
+  bitmap = (guint8 *)g_new0(guint8, height*rowstride*DEPTH);
+  memset (bitmap, 0xAA, height*rowstride*DEPTH);
+  pixmap = gdk_pixmap_create_from_data (NULL, bitmap, 
+				     rowstride, height, DEPTH*8,
+				     &color_black, &color_white);
+  gdk_draw_layout (pixmap, gc, 0, 0, layout);
+#endif
+  /* abuse_layout_object(layout,text); */
+  
+  g_object_unref(G_OBJECT(layout));
 
+  rgba = color_to_rgba(color);
+
+  art_affine_identity(affine);
+  art_affine_translate(tmpaffine, x, y);
+  art_affine_multiply(affine, affine, tmpaffine);
   
-  
+  if (bitmap != NULL)
+    art_rgb_rgba_affine (renderer->rgb_buffer,
+		    0, 0,
+		    renderer->renderer.pixel_width,
+		    renderer->renderer.pixel_height,
+		    renderer->renderer.pixel_width * 3,
+		    bitmap,
+		    width,
+		    height,
+		    rowstride*DEPTH,
+		    //rgba,
+		    affine,
+		    ART_FILTER_NEAREST, NULL);
+
+  g_free(bitmap);
 }
 
 static void
@@ -1236,21 +1241,27 @@ draw_image(RendererLibart *renderer,
       real_width, real_height);*/
 }
 
+#endif
 static real
-get_text_width(RendererLibart *renderer,
+get_text_width(DiaRenderer *self,
 	       const char *text, int length)
 {
+  DiaLibartRenderer *renderer = DIA_LIBART_RENDERER (self);
   int iwidth;
   
-  iwidth = gdk_text_width(renderer->gdk_font, text, length);
+  iwidth = dia_font_string_width(text,
+				 renderer->parent_instance.font, 
+				 renderer->parent_instance.font_height);
 
-  return ddisplay_untransform_length(renderer->ddisp, (real) iwidth);
+  return dia_untransform_length(renderer->transform, (real) iwidth);
 }
 
 
 static void
-clip_region_clear(RendererLibart *renderer)
+clip_region_clear(DiaRenderer *self)
 {
+  DiaLibartRenderer *renderer = DIA_LIBART_RENDERER (self);
+
   renderer->clip_rect_empty = 1;
   renderer->clip_rect.top = 0;
   renderer->clip_rect.bottom = 0;
@@ -1259,25 +1270,25 @@ clip_region_clear(RendererLibart *renderer)
 }
 
 static void
-clip_region_add_rect(RendererLibart *renderer,
+clip_region_add_rect(DiaRenderer *self,
 		     Rectangle *rect)
 {
-  DDisplay *ddisp = renderer->ddisp;
+  DiaLibartRenderer *renderer = DIA_LIBART_RENDERER (self);
   int x1,y1;
   int x2,y2;
   IntRectangle r;
 
-  ddisplay_transform_coords(ddisp, rect->left, rect->top,  &x1, &y1);
-  ddisplay_transform_coords(ddisp, rect->right, rect->bottom,  &x2, &y2);
+  dia_transform_coords(renderer->transform, rect->left, rect->top,  &x1, &y1);
+  dia_transform_coords(renderer->transform, rect->right, rect->bottom,  &x2, &y2);
 
   if (x1 < 0)
     x1 = 0;
   if (y1 < 0)
     y1 = 0;
-  if (x2 >= renderer->renderer.pixel_width)
-    x2 = renderer->renderer.pixel_width - 1;
-  if (y2 >= renderer->renderer.pixel_height)
-    y2 = renderer->renderer.pixel_height - 1;
+  if (x2 >= renderer->pixel_width)
+    x2 = renderer->pixel_width - 1;
+  if (y2 >= renderer->pixel_height)
+    y2 = renderer->pixel_height - 1;
   
   r.top = y1;
   r.bottom = y2;
@@ -1319,14 +1330,15 @@ clip_region_add_rect(RendererLibart *renderer,
 
 /* Does no clipping! */
 static void
-draw_hline(RendererLibart *renderer,
+draw_hline(DiaRenderer *self,
 	   int x, int y, int length,
 	   guint8 r, guint8 g, guint8 b)
 {
+  DiaLibartRenderer *renderer = DIA_LIBART_RENDERER (self);
   int stride;
   guint8 *ptr;
 
-  stride = renderer->renderer.pixel_width*3;
+  stride = renderer->pixel_width*3;
   ptr = renderer->rgb_buffer + x*3 + y*stride;
   if (length>=0)
     art_rgb_fill_run(ptr, r, g, b, length+1);
@@ -1334,14 +1346,15 @@ draw_hline(RendererLibart *renderer,
 
 /* Does no clipping! */
 static void
-draw_vline(RendererLibart *renderer,
+draw_vline(DiaRenderer *self,
 	   int x, int y, int height,
 	   guint8 r, guint8 g, guint8 b)
 {
+  DiaLibartRenderer *renderer = DIA_LIBART_RENDERER (self);
   int stride;
   guint8 *ptr;
 
-  stride = renderer->renderer.pixel_width*3;
+  stride = renderer->pixel_width*3;
   ptr = renderer->rgb_buffer + x*3 + y*stride;
   height+=y;
   while (y<=height) {
@@ -1354,11 +1367,12 @@ draw_vline(RendererLibart *renderer,
 }
 
 static void
-draw_pixel_line(RendererLibart *renderer,
+draw_pixel_line(DiaRenderer *self,
 		int x1, int y1,
 		int x2, int y2,
 		Color *color)
 {
+  DiaLibartRenderer *renderer = DIA_LIBART_RENDERER (self);
   guint8 r,g,b;
   guint8 *ptr;
   int start, len;
@@ -1384,7 +1398,7 @@ draw_pixel_line(RendererLibart *renderer,
     /* top line */
     if ( (y1>=renderer->clip_rect.top) &&
 	 (y1<=renderer->clip_rect.bottom) ) {
-      draw_hline(renderer, start, y1, len, r, g, b);
+      draw_hline(self, start, y1, len, r, g, b);
     }
     return;
   }
@@ -1397,7 +1411,7 @@ draw_pixel_line(RendererLibart *renderer,
     /* left line */
     if ( (x1>=renderer->clip_rect.left) &&
 	 (x1<=renderer->clip_rect.right) ) {
-      draw_vline(renderer, x1, start, len, r, g, b);
+      draw_vline(self, x1, start, len, r, g, b);
     }
     return;
   }
@@ -1407,7 +1421,7 @@ draw_pixel_line(RendererLibart *renderer,
    * It is also not very well tested.
    */
   
-  stride = renderer->renderer.pixel_width*3;
+  stride = renderer->pixel_width*3;
   clip_rect = &renderer->clip_rect;
   
   dx = x2-x1;
@@ -1504,11 +1518,12 @@ draw_pixel_line(RendererLibart *renderer,
 }
 
 static void
-draw_pixel_rect(RendererLibart *renderer,
+draw_pixel_rect(DiaRenderer *self,
 		int x, int y,
 		int width, int height,
 		Color *color)
 {
+  DiaLibartRenderer *renderer = DIA_LIBART_RENDERER (self);
   guint8 r,g,b;
   int start, len;
   int stride;
@@ -1517,7 +1532,7 @@ draw_pixel_rect(RendererLibart *renderer,
   g = color->green*0xff;
   b = color->blue*0xff;
 
-  stride = renderer->renderer.pixel_width*3;
+  stride = renderer->pixel_width*3;
   
   /* clip in x */
   start = x;
@@ -1527,13 +1542,13 @@ draw_pixel_rect(RendererLibart *renderer,
   /* top line */
   if ( (y>=renderer->clip_rect.top) &&
        (y<=renderer->clip_rect.bottom) ) {
-    draw_hline(renderer, start, y, len, r, g, b);
+    draw_hline(self, start, y, len, r, g, b);
   }
 
   /* bottom line */
   if ( (y+height>=renderer->clip_rect.top) &&
        (y+height<=renderer->clip_rect.bottom) ) {
-    draw_hline(renderer, start, y+height, len, r, g, b);
+    draw_hline(self, start, y+height, len, r, g, b);
   }
 
   /* clip in y */
@@ -1544,22 +1559,23 @@ draw_pixel_rect(RendererLibart *renderer,
   /* left line */
   if ( (x>=renderer->clip_rect.left) &&
        (x<renderer->clip_rect.right) ) {
-    draw_vline(renderer, x, start, len, r, g, b);
+    draw_vline(self, x, start, len, r, g, b);
   }
 
   /* right line */
   if ( (x+width>=renderer->clip_rect.left) &&
        (x+width<renderer->clip_rect.right) ) {
-    draw_vline(renderer, x+width, start, len, r, g, b);
+    draw_vline(self, x+width, start, len, r, g, b);
   }
 }
 
 static void
-fill_pixel_rect(RendererLibart *renderer,
+fill_pixel_rect(DiaRenderer *self,
 		int x, int y,
 		int width, int height,
 		Color *color)
 {
+  DiaLibartRenderer *renderer = DIA_LIBART_RENDERER (self);
   guint8 r,g,b;
   guint8 *ptr;
   int i;
@@ -1578,7 +1594,7 @@ fill_pixel_rect(RendererLibart *renderer,
   g = color->green*0xff;
   b = color->blue*0xff;
   
-  stride = renderer->renderer.pixel_width*3;
+  stride = renderer->pixel_width*3;
   ptr = renderer->rgb_buffer + x*3 + y*stride;
   for (i=0;i<=height;i++) {
     art_rgb_fill_run(ptr, r, g, b, width+1);
@@ -1588,27 +1604,10 @@ fill_pixel_rect(RendererLibart *renderer,
 
 #else
 
-RendererLibart *
-new_libart_renderer(DDisplay *ddisp, int interactive)
+DiaRenderer *
+new_libart_renderer(DiaTransform *transform, int interactive)
 {
   return NULL;
 }
 
-void
-destroy_libart_renderer(RendererLibart *renderer)
-{
-}
-
-void
-libart_renderer_set_size(RendererLibart *renderer, GdkWindow *window,
-			 int width, int height)
-{
-}
-
-extern void
-renderer_libart_copy_to_window(RendererLibart *renderer, GdkWindow *window,
-			       int x, int y, int width, int height)
-{
-}
-
 #endif
