@@ -15,15 +15,19 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
+#include <config.h>
+
 #include <stdio.h>
+#include <string.h> /* memcpy() */
 #include <assert.h>
 
 #include "connection.h"
 #include "message.h"
 
-void
+ObjectChange*
 connection_move_handle(Connection *conn, HandleId id,
-		       Point *to, HandleMoveReason reason)
+		       Point *to, ConnectionPoint *cp,
+		       HandleMoveReason reason, ModifierKeys modifiers)
 {
   switch(id) {
   case HANDLE_MOVE_STARTPOINT:
@@ -36,6 +40,7 @@ connection_move_handle(Connection *conn, HandleId id,
     message_error("Internal error in connection_move_handle.\n");
     break;
   }
+  return NULL;
 }
 
 void
@@ -51,28 +56,10 @@ connection_update_handles(Connection *conn)
 void
 connection_update_boundingbox(Connection *conn)
 {
-  Rectangle *bb;
-  Point *endpoints;
-  
   assert(conn != NULL);
-
-  bb = &conn->object.bounding_box;
-  endpoints = &conn->endpoints[0];
   
-  bb->left = endpoints[0].x;
-  bb->right = endpoints[1].x;
-  if (bb->left > bb->right) {
-    coord temp = bb->left;
-    bb->left = bb->right;
-    bb->right = temp;
-  }
-  bb->top = endpoints[0].y;
-  bb->bottom = endpoints[1].y;
-  if (bb->top > bb->bottom) {
-    coord temp = bb->top;
-    bb->top = bb->bottom;
-    bb->bottom = temp;
-  }
+  line_bbox(&conn->endpoints[0],&conn->endpoints[1],
+            &conn->extra_spacing,&conn->object.bounding_box);
 }
 
 /* Needs to have at least 2 handles 
@@ -122,6 +109,7 @@ connection_copy(Connection *from, Connection *to)
     to->endpoint_handles[i].connected_to = NULL;
     toobj->handles[i] = &to->endpoint_handles[i];
   }
+  memcpy(&to->extra_spacing,&from->extra_spacing,sizeof(to->extra_spacing));
 }
 
 void
