@@ -1,4 +1,4 @@
-/* Dia -- an diagram creation/manipulation program
+/* dia -- an diagram creation/manipulation program
  * Copyright (C) 1998 Alexander Larsson
  *
  * This program is free software; you can redistribute it and/or modify
@@ -18,17 +18,16 @@
 #ifndef DIA_XML_H
 #define DIA_XML_H
 
-#include "config.h"
 #include <glib.h>
 #include "geometry.h"
 #include "color.h"
 #include "font.h"
+#include "diavar.h"
+#include <libxml/tree.h>
+
+DIAVAR int pretty_formated_xml;
 
-#ifdef __XML_TREE_H__
 typedef xmlNodePtr XML_NODE;
-#else
-typedef void * XML_NODE;
-#endif
 
 typedef XML_NODE ObjectNode;
 typedef XML_NODE AttributeNode;
@@ -63,7 +62,8 @@ void data_color(DataNode data, Color *col);
 void data_point(DataNode data, Point *point);
 void data_rectangle(DataNode data, Rectangle *rect);
 char *data_string(DataNode data);
-Font *data_font(DataNode data);
+char *data_filename(DataNode data);
+DiaFont *data_font(DataNode data);
 
 AttributeNode new_attribute(ObjectNode obj_node, const char *attrname);
 AttributeNode composite_add_attribute(DataNode composite_node,
@@ -72,12 +72,14 @@ void data_add_int(AttributeNode attr, int data);
 void data_add_enum(AttributeNode attr, int data);
 void data_add_real(AttributeNode attr, real data);
 void data_add_boolean(AttributeNode attr, int data);
-void data_add_color(AttributeNode attr, Color *col);
-void data_add_point(AttributeNode attr, Point *point);
-void data_add_rectangle(AttributeNode attr, Rectangle *rect);
-void data_add_string(AttributeNode attr, char *str);
-void data_add_font(AttributeNode attr, Font *font);
-DataNode data_add_composite(AttributeNode attr, char *type); /* can be NULL */
+void data_add_color(AttributeNode attr, const Color *col);
+void data_add_point(AttributeNode attr, const Point *point);
+void data_add_rectangle(AttributeNode attr, const Rectangle *rect);
+void data_add_string(AttributeNode attr, const char *str);
+void data_add_filename(AttributeNode attr, const char *str);
+void data_add_font(AttributeNode attr, const DiaFont *font);
+DataNode data_add_composite(AttributeNode attr, 
+                            const char *type); /* can be NULL */
 
 #endif /* DIA_XML_H */
 
