@@ -15,57 +15,62 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
-#include <assert.h>
 
-#include "config.h"
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include "intl.h"
 #include "diagramdata.h"
+#include "diarenderer.h"
+#include "paper.h"
+#include "dynamic_obj.h"
+
+static const Rectangle invalid_extents = { -1.0,-1.0,-1.0,-1.0 };
+static void set_parent_layer(gpointer layer, gpointer object);
 
 DiagramData *
-new_diagram_data(void)
+new_diagram_data (NewDiagramData *prefs)
 {
-  DiagramData *data;
-  Layer *first_layer;
-
-  data = g_new(DiagramData, 1);
+	DiagramData *data;
+	Layer *first_layer;
    
-  data->extents.left = 0.0; 
-  data->extents.right = 10.0; 
-  data->extents.top = 0.0; 
-  data->extents.bottom = 10.0; 
+	data = g_new (DiagramData, 1);
+   
+	data->extents.left = 0.0; 
+	data->extents.right = 10.0; 
+	data->extents.top = 0.0; 
+	data->extents.bottom = 10.0; 
  
-  data->bg_color = color_white;
+	data->bg_color = prefs->bg_color;
+	data->pagebreak_color = prefs->pagebreak_color;
 
-  data->paper.name = g_strdup("A4");
-  data->paper.tmargin = data->paper.bmargin =
-    data->paper.lmargin = data->paper.rmargin = 2.82;
-  data->paper.is_portrait = TRUE;
-  data->paper.scaling = 1.0;
-  data->paper.fitto = FALSE;
-  data->paper.fitwidth = 1;
-  data->paper.fitheight = 1;
-  data->paper.width = 21.0 - 2 * 2.82;
-  data->paper.height = 29.7 - 2 * 2.82;
+	get_paper_info (&data->paper, -1, prefs);
 
-  data->grid.width_x = 1.0;
-  data->grid.width_y = 1.0;
-  data->grid.visible_x = 1;
-  data->grid.visible_y = 1;
+	data->grid.dynamic = TRUE;
+	data->grid.width_x = 1.0;
+	data->grid.width_y = 1.0;
+	data->grid.visible_x = 1;
+	data->grid.visible_y = 1;
+	data->grid.colour = prefs->grid_color;
 
-  data->guides.nhguides = 0;
-  data->guides.hguides = NULL;
-  data->guides.nvguides = 0;
-  data->guides.vguides = NULL;
+	data->guides.nhguides = 0;
+	data->guides.hguides = NULL;
+	data->guides.nvguides = 0;
+	data->guides.vguides = NULL;
 
-  first_layer = new_layer(g_strdup(_("Background")));
-  data->layers = g_ptr_array_new();
-  g_ptr_array_add(data->layers, first_layer);
-  data->active_layer = first_layer;
-
-  data->selected_count = 0;
-  data->selected = NULL;
+	first_layer = new_layer(g_strdup(_("Background")),data);
   
-  return data;
+	data->layers = g_ptr_array_new ();
+	g_ptr_array_add (data->layers, first_layer);
+	data->active_layer = first_layer;
+
+	data->selected_count = 0;
+	data->selected = NULL;
+  
+	data->is_compressed = prefs->compress_save; // Overridden by doc
+
+	return data;
 }
 
 void
@@ -74,6 +79,7 @@ diagram_data_destroy(DiagramData *data)
   int i;
 
   g_free(data->paper.name);
+
   for (i=0;i<data->layers->len;i++) {
     layer_destroy(g_ptr_array_index(data->layers, i));
   }
@@ -87,14 +93,15 @@ diagram_data_destroy(DiagramData *data)
 }
 
 Layer *
-new_layer(char *name)
+new_layer(gchar *name, DiagramData *parent)
 {
   Layer *layer;
 
   layer = g_new(Layer, 1);
 
   layer->name = name;
-  
+
+  layer->parent_diagram = parent;
   layer->visible = TRUE;
 
   layer->objects = NULL;
@@ -127,7 +134,7 @@ data_raise_layer(DiagramData *data, Layer *layer)
       layer_nr = i;
   }
 
-  assert(layer_nr>=0);
+  g_assert(layer_nr>=0);
 
   if (layer_nr < data->layers->len-1) {
     tmp = g_ptr_array_index(data->layers, layer_nr+1);
@@ -149,7 +156,7 @@ data_lower_layer(DiagramData *data, Layer *layer)
       layer_nr = i;
   }
 
-  assert(layer_nr>=0);
+  g_assert(layer_nr>=0);
 
   if (layer_nr > 0) {
     tmp = g_ptr_array_index(data->layers, layer_nr-1);
@@ -164,6 +171,7 @@ void
 data_add_layer(DiagramData *data, Layer *layer)
 {
   g_ptr_array_add(data->layers, layer);
+  layer->parent_diagram = data;
   layer_update_extents(layer);
   data_update_extents(data);
 }
@@ -183,6 +191,7 @@ data_add_layer_at(DiagramData *data, Layer *layer, int pos)
     g_ptr_array_index(data->layers, pos) = layer;
   }
   
+  layer->parent_diagram = data;
   layer_update_extents(layer);
   data_update_extents(data);
 }
@@ -202,6 +211,7 @@ data_delete_layer(DiagramData *data, Layer *layer)
   if (data->active_layer == layer) {
     data_remove_all_selected(data);
   }
+  layer->parent_diagram = NULL;
   g_ptr_array_remove(data->layers, layer);
 
   if (data->active_layer == layer) {
@@ -223,6 +233,9 @@ data_unselect(DiagramData *data, Object *obj)
   data->selected_count--;
 }
 
+/** Clears the list of selected objects.
+ * Does *not* remove these objects from the object list.
+ */
 void
 data_remove_all_selected(DiagramData *data)
 {
@@ -231,46 +244,93 @@ data_remove_all_selected(DiagramData *data)
   data->selected = NULL;
 }
 
-int
+static gboolean
+data_has_visible_layers(DiagramData *data)
+{
+  guint i;
+  for (i = 0; i < data->layers->len; i++) {
+    Layer *layer = g_ptr_array_index(data->layers, i);
+    if (layer->visible) return TRUE;
+  }
+  return FALSE;
+}
+
+static void
+data_get_layers_extents_union(DiagramData *data) {
+  guint i;
+  gboolean first = TRUE;
+  Rectangle new_extents;
+
+  for ( i = 0 ; i<data->layers->len; i++) {
+    Layer *layer = g_ptr_array_index(data->layers, i);    
+    if (!layer->visible) continue;
+    
+    layer_update_extents(layer);
+
+    if (first) {
+      new_extents = layer->extents;
+      first = rectangle_equals(&new_extents,&invalid_extents);
+    } else {
+      if (!rectangle_equals(&layer->extents,&invalid_extents)) {
+        rectangle_union(&new_extents, &layer->extents);
+      }
+    }
+  }
+
+  data->extents = new_extents;
+}
+
+static void
+data_adapt_scaling_to_extents(DiagramData *data)
+{
+  gdouble pwidth = data->paper.width * data->paper.scaling;
+  gdouble pheight = data->paper.height * data->paper.scaling;
+
+  gdouble xscale = data->paper.fitwidth * pwidth /
+    (data->extents.right - data->extents.left);
+  gdouble yscale = data->paper.fitheight * pheight /
+    (data->extents.bottom - data->extents.top);
+  
+  data->paper.scaling = MIN(xscale, yscale);
+  data->paper.width  = pwidth  / data->paper.scaling;
+  data->paper.height = pheight / data->paper.scaling;
+}
+
+static gboolean
+data_compute_extents(DiagramData *data) 
+{
+  Rectangle old_extents = data->extents;
+
+  if (!data_has_visible_layers(data)) {   
+    if (data->layers->len > 0) {
+      Layer *layer = g_ptr_array_index(data->layers, 0);    
+      layer_update_extents(layer);
+      
+      data->extents = layer->extents;
+    } else {
+      data->extents = invalid_extents;
+    }
+  } else {
+    data_get_layers_extents_union(data);
+  }
+
+  if (rectangle_equals(&data->extents,&invalid_extents)) {
+      data->extents.left = 0.0;
+      data->extents.right = 10.0;
+      data->extents.top = 0.0;
+      data->extents.bottom = 10.0;
+  }
+  return (!rectangle_equals(&data->extents,&old_extents));
+}
+
+gboolean
 data_update_extents(DiagramData *data)
 {
-  Layer *layer;
-  Rectangle new_extents;
-  Rectangle *extents;
-  int changed;
-  int i;
 
-  extents = &data->extents;
+  gboolean changed;
 
-  layer = (Layer *) g_ptr_array_index(data->layers, 0);
-  new_extents = layer->extents;
-  for (i=1;i<data->layers->len;i++) {
-    layer = (Layer *) g_ptr_array_index(data->layers, i);
-    rectangle_union(&new_extents, &layer->extents);
-  }
-
-  changed =  ( (new_extents.left != extents->left) ||
-	       (new_extents.right != extents->right) ||
-	       (new_extents.top != extents->top) ||
-	       (new_extents.bottom != extents->bottom) );
-
-  *extents = new_extents;
-
-  /* update scaling if extents changed and we are in fit to scaling mode */
-  if (changed && data->paper.fitto) {
-    gdouble xscale, yscale;
-    gdouble pwidth = data->paper.width * data->paper.scaling;
-    gdouble pheight = data->paper.height * data->paper.scaling;
-
-    xscale = data->paper.fitwidth * pwidth /
-      (new_extents.right - new_extents.left);
-    yscale = data->paper.fitheight * pheight /
-      (new_extents.bottom - new_extents.top);
-
-    data->paper.scaling = MIN(xscale, yscale);
-    data->paper.width  = pwidth  / data->paper.scaling;
-    data->paper.height = pheight / data->paper.scaling;
-  }
+  changed = data_compute_extents(data);
+  if (changed && data->paper.fitto) data_adapt_scaling_to_extents(data);
 
   return changed;
 }
@@ -300,6 +360,9 @@ data_get_sorted_selected(DiagramData *data)
   return sorted_list;
 }
 
+/** Remove the currently selected objects from the list of objects.
+ * The selected objects are returned in a newly created GList.
+ */
 GList *
 data_get_sorted_selected_remove(DiagramData *data)
 {
@@ -332,7 +395,7 @@ data_get_sorted_selected_remove(DiagramData *data)
 }
 
 void
-data_render(DiagramData *data, Renderer *renderer, Rectangle *update,
+data_render(DiagramData *data, DiaRenderer *renderer, Rectangle *update,
 	    ObjectRenderer obj_renderer /* Can be NULL */,
 	    gpointer gdata)
 {
@@ -340,7 +403,7 @@ data_render(DiagramData *data, Renderer *renderer, Rectangle *update,
   int i;
   int active_layer;
 
-  (renderer->ops->begin_render)(renderer);
+  if (!renderer->is_interactive) (DIA_RENDERER_GET_CLASS(renderer)->begin_render)(renderer);
   
   for (i=0; i<data->layers->len; i++) {
     layer = (Layer *) g_ptr_array_index(data->layers, i);
@@ -349,20 +412,23 @@ data_render(DiagramData *data, Renderer *renderer, Rectangle *update,
       layer_render(layer, renderer, update, obj_renderer, gdata, active_layer);
   }
   
-  (renderer->ops->end_render)(renderer);
+  if (!renderer->is_interactive) (DIA_RENDERER_GET_CLASS(renderer)->end_render)(renderer);
 }
 
 static void
-normal_render(Object *obj, Renderer *renderer,
+normal_render(Object *obj, DiaRenderer *renderer,
 	      int active_layer,
 	      gpointer data)
 {
-  obj->ops->draw(obj, renderer);
+  DIA_RENDERER_GET_CLASS(renderer)->draw_object(renderer, obj);
 }
 
+
+int render_bounding_boxes = FALSE;
+
 /* If obj_renderer is NULL normal_render is used. */
 void
-layer_render(Layer *layer, Renderer *renderer, Rectangle *update,
+layer_render(Layer *layer, DiaRenderer *renderer, Rectangle *update,
 	     ObjectRenderer obj_renderer,
 	     gpointer data,
 	     int active_layer)
@@ -378,13 +444,33 @@ layer_render(Layer *layer, Renderer *renderer, Rectangle *update,
   while (list!=NULL) {
     obj = (Object *) list->data;
 
-    if (update==NULL || rectangle_intersects(update, &obj->bounding_box))
-	(*obj_renderer)(obj, renderer, active_layer, data);
+    if (update==NULL || rectangle_intersects(update, &obj->bounding_box)) {
+      if ((render_bounding_boxes) && (renderer->is_interactive)) {
+	Point p1, p2;
+	Color col;
+	p1.x = obj->bounding_box.left;
+	p1.y = obj->bounding_box.top;
+	p2.x = obj->bounding_box.right;
+	p2.y = obj->bounding_box.bottom;
+	col.red = 1.0;
+	col.green = 0.0;
+	col.blue = 1.0;
+
+        DIA_RENDERER_GET_CLASS(renderer)->set_linewidth(renderer,0.01);
+	DIA_RENDERER_GET_CLASS(renderer)->draw_rect(renderer, &p1, &p2, &col);
+      }
+      (*obj_renderer)(obj, renderer, active_layer, data);
+    }
     
     list = g_list_next(list);
   }
 }
 
+static void
+set_parent_layer(gpointer element, gpointer user_data) {
+  ((Object*)element)->parent_layer = (Layer*)user_data;
+}
+
 int
 layer_object_index(Layer *layer, Object *obj)
 {
@@ -395,30 +481,36 @@ void
 layer_add_object(Layer *layer, Object *obj)
 {
   layer->objects = g_list_append(layer->objects, (gpointer) obj);
+  set_parent_layer(obj, layer);
 }
 
 void
 layer_add_object_at(Layer *layer, Object *obj, int pos)
 {
   layer->objects = g_list_insert(layer->objects, (gpointer) obj, pos);
+  set_parent_layer(obj, layer);
 }
 
 void
 layer_add_objects(Layer *layer, GList *obj_list)
 {
   layer->objects = g_list_concat(layer->objects, obj_list);
+  g_list_foreach(obj_list, set_parent_layer, layer);
 }
 
 void
 layer_add_objects_first(Layer *layer, GList *obj_list)
 {
   layer->objects = g_list_concat(obj_list, layer->objects);
+  g_list_foreach(obj_list, set_parent_layer, layer);
 }
 
 void
 layer_remove_object(Layer *layer, Object *obj)
 {
   layer->objects = g_list_remove(layer->objects, obj);
+  dynobj_list_remove_object(obj);
+  set_parent_layer(obj, NULL);
 }
 
 void
@@ -431,6 +523,8 @@ layer_remove_objects(Layer *layer, GList *obj_list)
     layer->objects = g_list_remove(layer->objects, obj);
     
     obj_list = g_list_next(obj_list);
+    dynobj_list_remove_object(obj);
+    set_parent_layer(obj, NULL);
   }
 }
 
@@ -479,36 +573,54 @@ layer_find_objects_in_rectangle(Layer *layer, Rectangle *rect)
   return selected_list;
 }
 
+/** Find the object closest to the given point in the layer,
+ * no further away than maxdist, and not included in avoid.
+ * Stops it if finds an object that includes the point.
+ */
 Object *
-layer_find_closest_object(Layer *layer, Point *pos, real maxdist)
+layer_find_closest_object_except(Layer *layer, Point *pos,
+				 real maxdist, GList *avoid)
 {
   GList *l;
   Object *closest;
   Object *obj;
   real dist;
-  
+  GList *avoid_tmp;
+
   closest = NULL;
   
-  l = layer->objects;
-  while(l!=NULL) {
+  for (l = layer->objects; l!=NULL; l = g_list_next(l)) {
     obj = (Object *) l->data;
 
     /* Check bounding box here too. Might give speedup. */
     dist = obj->ops->distance_from(obj, pos);
 
-    if (dist<=maxdist) {
+    if (maxdist-dist > 0.00000001) {
+      for (avoid_tmp = avoid; avoid_tmp != NULL; avoid_tmp = avoid_tmp->next) {
+	if (avoid_tmp->data == obj) {
+	  goto NEXTOBJECT;
+	}
+      }
       closest = obj;
     }
-    
-    l = g_list_next(l);
+  NEXTOBJECT:
+  ;
   }
 
   return closest;
 }
 
+Object *
+layer_find_closest_object(Layer *layer, Point *pos, real maxdist)
+{
+  return layer_find_closest_object_except(layer, pos, maxdist, NULL);
+}
+
+
 real layer_find_closest_connectionpoint(Layer *layer,
 					ConnectionPoint **closest,
-					Point *pos)
+					Point *pos,
+					Object *notthis)
 {
   GList *l;
   Object *obj;
@@ -520,10 +632,10 @@ real layer_find_closest_connectionpoint(Layer *layer,
   
   *closest = NULL;
   
-  l = layer->objects;
-  while(l!=NULL) {
+  for (l = layer->objects; l!=NULL; l = g_list_next(l) ) {
     obj = (Object *) l->data;
 
+    if (obj == notthis) continue;
     for (i=0;i<obj->num_connections;i++) {
       cp = obj->connections[i];
       /* Note: Uses manhattan metric for speed... */
@@ -534,8 +646,7 @@ real layer_find_closest_connectionpoint(Layer *layer,
       }
     }
     
-    l = g_list_next(l);
-  }
+ }
 
   return mindist;
 }
@@ -545,11 +656,7 @@ int layer_update_extents(Layer *layer)
   GList *l;
   Object *obj;
   Rectangle new_extents;
-  Rectangle *extents;
-  int changed;
   
-  extents = &layer->extents;
-
   l = layer->objects;
   if (l!=NULL) {
     obj = (Object *) l->data;
@@ -562,20 +669,13 @@ int layer_update_extents(Layer *layer)
       l = g_list_next(l);
     }
   } else {
-    new_extents.left = 0.0; 
-    new_extents.right = 10.0; 
-    new_extents.top = 0.0; 
-    new_extents.bottom = 10.0; 
+    new_extents = invalid_extents;
   }
 
-  changed =  ( (new_extents.left != extents->left) ||
-	       (new_extents.right != extents->right) ||
-	       (new_extents.top != extents->top) ||
-	       (new_extents.bottom != extents->bottom) );
+  if (rectangle_equals(&new_extents,&layer->extents)) return FALSE;
 
-  *extents = new_extents;
-
-  return changed;
+  layer->extents = new_extents;
+  return TRUE;
 }
 
 void
@@ -586,8 +686,11 @@ layer_replace_object_with_list(Layer *layer, Object *remove_obj,
 
   list = g_list_find(layer->objects, remove_obj);
 
-  assert(list!=NULL);
-  
+  g_assert(list!=NULL);
+  set_parent_layer(remove_obj, NULL);
+  dynobj_list_remove_object(remove_obj);
+  g_list_foreach(insert_list, set_parent_layer, layer);
+
   if (list->prev == NULL) {
     layer->objects = insert_list;
   } else {
@@ -603,12 +706,23 @@ layer_replace_object_with_list(Layer *layer, Object *remove_obj,
   g_list_free_1(list);
 }
 
-void layer_set_object_list(Layer *layer, GList *list)
+static void
+layer_remove_dynobj(gpointer obj, gpointer userdata)
 {
-  g_list_free(layer->objects);
-  layer->objects = list;
+  dynobj_list_remove_object((Object*)obj);
 }
 
+void layer_set_object_list(Layer *layer, GList *list)
+{
+  g_list_foreach(layer->objects, set_parent_layer, NULL);
+  g_list_foreach(layer->objects, layer_remove_dynobj, NULL);
+  g_list_free(layer->objects);
+  layer->objects = list;
+  g_list_foreach(layer->objects, set_parent_layer, layer);
+}
 
-
-
+DiagramData *
+layer_get_parent_diagram(Layer *layer)
+{
+  return layer->parent_diagram;
+}
