@@ -34,20 +34,37 @@ struct _NewDiagramData {
   gfloat scaling;
   gboolean fitto;
   gint fitwidth, fitheight;
+  Color bg_color, pagebreak_color, grid_color;
+  int compress_save;
 };
 
 struct _DiagramData {
   Rectangle extents;      /* The extents of the diagram        */
 
   Color bg_color;
+  Color pagebreak_color;
 
   PaperInfo paper;       /* info about the page info for the diagram */
+  gboolean is_compressed; /* TRUE if by default it should be save compressed.
+			     The user can override this in Save As... */
 
   struct  {
     /* grid line intervals */
     real width_x, width_y;
     /* the interval between visible grid lines */
     guint visible_x, visible_y;
+    /* the interval between major lines (non-stippled).
+     * if 0, no major lines are drawn (all lines are stippled).
+     * if 1, all lines are solid.
+     */
+    guint major_lines;
+    /* True if the grid is dynamically calculated.
+     * When true, width_x and width_y are ignored.
+     */
+    gboolean dynamic;
+    /* The color of the grid lines.
+     */
+    Color colour;
   } grid;
 
   struct {
@@ -82,6 +99,7 @@ struct _Layer {
 };
 
 #include "object.h"
+#include "dynamic_obj.h"
 
 DIAVAR int render_bounding_boxes;
 
@@ -126,6 +144,8 @@ void layer_remove_objects(Layer *layer, GList *obj_list);
 GList *layer_find_objects_intersecting_rectangle(Layer *layer, Rectangle*rect);
 GList *layer_find_objects_in_rectangle(Layer *layer, Rectangle *rect);
 Object *layer_find_closest_object(Layer *layer, Point *pos, real maxdist);
+Object *layer_find_closest_object_except(Layer *layer, Point *pos,
+					 real maxdist, GList *avoid);
 real layer_find_closest_connectionpoint(Layer *layer,
 					ConnectionPoint **closest,
 					Point *pos,
