@@ -25,6 +25,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h> /* strlen */
+#include <time.h>
 
 #include <pango/pango.h>
 #ifdef HAVE_FREETYPE
@@ -285,7 +286,6 @@ dia_font_new_from_style(DiaFontStyle style, real height)
   
   retval = DIA_FONT(g_type_create_instance(dia_font_get_type()));
   retval->pfd = pfd;
-  dia_font_ref(retval);
   return retval;
 }
 
@@ -304,6 +304,7 @@ dia_font_finalize(GObject* object)
     font = DIA_FONT(object);
     
     if (font->pfd) pango_font_description_free(font->pfd);
+    G_OBJECT_CLASS(parent_class)->finalize(object);
 }
 
 DiaFont* dia_font_ref(DiaFont* font)
@@ -501,6 +502,98 @@ real dia_font_descent(const char* string, DiaFont* font, real height)
     return dia_font_scaled_descent(string,font,height,global_zoom_factor);
 }
 
+typedef struct {
+  gchar *string;
+  DiaFont *font;
+  real height;
+  PangoLayout *layout;
+  int usecount;
+} LayoutCacheItem;
+
+static GHashTable *layoutcache;
+
+static gboolean
+layout_cache_equals(gconstpointer e1, gconstpointer e2) 
+{
+  LayoutCacheItem *i1 = (LayoutCacheItem*)e1,
+    *i2 = (LayoutCacheItem*)e2;
+
+  return strcmp(i1->string, i2->string) == 0 &&
+    fabs(i1->height - i2->height) < 0.00001 &&
+    i1->font == i2->font;
+}
+
+static guint
+layout_cache_hash(gconstpointer el) 
+{
+  LayoutCacheItem *item = (LayoutCacheItem*)el;
+
+  return g_str_hash(item->string) ^
+    (int)(item->height*1000) ^
+    (int)(item->font);
+}
+
+static long layout_cache_last_use;
+
+static gboolean
+layout_cache_cleanup_entry(gpointer key, gpointer value, gpointer data)
+{
+  LayoutCacheItem *item = (LayoutCacheItem*)value;
+  /** Remove unused items */
+  if (item->usecount == 0) return TRUE;
+  item->usecount = 0;
+  return FALSE;
+}
+
+/** The actual hash cleanup, called when idle. */
+static gboolean
+layout_cache_cleanup_idle(gpointer data)
+{
+  GHashTable *table = (GHashTable*)(data);
+
+  g_hash_table_foreach_remove(table, layout_cache_cleanup_entry, NULL);
+
+  return FALSE;
+}
+
+/** Every ten minutes, clean up those strings that haven't seen use since
+ * last cleanup.
+ */
+static gboolean
+layout_cache_cleanup(gpointer data)
+{
+  /* Only cleanup if there has been font activity since last cleanup */
+  if (time(0) - layout_cache_last_use < 60*10) {
+    /* Don't go directly to cleanup, wait till there's a pause. */
+    g_idle_add(layout_cache_cleanup_idle, data);
+  }
+  /* Keep doing this */
+  return TRUE;
+}
+
+static void
+layout_cache_free_key(gpointer data)
+{
+    LayoutCacheItem *item = (LayoutCacheItem*)data;
+
+    if (item->string != NULL) {
+      g_free(item->string);
+      item->string = NULL;
+    }
+
+    if (item->font != NULL) {
+      g_object_unref(item->font);
+      item->font = NULL;
+    }
+
+    if (item->layout != NULL) {
+      g_object_unref(item->layout);
+      item->layout = NULL;
+    }
+    g_free(item);
+}
+
+
 PangoLayout*
 dia_font_build_layout(const char* string, DiaFont* font, real height)
 {
@@ -508,6 +601,36 @@ dia_font_build_layout(const char* string, DiaFont* font, real height)
     PangoAttrList* list;
     PangoAttribute* attr;
     guint length;
+    gchar *desc = NULL;
+
+    LayoutCacheItem *cached;
+
+    layout_cache_last_use = time(0);
+    if (layoutcache == NULL) {
+      /** Note that key and value are the same -- it's a HashSet */
+      layoutcache = g_hash_table_new_full(layout_cache_hash,
+					  layout_cache_equals,
+					  layout_cache_free_key,
+					  NULL);
+      g_timeout_add(10*60*1000, layout_cache_cleanup, (gpointer)layoutcache);
+    } else {
+      LayoutCacheItem item;
+      item.string = string;
+      item.font = font;
+      item.height = height;
+      cached = g_hash_table_lookup(layoutcache, &item);
+      if (cached != NULL) {
+	g_object_ref(cached->layout);
+	cached->usecount ++;
+	return cached->layout;
+      }
+    }
+
+    cached = g_new0(LayoutCacheItem,1);
+    cached->string = g_strdup(string);
+    cached->font = font;
+    g_object_ref(font);
+    cached->height = height;
 
     height *= 0.7;
 
@@ -518,7 +641,9 @@ dia_font_build_layout(const char* string, DiaFont* font, real height)
     pango_layout_set_text(layout,string,length);
         
     list = pango_attr_list_new();
-    pango_font_description_set_family(font->pfd, g_utf8_strdown(pango_font_description_get_family(font->pfd), -1));
+    desc = g_utf8_strdown(pango_font_description_get_family(font->pfd), -1);
+    pango_font_description_set_family(font->pfd, desc);
+    g_free(desc);
     attr = pango_attr_font_desc_new(font->pfd);
     attr->start_index = 0;
     attr->end_index = length;
@@ -531,6 +656,11 @@ dia_font_build_layout(const char* string, DiaFont* font, real height)
     pango_layout_set_justify(layout,FALSE);
     pango_layout_set_alignment(layout,PANGO_ALIGN_LEFT);
   
+    cached->layout = layout;
+    g_object_ref(layout);
+    cached->usecount = 1;
+    g_hash_table_insert(layoutcache, cached, cached);
+
     return layout;
 }
 
