@@ -22,11 +22,18 @@
 #include <assert.h>
 #include <math.h>
 #include <string.h> /* memcpy() */
+#include <glib.h>
 
 #include "orth_conn.h"
 #include "message.h"
 #include "diamenu.h"
 #include "handle.h"
+#include "diarenderer.h"
+#include "autoroute.h"
+
+static void place_handle_by_swapping(OrthConn *orth, 
+                                     int index, Handle *handle);
+static ObjectChange *orthconn_set_autorouting(OrthConn *orth, gboolean on);
 
 enum change_type {
   TYPE_ADD_SEGMENT,
@@ -88,6 +95,14 @@ struct EndSegmentChange {
 			  remove segment */
 };
 
+static ObjectChange*
+autoroute_create_change(OrthConn *orth, gboolean on);
+
+struct AutorouteChange {
+  ObjectChange obj_change;
+  gboolean on;
+  Point *points;
+};
 
 static void set_midpoint(Point *point, OrthConn *orth, int segment)
 {
@@ -146,16 +161,23 @@ static int get_segment_nr(OrthConn *orth, Point *point, real max_dist)
 }
 
 
-void
+ObjectChange *
 orthconn_move_handle(OrthConn *orth, Handle *handle,
-		     Point *to, HandleMoveReason reason)
+		     Point *to, ConnectionPoint *cp,
+		     HandleMoveReason reason, ModifierKeys modifiers)
 {
   int n;
   int handle_nr;
-  
+  Object *obj = (Object *)orth;
+  ObjectChange *change = NULL;
+
   switch(handle->id) {
   case HANDLE_MOVE_STARTPOINT:
     orth->points[0] = *to;
+    if (orth->autorouting &&
+	autoroute_layout_orthconn(orth, cp,
+				  obj->handles[1]->connected_to))
+      break;
     switch (orth->orientation[0]) {
     case HORIZONTAL:
       orth->points[1].y = to->y;
@@ -168,6 +190,10 @@ orthconn_move_handle(OrthConn *orth, Handle *handle,
   case HANDLE_MOVE_ENDPOINT:
     n = orth->numpoints - 1;
     orth->points[n] = *to;
+    if (orth->autorouting &&
+	autoroute_layout_orthconn(orth, obj->handles[0]->connected_to,
+				  cp))
+      break;
     switch (orth->orientation[n-1]) {
     case HORIZONTAL:
       orth->points[n-1].y = to->y;
@@ -175,12 +201,13 @@ orthconn_move_handle(OrthConn *orth, Handle *handle,
     case VERTICAL:
       orth->points[n-1].x = to->x;
       break;
-    } 
+    }
     break;
   case HANDLE_MIDPOINT:
     n = orth->numpoints - 1;
     handle_nr = get_handle_nr(orth, handle);
-
+    if (orth->autorouting)
+      change = orthconn_set_autorouting(orth, FALSE);
     switch (orth->orientation[handle_nr]) {
     case HORIZONTAL:
       orth->points[handle_nr].y = to->y;
@@ -196,9 +223,11 @@ orthconn_move_handle(OrthConn *orth, Handle *handle,
     message_error("Internal error in orthconn_move_handle.\n");
     break;
   }
+
+  return change;
 }
 
-void
+ObjectChange *
 orthconn_move(OrthConn *orth, Point *to)
 {
   Point p;
@@ -211,6 +240,7 @@ orthconn_move(OrthConn *orth, Point *to)
   for (i=1;i<orth->numpoints;i++) {
     point_add(&orth->points[i], &p);
   }
+  return NULL;
 }
 
 real
@@ -230,14 +260,61 @@ orthconn_distance_from(OrthConn *orth, Point *point, real line_width)
 }
 
 
+static void
+adjust_handle_count_to(OrthConn *orth, guint count) {
+  /* This will shrink or expand orth->handles as necessary (so that 
+     orth->numhandles matches orth->numpoints-1, most probably), by adding or
+     removing minor handles and keeping the endpoint handles at the 
+     extremities of the array. */
+
+  if (orth->numhandles == count) return;
+  if (orth->numhandles < count) { /* adding */
+    int i;
+    orth->handles = g_realloc(orth->handles,
+                              (count)*sizeof(Handle *));
+    orth->handles[count-1] = orth->handles[orth->numhandles-1];
+    orth->handles[orth->numhandles-1] = NULL; 
+    for (i=orth->numhandles-1; i<count-1; i++) {  
+      Handle *handle = g_new0(Handle,1);
+      setup_midpoint_handle(handle);
+      object_add_handle(&orth->object,handle);
+      orth->handles[i] = handle;
+    }
+  } else {  /* removing */
+    int i;
+    for (i=count-1; i<orth->numhandles-1; i++) {
+      Handle *handle = orth->handles[i];
+      object_remove_handle(&orth->object,handle);
+      g_free(handle);
+      orth->handles[i] = NULL;
+    }
+    orth->handles[count-1] = orth->handles[orth->numhandles-1];
+    orth->handles[orth->numhandles-1] = NULL;
+    orth->handles = g_realloc(orth->handles,
+			  (count)*sizeof(Handle *));
+  }
+  orth->numhandles = count;
+  /* handles' positions will be set now */ 
+}
+
 void
 orthconn_update_data(OrthConn *orth)
 {
   int i;
   Object *obj = (Object *)orth;
   
+  if (!orth->points) {
+    g_warning("very sick OrthConn object...");
+    return;
+  }
   obj->position = orth->points[0];
 
+  adjust_handle_count_to(orth,orth->numpoints-1);
+
+  /* Make sure start-handle is first and end-handle is second. */
+  place_handle_by_swapping(orth, 0, orth->handles[0]);
+  place_handle_by_swapping(orth, 1, orth->handles[orth->numpoints-2]);
+
   /* Update handles: */
   orth->handles[0]->pos = orth->points[0];
   orth->handles[orth->numpoints-2]->pos = orth->points[orth->numpoints-1];
@@ -250,77 +327,33 @@ orthconn_update_data(OrthConn *orth)
 void
 orthconn_update_boundingbox(OrthConn *orth)
 {
-  Rectangle *bb;
-  Point *points;
-  int i;
-  Point pt1,pt0;
-  OrthConnBBExtras *extra = &orth->extra_spacing;
-  
   assert(orth != NULL);
-
-  bb = &orth->object.bounding_box;
-  points = &orth->points[0];
-
-  bb->right = bb->left = points[0].x;
-  bb->top = bb->bottom = points[0].y;
-  
-  for (i=0;i<(orth->numpoints-1);i++) {
-    pt1 = points[i];
-    point_sub(&pt1,&(points[i+1]));
-    point_normalize(&pt1);
-
-    if (i==0) {
-      real trans = MAX(extra->start_trans,extra->middle_trans);
-      check_bb_x(bb,points[i].x + (extra->start_long * pt1.x),pt1.x);
-      check_bb_x(bb,points[i].x + (trans * pt1.y),pt1.y); 
-      check_bb_x(bb,points[i].x - (trans * pt1.y),pt1.y); 
-
-      check_bb_y(bb,points[i].y + (trans * pt1.x),pt1.x); 
-      check_bb_y(bb,points[i].y - (trans * pt1.x),pt1.x);       
-    } else {
-      check_bb_x(bb, points[i].x + (extra->middle_trans * pt1.y),pt1.y);
-      check_bb_x(bb, points[i].x - (extra->middle_trans * pt1.y),pt1.y);
-      check_bb_y(bb, points[i].y + (extra->middle_trans * pt1.x),pt1.x);
-      check_bb_y(bb, points[i].y - (extra->middle_trans * pt1.x),pt1.x);
-    }
-    if (i!=0) {
-      real overshoot = extra->middle_trans / (sin(M_PI/4.0));
-      Point pts;
-
-      pts = pt1; point_sub(&pts,&pt0);
-      point_normalize(&pts);
-
-      check_bb_x(bb,points[i].x + (overshoot * pts.x),pts.x);
-      check_bb_y(bb,points[i].y + (overshoot * pts.y),pts.y);
-    }
-    if (i==(orth->numpoints-2)) {
-      check_bb_x(bb,points[i+1].x - (extra->end_long * pt1.x),pt1.x);
-      check_bb_x(bb,points[i+1].x - (extra->end_trans * pt1.y),pt1.y);
-      check_bb_x(bb,points[i+1].x + (extra->end_trans * pt1.y),pt1.y);
-
-      check_bb_y(bb,points[i+1].y - (extra->end_trans * pt1.x),pt1.y);
-      check_bb_y(bb,points[i+1].y + (extra->end_trans * pt1.x),pt1.y);
-    }
-    pt0=pt1;
-  }
+  polyline_bbox(&orth->points[0],
+                orth->numpoints,
+                &orth->extra_spacing, FALSE,
+                &orth->object.bounding_box);
 }
 
 void
-orthconn_simple_draw(OrthConn *orth, Renderer *renderer, real width)
+orthconn_simple_draw(OrthConn *orth, DiaRenderer *renderer, real width)
 {
   Point *points;
   
   assert(orth != NULL);
   assert(renderer != NULL);
 
+  if (!orth->points) {
+    g_warning("very sick OrthConn object...");
+    return;
+  }
   points = &orth->points[0];
   
-  renderer->ops->set_linewidth(renderer, width);
-  renderer->ops->set_linestyle(renderer, LINESTYLE_SOLID);
-  renderer->ops->set_linejoin(renderer, LINEJOIN_MITER);
-  renderer->ops->set_linecaps(renderer, LINECAPS_BUTT);
+  DIA_RENDERER_GET_CLASS(renderer)->set_linewidth(renderer, width);
+  DIA_RENDERER_GET_CLASS(renderer)->set_linestyle(renderer, LINESTYLE_SOLID);
+  DIA_RENDERER_GET_CLASS(renderer)->set_linejoin(renderer, LINEJOIN_MITER);
+  DIA_RENDERER_GET_CLASS(renderer)->set_linecaps(renderer, LINECAPS_BUTT);
 
-  renderer->ops->draw_polyline(renderer, points, orth->numpoints,
+  DIA_RENDERER_GET_CLASS(renderer)->draw_polyline(renderer, points, orth->numpoints,
 			       &color_black);
 }
 
@@ -379,12 +412,14 @@ orthconn_init(OrthConn *orth, Point *startpoint)
   object_init(obj, 3, 0);
   
   orth->numpoints = 4;
+  orth->numorient = orth->numpoints - 1;
 
-  orth->points = g_malloc(4*sizeof(Point));
+  orth->points = g_malloc0(4*sizeof(Point));
 
-  orth->orientation = g_malloc(3*sizeof(Orientation));
+  orth->orientation = g_malloc0(3*sizeof(Orientation));
 
-  orth->handles = g_malloc(3*sizeof(Handle *));
+  orth->numhandles = 3;
+  orth->handles = g_malloc0(3*sizeof(Handle *));
 
   orth->handles[0] = g_new(Handle, 1);
   setup_endpoint_handle(orth->handles[0], HANDLE_MOVE_STARTPOINT);
@@ -398,6 +433,8 @@ orthconn_init(OrthConn *orth, Point *startpoint)
   setup_endpoint_handle(orth->handles[2], HANDLE_MOVE_ENDPOINT);
   obj->handles[2] = orth->handles[2];
 
+  orth->autorouting = TRUE;
+
   /* Just so we have some position: */
   orth->points[0] = *startpoint;
   orth->points[1].x = startpoint->x;
@@ -414,6 +451,37 @@ orthconn_init(OrthConn *orth, Point *startpoint)
   orthconn_update_data(orth);
 }
 
+/** This function does *not* set up handles */
+void
+orthconn_set_points(OrthConn *orth, int num_points, Point *points) 
+{
+  int i;
+  gboolean horiz;
+
+  orth->numpoints = num_points;
+
+  if (orth->points)
+    g_free(orth->points);
+
+  orth->points = g_malloc((orth->numpoints)*sizeof(Point));
+
+  for (i=0;i<orth->numpoints;i++) {
+    orth->points[i] = points[i];
+  }
+
+  /* Set up the orientation array. */
+  /* Maybe we could get rid of this array altogether? */
+  orth->numorient = orth->numpoints-1;
+  if (orth->orientation) g_free(orth->orientation);
+  orth->orientation = g_new(Orientation, orth->numorient);
+  horiz = (fabs(orth->points[0].y-orth->points[1].y) < 0.00001);
+  for (i = 0; i < orth->numorient; i++) {
+    if (horiz) orth->orientation[i] = HORIZONTAL;
+    else orth->orientation[i] = VERTICAL;
+    horiz = !horiz;
+  }
+}
+
 void
 orthconn_copy(OrthConn *from, OrthConn *to)
 {
@@ -426,15 +494,18 @@ orthconn_copy(OrthConn *from, OrthConn *to)
   object_copy(fromobj, toobj);
 
   to->numpoints = from->numpoints;
+  to->numorient = from->numorient;
 
-  to->points = g_malloc((to->numpoints)*sizeof(Point));
+  to->points = g_malloc0((to->numpoints)*sizeof(Point));
 
   for (i=0;i<to->numpoints;i++) {
     to->points[i] = from->points[i];
   }
 
-  to->orientation = g_malloc((to->numpoints-1)*sizeof(Orientation));
-  to->handles = g_malloc((to->numpoints-1)*sizeof(Handle *));
+  to->autorouting = from->autorouting;
+  to->orientation = g_malloc0((to->numpoints-1)*sizeof(Orientation));
+  to->numhandles = from->numhandles;
+  to->handles = g_malloc0((to->numpoints-1)*sizeof(Handle *));
 
   for (i=0;i<to->numpoints-1;i++) {
     to->orientation[i] = from->orientation[i];
@@ -507,6 +578,9 @@ orthconn_save(OrthConn *orth, ObjectNode obj_node)
   for (i=0;i<orth->numpoints-1;i++) {
     data_add_enum(attr, orth->orientation[i]);
   }
+
+  if (!orth->autorouting)
+    data_add_boolean(new_attribute(obj_node, "autorouting"), FALSE);
 }
 
 void
@@ -528,10 +602,12 @@ orthconn_load(OrthConn *orth, ObjectNode obj_node) /* NOTE: Does object_init() *
   else
     orth->numpoints = 0;
 
+  orth->numorient = orth->numpoints - 1;
+
   object_init(obj, orth->numpoints-1, 0);
 
   data = attribute_first_data(attr);
-  orth->points = g_malloc((orth->numpoints)*sizeof(Point));
+  orth->points = g_malloc0((orth->numpoints)*sizeof(Point));
   for (i=0;i<orth->numpoints;i++) {
     data_point(data, &orth->points[i]);
     data = data_next(data);
@@ -540,13 +616,18 @@ orthconn_load(OrthConn *orth, ObjectNode obj_node) /* NOTE: Does object_init() *
   attr = object_find_attribute(obj_node, "orth_orient");
 
   data = attribute_first_data(attr);
-  orth->orientation = g_malloc((orth->numpoints-1)*sizeof(Orientation));
+  orth->orientation = g_malloc0((orth->numpoints-1)*sizeof(Orientation));
   for (i=0;i<orth->numpoints-1;i++) {
     orth->orientation[i] = data_enum(data);
     data = data_next(data);
   }
 
-  orth->handles = g_malloc((orth->numpoints-1)*sizeof(Handle *));
+  orth->autorouting = TRUE;
+  attr = object_find_attribute(obj_node, "autorouting");
+  if (attr != NULL)
+    orth->autorouting = data_boolean(attribute_first_data(attr));
+
+  orth->handles = g_malloc0((orth->numpoints-1)*sizeof(Handle *));
 
   orth->handles[0] = g_new(Handle, 1);
   setup_endpoint_handle(orth->handles[0], HANDLE_MOVE_STARTPOINT);
@@ -564,6 +645,7 @@ orthconn_load(OrthConn *orth, ObjectNode obj_node) /* NOTE: Does object_init() *
     setup_midpoint_handle(orth->handles[i]);
     obj->handles[i+1] = orth->handles[i];
   }
+  orth->numhandles = orth->numpoints-1;
 
   orthconn_update_data(orth);
 }
@@ -662,12 +744,28 @@ orthconn_add_segment(OrthConn *orth, Point *clickedpoint)
   return change;
 }
 
+
+/* Set autorouting on or off.  If setting on, try to autoroute and
+ * return the changes from that.
+ */
+static ObjectChange *
+orthconn_set_autorouting(OrthConn *conn, gboolean on)
+{
+  Object *obj = (Object *)conn;
+  ObjectChange *change;
+
+  change = autoroute_create_change(conn, on);
+  change->apply(change, obj);
+  return change;
+}
+
 static void
 delete_point(OrthConn *orth, int pos)
 {
   int i;
   
   orth->numpoints--;
+  orth->numorient = orth->numpoints - 1;
 
   for (i=pos;i<orth->numpoints;i++) {
     orth->points[i] = orth->points[i+1];
@@ -698,6 +796,7 @@ remove_handle(OrthConn *orth, int segment)
 			  (orth->numpoints-1)*sizeof(Handle *));
   
   object_remove_handle(&orth->object, handle);
+  orth->numhandles = orth->numpoints-1;
 }
 
 
@@ -707,6 +806,7 @@ add_point(OrthConn *orth, int pos, Point *point)
   int i;
   
   orth->numpoints++;
+  orth->numorient = orth->numpoints-1;
 
   orth->points = g_realloc(orth->points, orth->numpoints*sizeof(Point));
   for (i=orth->numpoints-1;i>pos;i--) {
@@ -736,10 +836,9 @@ insert_handle(OrthConn *orth, int segment,
   orth->orientation[segment] = orient;
   
   object_add_handle(&orth->object, handle);
+  orth->numhandles = orth->numpoints-1;
 }
 
-
-
 static void
 endsegment_change_free(struct EndSegmentChange *change)
 {
@@ -960,6 +1059,79 @@ midsegment_create_change(OrthConn *orth, enum change_type type,
   return (ObjectChange *)change;
 }
 
+static void
+autoroute_change_free(struct AutorouteChange *change)
+{
+  g_free(change->points);
+}
 
+static void
+autoroute_change_apply(struct AutorouteChange *change, Object *obj)
+{
+  OrthConn *orth = (OrthConn*)obj;
 
+  if (change->on) {
+    orth->autorouting = TRUE;
+    autoroute_layout_orthconn(orth, obj->handles[0]->connected_to,
+			      obj->handles[1]->connected_to);
+  } else {
+    orth->autorouting = FALSE;
+    orthconn_set_points(orth, orth->numpoints, change->points);
+  }
+}
 
+static void
+autoroute_change_revert(struct AutorouteChange *change, Object *obj)
+{
+  OrthConn *orth = (OrthConn*)obj;
+
+  if (change->on) {
+    orth->autorouting = FALSE;
+    orthconn_set_points(orth, orth->numpoints, change->points);
+  } else {
+    orth->autorouting = TRUE;
+    autoroute_layout_orthconn(orth, obj->handles[0]->connected_to,
+			      obj->handles[1]->connected_to);
+  }
+}
+
+static ObjectChange *
+autoroute_create_change(OrthConn *orth, gboolean on)
+{
+  struct AutorouteChange *change;
+  int i;
+
+  change = g_new(struct AutorouteChange, 1);
+
+  change->obj_change.apply = (ObjectChangeApplyFunc) autoroute_change_apply;
+  change->obj_change.revert = (ObjectChangeRevertFunc) autoroute_change_revert;
+  change->obj_change.free = (ObjectChangeFreeFunc) autoroute_change_free;
+
+  change->on = on;
+  change->points = g_new(Point, orth->numpoints);
+  for (i = 0; i < orth->numpoints; i++)
+    change->points[i] = orth->points[i];
+
+  return (ObjectChange *)change;
+}
+
+ObjectChange *
+orthconn_toggle_autorouting_callback(Object *obj, Point *clicked, gpointer data)
+{
+  ObjectChange *change;
+  /* This is kinda hackish.  Since we can't see the menu item, we have to
+   * assume that we're right about toggling and just send !orth->autorouting.
+   */
+  change = orthconn_set_autorouting((OrthConn*)obj, 
+				    !((OrthConn*)obj)->autorouting);
+  orthconn_update_data((OrthConn *)obj);
+  return change;
+}
+
+void
+orthconn_update_object_menu(OrthConn *orth, Point *clicked,
+			    DiaMenuItem *object_menu_items)
+{
+  object_menu_items[0].active = DIAMENU_ACTIVE|DIAMENU_TOGGLE|
+    (orth->autorouting?DIAMENU_TOGGLE_ON:0);
+}
