@@ -418,21 +418,7 @@ text_distance_from(Text *text, Point *point)
 void
 text_draw(Text *text, DiaRenderer *renderer)
 {
-  Point pos;
-  int i;
-  
-  DIA_RENDERER_GET_CLASS(renderer)->set_font(renderer, text->font, text->height);
-  
-  pos = text->position;
-  
-  for (i=0;i<text->numlines;i++) {
-    DIA_RENDERER_GET_CLASS(renderer)->draw_string(renderer,
-			       text->line[i],
-			       &pos, text->alignment,
-			       &text->color);
-    pos.y += text->height;
-  }
-
+  DIA_RENDERER_GET_CLASS(renderer)->draw_text(renderer, text);
 
   if ((renderer->is_interactive) && (text->focus.has_focus)) {
     real curs_x, curs_y;
@@ -885,11 +871,10 @@ text_key_event(Focus *focus, guint keyval, const gchar *str, int strlen,
         break;
       default:
         if (str || (strlen>0)) {
-
           return_val = TRUE;
           utf = str;
-          for (utf = str ; utf && *utf && (utf-str <= strlen) ; 
-	       utf = g_utf8_next_char (utf)) {
+          for (utf = str; utf && *utf && strlen > 0 ;
+	       utf = g_utf8_next_char (utf), strlen--) {
             c = g_utf8_get_char (utf);
             
             *change = text_create_change (text, TYPE_INSERT_CHAR, c,
