@@ -16,12 +16,13 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+/* DO NOT USE THIS OBJECT AS A BASIS FOR A NEW OBJECT. */
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
 #include <assert.h>
-#include <gtk/gtk.h>
 #include <math.h>
 #include <string.h>
 
@@ -29,7 +30,7 @@
 #include "object.h"
 #include "element.h"
 #include "connectionpoint.h"
-#include "render.h"
+#include "diarenderer.h"
 #include "attributes.h"
 #include "widgets.h"
 #include "properties.h"
@@ -49,7 +50,7 @@ typedef struct _Attribute Attribute;
 struct _AttributeState {
   ObjectState obj_state;
 
-  utfchar *name;
+  gchar *name;
   real name_width;
 
   gboolean key;
@@ -67,7 +68,7 @@ struct _Attribute {
 
   DiaFont *font;
   real font_height;
-  utfchar *name;
+  gchar *name;
   real name_width;
 
   ConnectionPoint connections[8];
@@ -84,12 +85,13 @@ struct _Attribute {
 
 static real attribute_distance_from(Attribute *attribute, Point *point);
 static void attribute_select(Attribute *attribute, Point *clicked_point,
-			   Renderer *interactive_renderer);
-static void attribute_move_handle(Attribute *attribute, Handle *handle,
-				  Point *to, HandleMoveReason reason, 
-				  ModifierKeys modifiers);
-static void attribute_move(Attribute *attribute, Point *to);
-static void attribute_draw(Attribute *attribute, Renderer *renderer);
+			   DiaRenderer *interactive_renderer);
+static ObjectChange* attribute_move_handle(Attribute *attribute, Handle *handle,
+					   Point *to, ConnectionPoint *cp,
+					   HandleMoveReason reason, 
+					   ModifierKeys modifiers);
+static ObjectChange* attribute_move(Attribute *attribute, Point *to);
+static void attribute_draw(Attribute *attribute, DiaRenderer *renderer);
 static void attribute_update_data(Attribute *attribute);
 static Object *attribute_create(Point *startpoint,
 			      void *user_data,
@@ -211,35 +213,41 @@ attribute_distance_from(Attribute *attribute, Point *point)
 
 static void
 attribute_select(Attribute *attribute, Point *clicked_point,
-	       Renderer *interactive_renderer)
+	       DiaRenderer *interactive_renderer)
 {
   element_update_handles(&attribute->element);
 }
 
-static void
+static ObjectChange*
 attribute_move_handle(Attribute *attribute, Handle *handle,
-		      Point *to, HandleMoveReason reason, 
-		      ModifierKeys modifiers)
+		      Point *to, ConnectionPoint *cp,
+		      HandleMoveReason reason, ModifierKeys modifiers)
 {
   assert(attribute!=NULL);
   assert(handle!=NULL);
   assert(to!=NULL);
 
   assert(handle->id < 8);
-  element_move_handle(&attribute->element, handle->id, to, reason);
+  element_move_handle(&attribute->element, handle->id, to, cp, 
+		      reason, modifiers);
   attribute_update_data(attribute);
+
+  return NULL;
 }
 
-static void
+static ObjectChange*
 attribute_move(Attribute *attribute, Point *to)
 {
   attribute->element.corner = *to;
   attribute_update_data(attribute);
+
+  return NULL;
 }
 
 static void
-attribute_draw(Attribute *attribute, Renderer *renderer)
+attribute_draw(Attribute *attribute, DiaRenderer *renderer)
 {
+  DiaRendererClass *renderer_ops = DIA_RENDERER_GET_CLASS (renderer);
   Point center;
   Point start, end;
   Point p;
@@ -254,25 +262,25 @@ attribute_draw(Attribute *attribute, Renderer *renderer)
   center.x = elem->corner.x + elem->width/2;
   center.y = elem->corner.y + elem->height/2;
   
-  renderer->ops->set_fillstyle(renderer, FILLSTYLE_SOLID);
-  renderer->ops->fill_ellipse(renderer, &center,
+  renderer_ops->set_fillstyle(renderer, FILLSTYLE_SOLID);
+  renderer_ops->fill_ellipse(renderer, &center,
 			      elem->width, elem->height,
 			      &attribute->inner_color);
 
-  renderer->ops->set_linewidth(renderer, attribute->border_width);
+  renderer_ops->set_linewidth(renderer, attribute->border_width);
   if (attribute->derived) {
-    renderer->ops->set_linestyle(renderer, LINESTYLE_DASHED);
-    renderer->ops->set_dashlength(renderer, 0.3);
+    renderer_ops->set_linestyle(renderer, LINESTYLE_DASHED);
+    renderer_ops->set_dashlength(renderer, 0.3);
   } else {
-    renderer->ops->set_linestyle(renderer, LINESTYLE_SOLID);
+    renderer_ops->set_linestyle(renderer, LINESTYLE_SOLID);
   }
 
-  renderer->ops->draw_ellipse(renderer, &center,
+  renderer_ops->draw_ellipse(renderer, &center,
 			      elem->width, elem->height,
 			      &attribute->border_color);
 
   if(attribute->multivalue) {
-    renderer->ops->draw_ellipse(renderer, &center,
+    renderer_ops->draw_ellipse(renderer, &center,
 				elem->width - 2*MULTIVALUE_BORDER_WIDTH_X,
 				elem->height - 2*MULTIVALUE_BORDER_WIDTH_Y,
 				&attribute->border_color);
@@ -280,26 +288,28 @@ attribute_draw(Attribute *attribute, Renderer *renderer)
 
   p.x = elem->corner.x + elem->width / 2.0;
   p.y = elem->corner.y + (elem->height - attribute->font_height)/2.0 +
-         font_ascent(attribute->font, attribute->font_height);
+         dia_font_ascent(attribute->name,
+                         attribute->font, attribute->font_height);
 
-  renderer->ops->set_font(renderer,  attribute->font, attribute->font_height);
-  renderer->ops->draw_string(renderer, attribute->name, 
+  renderer_ops->set_font(renderer,  attribute->font, attribute->font_height);
+  renderer_ops->draw_string(renderer, attribute->name, 
 			     &p, ALIGN_CENTER, 
 			     &color_black);
 
   if (attribute->key || attribute->weakkey) {
     if (attribute->weakkey) {
-      renderer->ops->set_linestyle(renderer, LINESTYLE_DASHED);
-      renderer->ops->set_dashlength(renderer, 0.3);
+      renderer_ops->set_linestyle(renderer, LINESTYLE_DASHED);
+      renderer_ops->set_dashlength(renderer, 0.3);
     } else {
-      renderer->ops->set_linestyle(renderer, LINESTYLE_SOLID);
+      renderer_ops->set_linestyle(renderer, LINESTYLE_SOLID);
     }
-    width = font_string_width(attribute->name, attribute->font, attribute->font_height);
+    width = dia_font_string_width(attribute->name,
+                                  attribute->font, attribute->font_height);
     start.x = center.x - width / 2;
     start.y = center.y + 0.4;
     end.x = center.x + width / 2;
     end.y = center.y + 0.4;
-    renderer->ops->draw_line(renderer, &start, &end, &color_black);
+    renderer_ops->draw_line(renderer, &start, &end, &color_black);
   }
 }
 
@@ -313,7 +323,8 @@ attribute_update_data(Attribute *attribute)
   real half_x, half_y;
 
   attribute->name_width =
-    font_string_width(attribute->name, attribute->font, attribute->font_height);
+    dia_font_string_width(attribute->name,
+                          attribute->font, attribute->font_height);
 
   elem->width = attribute->name_width + 2*TEXT_BORDER_WIDTH_X;
   elem->height = attribute->font_height + 2*TEXT_BORDER_WIDTH_Y;
@@ -325,22 +336,38 @@ attribute_update_data(Attribute *attribute)
   half_y = elem->height * M_SQRT1_2 / 2.0;
     
   /* Update connections: */
-  attribute->connections[0].pos.x = center.x - half_x;
-  attribute->connections[0].pos.y = center.y - half_y;
-  attribute->connections[1].pos.x = center.x;
-  attribute->connections[1].pos.y = elem->corner.y;
-  attribute->connections[2].pos.x = center.x + half_x;
-  attribute->connections[2].pos.y = center.y - half_y;
-  attribute->connections[3].pos.x = elem->corner.x;
-  attribute->connections[3].pos.y = center.y;
-  attribute->connections[4].pos.x = elem->corner.x + elem->width;
-  attribute->connections[4].pos.y = elem->corner.y + elem->height / 2.0;
-  attribute->connections[5].pos.x = center.x - half_x;
-  attribute->connections[5].pos.y = center.y + half_y;
-  attribute->connections[6].pos.x = elem->corner.x + elem->width / 2.0;
-  attribute->connections[6].pos.y = elem->corner.y + elem->height;
-  attribute->connections[7].pos.x = center.x + half_x;
-  attribute->connections[7].pos.y = center.y + half_y;
+  connpoint_update(&attribute->connections[0],
+		    center.x - half_x,
+		    center.y - half_y,
+		    DIR_NORTHWEST);
+  connpoint_update(&attribute->connections[1],
+		    center.x,
+		    elem->corner.y,
+		    DIR_NORTH);
+  connpoint_update(&attribute->connections[2],
+		    center.x + half_x,
+		    center.y - half_y,
+		    DIR_NORTHEAST);
+  connpoint_update(&attribute->connections[3],
+		    elem->corner.x,
+		    center.y,
+		    DIR_WEST);
+  connpoint_update(&attribute->connections[4],
+		    elem->corner.x + elem->width,
+		    elem->corner.y + elem->height / 2.0,
+		    DIR_EAST);
+  connpoint_update(&attribute->connections[5],
+		    center.x - half_x,
+		    center.y + half_y,
+		    DIR_SOUTHWEST);
+  connpoint_update(&attribute->connections[6],
+		    elem->corner.x + elem->width / 2.0,
+		    elem->corner.y + elem->height,
+		    DIR_SOUTH);
+  connpoint_update(&attribute->connections[7],
+		    center.x + half_x,
+		    center.y + half_y,
+		    DIR_SOUTHEAST);
 
   extra->border_trans = attribute->border_width/2.0;
   element_update_boundingbox(elem);
@@ -371,7 +398,7 @@ attribute_create(Point *startpoint,
 
   elem->corner = *startpoint;
   elem->width = DEFAULT_WIDTH;
-  elem->height = DEFAULT_WIDTH;
+  elem->height = DEFAULT_HEIGHT;
 
   attribute->border_width =  attributes_get_default_linewidth();
   attribute->border_color = attributes_get_foreground();
@@ -389,18 +416,13 @@ attribute_create(Point *startpoint,
   attribute->weakkey = FALSE;
   attribute->derived = FALSE;
   attribute->multivalue = FALSE;
-  /* choose default font name for your locale. see also font_data structure
-     in lib/font.c. if "Courier" works for you, it would be better.  */
-  attribute->font = font_getfont(_("Courier"));
+  attribute->font = dia_font_new_from_style(DIA_FONT_MONOSPACE,FONT_HEIGHT);
   attribute->font_height = FONT_HEIGHT;
-#ifdef GTK_DOESNT_TALK_UTF8_WE_DO
-  attribute->name = charconv_local8_to_utf8 (_("Attribute"));
-#else
   attribute->name = g_strdup(_("Attribute"));
-#endif
 
   attribute->name_width =
-    font_string_width(attribute->name, attribute->font, attribute->font_height);
+    dia_font_string_width(attribute->name,
+                          attribute->font, attribute->font_height);
 
   attribute_update_data(attribute);
 
@@ -447,10 +469,10 @@ attribute_copy(Attribute *attribute)
     newattribute->connections[i].pos = attribute->connections[i].pos;
     newattribute->connections[i].last_pos = attribute->connections[i].last_pos;
   }
-
-  newattribute->font = attribute->font;
+  
+  newattribute->font = dia_font_ref(attribute->font);
   newattribute->font_height = attribute->font_height;
-  newattribute->name = strdup(attribute->name);
+  newattribute->name = g_strdup(attribute->name);
   newattribute->name_width = attribute->name_width;
 
   newattribute->key = attribute->key;
@@ -544,7 +566,11 @@ static Object *attribute_load(ObjectNode obj_node, int version,
   if (attr != NULL)
     attribute->multivalue = data_boolean(attribute_first_data(attr));
 
-  attribute->font = NULL;
+  if (attribute->font != NULL) {
+    /* This shouldn't happen, but doesn't hurt */
+    dia_font_unref(attribute->font);
+    attribute->font = NULL;
+  }
   attr = object_find_attribute (obj_node, "font");
   if (attr != NULL)
 	  attribute->font = data_font (attribute_first_data (attr));
@@ -563,12 +589,13 @@ static Object *attribute_load(ObjectNode obj_node, int version,
   }
 
   if (attribute->font == NULL) {
-	  /* choose default font name for your locale. see also font_data structure
-	     in lib/font.c. if "Courier" works for you, it would be better.  */
-	  attribute->font = font_getfont(_("Courier"));
+	  attribute->font = dia_font_new_from_style(DIA_FONT_MONOSPACE,
+                                              attribute->font_height);
   }
 
-  attribute->name_width = font_string_width(attribute->name, attribute->font, attribute->font_height);
+  attribute->name_width = dia_font_string_width(attribute->name,
+                                                attribute->font,
+                                                attribute->font_height);
   attribute_update_data(attribute);
 
   for (i=0;i<8;i++) {
