@@ -16,12 +16,13 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+/* DO NOT USE THIS OBJECT AS A BASIS FOR A NEW OBJECT. */
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
 #include <assert.h>
-#include <gtk/gtk.h>
 #include <math.h>
 #include <string.h>
 
@@ -29,7 +30,7 @@
 #include "object.h"
 #include "element.h"
 #include "connectionpoint.h"
-#include "render.h"
+#include "diarenderer.h"
 #include "attributes.h"
 #include "widgets.h"
 #include "properties.h"
@@ -42,6 +43,7 @@
 #define TEXT_BORDER_WIDTH_Y 0.5
 #define WEAK_BORDER_WIDTH 0.25
 #define FONT_HEIGHT 0.8
+#define DIAMOND_RATIO 0.6
 
 typedef struct _Entity Entity;
 
@@ -54,7 +56,10 @@ struct _Entity {
   Color border_color;
   Color inner_color;
   
-  Font *font;
+  gboolean associative;
+  
+  DiaFont *font;
+  real font_height;
   char *name;
   real name_width;
   
@@ -64,11 +69,13 @@ struct _Entity {
 
 static real entity_distance_from(Entity *entity, Point *point);
 static void entity_select(Entity *entity, Point *clicked_point,
-		       Renderer *interactive_renderer);
-static void entity_move_handle(Entity *entity, Handle *handle,
-			    Point *to, HandleMoveReason reason, ModifierKeys modifiers);
-static void entity_move(Entity *entity, Point *to);
-static void entity_draw(Entity *entity, Renderer *renderer);
+		       DiaRenderer *interactive_renderer);
+static ObjectChange* entity_move_handle(Entity *entity, Handle *handle,
+					Point *to, ConnectionPoint *cp,
+					HandleMoveReason reason,
+					ModifierKeys modifiers);
+static ObjectChange* entity_move(Entity *entity, Point *to);
+static void entity_draw(Entity *entity, DiaRenderer *renderer);
 static void entity_update_data(Entity *entity);
 static Object *entity_create(Point *startpoint,
 			     void *user_data,
@@ -78,10 +85,8 @@ static void entity_destroy(Entity *entity);
 static Object *entity_copy(Entity *entity);
 static PropDescription *
 entity_describe_props(Entity *entity);
-static void
-entity_get_props(Entity *entity, Property *props, guint nprops);
-static void
-entity_set_props(Entity *entity, Property *props, guint nprops);
+static void entity_get_props(Entity *entity, GPtrArray *props);
+static void entity_set_props(Entity *entity, GPtrArray *props);
 
 static void entity_save(Entity *entity, ObjectNode obj_node,
 			const char *filename);
@@ -128,9 +133,13 @@ static PropDescription entity_props[] = {
     N_("Name:"), NULL, NULL },
   { "weak", PROP_TYPE_BOOL, PROP_FLAG_VISIBLE,
     N_("Weak:"), NULL, NULL },
+  { "associative", PROP_TYPE_BOOL, PROP_FLAG_VISIBLE,
+    N_("Associative:"), NULL, NULL },
   PROP_STD_LINE_WIDTH,
   PROP_STD_LINE_COLOUR,
   PROP_STD_FILL_COLOUR,
+  PROP_STD_TEXT_FONT,
+  PROP_STD_TEXT_HEIGHT,
   PROP_DESC_END
 };
 
@@ -146,25 +155,28 @@ static PropOffset entity_offsets[] = {
   ELEMENT_COMMON_PROPERTIES_OFFSETS,
   { "name", PROP_TYPE_STRING, offsetof(Entity, name) },
   { "weak", PROP_TYPE_BOOL, offsetof(Entity, weak) },
+  { "associative", PROP_TYPE_BOOL, offsetof(Entity, associative) },
   { "line_width", PROP_TYPE_REAL, offsetof(Entity, border_width) },
   { "line_colour", PROP_TYPE_COLOUR, offsetof(Entity, border_color) },
   { "fill_colour", PROP_TYPE_COLOUR, offsetof(Entity, inner_color) },
+  { "text_font", PROP_TYPE_FONT, offsetof (Entity, font) },
+  { "text_height", PROP_TYPE_REAL, offsetof(Entity, font_height) },
   { NULL, 0, 0}
 };
 
 
 static void
-entity_get_props(Entity *entity, Property *props, guint nprops)
+entity_get_props(Entity *entity, GPtrArray *props)
 {
   object_get_props_from_offsets(&entity->element.object, 
-                                entity_offsets, props, nprops);
+                                entity_offsets, props);
 }
 
 static void
-entity_set_props(Entity *entity, Property *props, guint nprops)
+entity_set_props(Entity *entity, GPtrArray *props)
 {
   object_set_props_from_offsets(&entity->element.object, 
-                                entity_offsets, props, nprops);
+                                entity_offsets, props);
   entity_update_data(entity);
 }
 
@@ -183,35 +195,41 @@ entity_distance_from(Entity *entity, Point *point)
 
 static void
 entity_select(Entity *entity, Point *clicked_point,
-	   Renderer *interactive_renderer)
+	   DiaRenderer *interactive_renderer)
 {
   element_update_handles(&entity->element);
 }
 
-static void
+static ObjectChange*
 entity_move_handle(Entity *entity, Handle *handle,
-		Point *to, HandleMoveReason reason, ModifierKeys modifiers)
+		   Point *to, ConnectionPoint *cp,
+		   HandleMoveReason reason, ModifierKeys modifiers)
 {
   assert(entity!=NULL);
   assert(handle!=NULL);
   assert(to!=NULL);
 
-  element_move_handle(&entity->element, handle->id, to, reason);
+  element_move_handle(&entity->element, handle->id, to, cp, reason, modifiers);
 
   entity_update_data(entity);
+
+  return NULL;
 }
 
-static void
+static ObjectChange*
 entity_move(Entity *entity, Point *to)
 {
   entity->element.corner = *to;
   
   entity_update_data(entity);
+
+  return NULL;
 }
 
 static void
-entity_draw(Entity *entity, Renderer *renderer)
+entity_draw(Entity *entity, DiaRenderer *renderer)
 {
+  DiaRendererClass *renderer_ops = DIA_RENDERER_GET_CLASS (renderer);
   Point ul_corner, lr_corner;
   Point p;
   Element *elem;
@@ -227,18 +245,18 @@ entity_draw(Entity *entity, Renderer *renderer)
   lr_corner.x = elem->corner.x + elem->width;
   lr_corner.y = elem->corner.y + elem->height;
   
-  renderer->ops->set_fillstyle(renderer, FILLSTYLE_SOLID);
+  renderer_ops->set_fillstyle(renderer, FILLSTYLE_SOLID);
   
-  renderer->ops->fill_rect(renderer, 
+  renderer_ops->fill_rect(renderer, 
 			   &ul_corner,
 			   &lr_corner, 
 			   &entity->inner_color);
 
-  renderer->ops->set_linewidth(renderer, entity->border_width);
-  renderer->ops->set_linestyle(renderer, LINESTYLE_SOLID);
-  renderer->ops->set_linejoin(renderer, LINEJOIN_MITER);
+  renderer_ops->set_linewidth(renderer, entity->border_width);
+  renderer_ops->set_linestyle(renderer, LINESTYLE_SOLID);
+  renderer_ops->set_linejoin(renderer, LINEJOIN_MITER);
 
-  renderer->ops->draw_rect(renderer, 
+  renderer_ops->draw_rect(renderer, 
 			   &ul_corner,
 			   &lr_corner, 
 			   &entity->border_color);
@@ -249,19 +267,41 @@ entity_draw(Entity *entity, Renderer *renderer)
     ul_corner.y += diff;
     lr_corner.x -= diff;
     lr_corner.y -= diff;
-    renderer->ops->draw_rect(renderer, 
+    renderer_ops->draw_rect(renderer, 
 			     &ul_corner, &lr_corner,
 			     &entity->border_color);
   }
+  if(entity->associative){
+    Point corners[4];
+    corners[0].x = elem->corner.x;
+    corners[0].y = elem->corner.y + elem->height / 2;
+    corners[1].x = elem->corner.x + elem->width / 2;
+    corners[1].y = elem->corner.y;
+    corners[2].x = elem->corner.x + elem->width;
+    corners[2].y = elem->corner.y + elem->height / 2;
+    corners[3].x = elem->corner.x + elem->width / 2;
+    corners[3].y = elem->corner.y + elem->height;
+    renderer_ops->set_fillstyle(renderer, FILLSTYLE_SOLID);
+
+    renderer_ops->fill_polygon(renderer, corners, 4, 
+			       &entity->inner_color);
+
+    renderer_ops->set_linewidth(renderer, entity->border_width);
+    renderer_ops->set_linestyle(renderer, LINESTYLE_SOLID);
+    renderer_ops->set_linejoin(renderer, LINEJOIN_MITER);
+
+    renderer_ops->draw_polygon(renderer, corners, 4, 
+			       &entity->border_color);
+  }
 
   p.x = elem->corner.x + elem->width / 2.0;
-  p.y = elem->corner.y + (elem->height - FONT_HEIGHT)/2.0 + font_ascent(entity->font, FONT_HEIGHT);
-  renderer->ops->set_font(renderer, 
-			  entity->font, FONT_HEIGHT);
-  renderer->ops->draw_string(renderer, 
-			     entity->name, 
-			     &p, ALIGN_CENTER, 
-			     &color_black);
+  p.y = elem->corner.y + (elem->height - entity->font_height)/2.0 +
+      dia_font_ascent(entity->name,entity->font, entity->font_height);
+  renderer_ops->set_font(renderer, entity->font, entity->font_height);
+  renderer_ops->draw_string(renderer, 
+                             entity->name, 
+                             &p, ALIGN_CENTER, 
+                             &color_black);
 }
 
 static void
@@ -272,27 +312,50 @@ entity_update_data(Entity *entity)
   ElementBBExtras *extra = &elem->extra_spacing;
 
   entity->name_width =
-    font_string_width(entity->name, entity->font, FONT_HEIGHT);
+    dia_font_string_width(entity->name, entity->font, entity->font_height);
 
+  if(entity->associative){
+    elem->width = entity->name_width + 2*TEXT_BORDER_WIDTH_X;
+    elem->height = elem->width * DIAMOND_RATIO;
+  }
+  else {
   elem->width = entity->name_width + 2*TEXT_BORDER_WIDTH_X;
-  elem->height = FONT_HEIGHT + 2*TEXT_BORDER_WIDTH_Y;
+  elem->height = entity->font_height + 2*TEXT_BORDER_WIDTH_Y;
+  }
 
   /* Update connections: */
-  entity->connections[0].pos = elem->corner;
-  entity->connections[1].pos.x = elem->corner.x + elem->width / 2.0;
-  entity->connections[1].pos.y = elem->corner.y;
-  entity->connections[2].pos.x = elem->corner.x + elem->width;
-  entity->connections[2].pos.y = elem->corner.y;
-  entity->connections[3].pos.x = elem->corner.x;
-  entity->connections[3].pos.y = elem->corner.y + elem->height / 2.0;
-  entity->connections[4].pos.x = elem->corner.x + elem->width;
-  entity->connections[4].pos.y = elem->corner.y + elem->height / 2.0;
-  entity->connections[5].pos.x = elem->corner.x;
-  entity->connections[5].pos.y = elem->corner.y + elem->height;
-  entity->connections[6].pos.x = elem->corner.x + elem->width / 2.0;
-  entity->connections[6].pos.y = elem->corner.y + elem->height;
-  entity->connections[7].pos.x = elem->corner.x + elem->width;
-  entity->connections[7].pos.y = elem->corner.y + elem->height;
+  connpoint_update(&entity->connections[0],
+		    elem->corner.x,
+		    elem->corner.y,
+		    DIR_NORTHWEST);
+  connpoint_update(&entity->connections[1],
+		   elem->corner.x + elem->width / 2.0,
+		   elem->corner.y,
+		   DIR_NORTH);
+  connpoint_update(&entity->connections[2],
+		   elem->corner.x + elem->width,
+		   elem->corner.y,
+		   DIR_NORTHEAST);
+  connpoint_update(&entity->connections[3],
+		   elem->corner.x,
+		   elem->corner.y + elem->height / 2.0,
+		   DIR_WEST);
+  connpoint_update(&entity->connections[4],
+		   elem->corner.x + elem->width,
+		   elem->corner.y + elem->height / 2.0,
+		   DIR_EAST);
+  connpoint_update(&entity->connections[5],
+		   elem->corner.x,
+		   elem->corner.y + elem->height,
+		   DIR_SOUTHWEST);
+  connpoint_update(&entity->connections[6],
+		   elem->corner.x + elem->width / 2.0,
+		   elem->corner.y + elem->height,
+		   DIR_SOUTH);
+  connpoint_update(&entity->connections[7],
+		   elem->corner.x + elem->width,
+		   elem->corner.y + elem->height,
+		   DIR_SOUTHEAST);
 
   extra->border_trans = entity->border_width/2.0;
   element_update_boundingbox(elem);
@@ -323,7 +386,7 @@ entity_create(Point *startpoint,
 
   elem->corner = *startpoint;
   elem->width = DEFAULT_WIDTH;
-  elem->height = DEFAULT_WIDTH;
+  elem->height = DEFAULT_HEIGHT;
 
   entity->border_width =  attributes_get_default_linewidth();
   entity->border_color = attributes_get_foreground();
@@ -338,11 +401,12 @@ entity_create(Point *startpoint,
   }
 
   entity->weak = GPOINTER_TO_INT(user_data);
-  entity->font = font_getfont("Courier");
+  entity->font = dia_font_new_from_style(DIA_FONT_MONOSPACE,FONT_HEIGHT);
+  entity->font_height = FONT_HEIGHT;
   entity->name = g_strdup(_("Entity"));
 
   entity->name_width =
-    font_string_width(entity->name, entity->font, FONT_HEIGHT);
+    dia_font_string_width(entity->name, entity->font, entity->font_height);
   
   entity_update_data(entity);
 
@@ -358,6 +422,7 @@ entity_create(Point *startpoint,
 static void
 entity_destroy(Entity *entity)
 {
+  dia_font_unref(entity->font);
   element_destroy(&entity->element);
   g_free(entity->name);
 }
@@ -390,8 +455,9 @@ entity_copy(Entity *entity)
     newentity->connections[i].last_pos = entity->connections[i].last_pos;
   }
 
-  newentity->font = entity->font;
-  newentity->name = strdup(entity->name);
+  newentity->font = dia_font_ref(entity->font);
+  newentity->font_height = entity->font_height;
+  newentity->name = g_strdup(entity->name);
   newentity->name_width = entity->name_width;
 
   newentity->weak = entity->weak;
@@ -414,6 +480,12 @@ entity_save(Entity *entity, ObjectNode obj_node, const char *filename)
 		  entity->name);
   data_add_boolean(new_attribute(obj_node, "weak"),
 		   entity->weak);
+  data_add_boolean(new_attribute(obj_node, "associative"),
+		   entity->associative);
+  data_add_font (new_attribute (obj_node, "font"),
+		 entity->font);
+  data_add_real(new_attribute(obj_node, "font_height"),
+  		entity->font_height);
 }
 
 static Object *
@@ -458,6 +530,24 @@ entity_load(ObjectNode obj_node, int version, const char *filename)
   if (attr != NULL)
     entity->weak = data_boolean(attribute_first_data(attr));
 
+  attr = object_find_attribute(obj_node, "associative");
+  if (attr != NULL)
+    entity->associative = data_boolean(attribute_first_data(attr));
+
+  if (entity->font != NULL) {
+    /* This shouldn't happen, but doesn't hurt */
+    dia_font_unref(entity->font);
+    entity->font = NULL;
+  }
+  attr = object_find_attribute (obj_node, "font");
+  if (attr != NULL)
+    entity->font = data_font (attribute_first_data (attr));
+
+  entity->font_height = FONT_HEIGHT;
+  attr = object_find_attribute(obj_node, "font_height");
+  if (attr != NULL)
+    entity->font_height = data_real(attribute_first_data(attr));
+
   element_init(elem, 8, 8);
 
   for (i=0;i<8;i++) {
@@ -466,10 +556,12 @@ entity_load(ObjectNode obj_node, int version, const char *filename)
     entity->connections[i].connected = NULL;
   }
 
-  entity->font = font_getfont("Courier");
+  if (entity->font == NULL) {
+    entity->font = dia_font_new_from_style(DIA_FONT_MONOSPACE,1.0);
+  }
 
   entity->name_width =
-    font_string_width(entity->name, entity->font, FONT_HEIGHT);
+    dia_font_string_width(entity->name, entity->font, entity->font_height);
 
   entity_update_data(entity);
 
