@@ -24,14 +24,13 @@
 #endif
 
 #include <assert.h>
-#include <gtk/gtk.h>
 #include <math.h>
 
 #include "intl.h"
 #include "object.h"
 #include "connection.h"
 #include "connectionpoint.h"
-#include "render.h"
+#include "diarenderer.h"
 #include "attributes.h"
 #include "widgets.h"
 #include "message.h"
@@ -46,9 +45,8 @@
 #include "pixmaps/condition.xpm"
 
 #define CONDITION_LINE_WIDTH GRAFCET_GENERAL_LINE_WIDTH
-/* choose default font name for your locale. see also font_data structure
-   in lib/font.c. */
-#define CONDITION_FONT N_("Helvetica-Bold")
+#define CONDITION_FONT BASIC_SANS_FONT
+#define CONDITION_FONT_STYLE STYLE_BOLD
 #define CONDITION_FONT_HEIGHT 0.8
 #define CONDITION_LENGTH (1.5)
 #define CONDITION_ARROW_SIZE 0.0 /* XXX The norm says there's no arrow head.
@@ -68,12 +66,13 @@ typedef struct _Condition {
   Rectangle labelbb;
 } Condition;
 
-static void condition_move_handle(Condition *condition, Handle *handle,
-				   Point *to, HandleMoveReason reason, ModifierKeys modifiers);
-static void condition_move(Condition *condition, Point *to);
+static ObjectChange* condition_move_handle(Condition *condition, Handle *handle,
+					   Point *to, ConnectionPoint *cp,
+					   HandleMoveReason reason, ModifierKeys modifiers);
+static ObjectChange* condition_move(Condition *condition, Point *to);
 static void condition_select(Condition *condition, Point *clicked_point,
-			      Renderer *interactive_renderer);
-static void condition_draw(Condition *condition, Renderer *renderer);
+			      DiaRenderer *interactive_renderer);
+static void condition_draw(Condition *condition, DiaRenderer *renderer);
 static Object *condition_create(Point *startpoint,
 				 void *user_data,
 				 Handle **handle1,
@@ -171,7 +170,8 @@ condition_set_props(Condition *condition, GPtrArray *props)
                                 condition_offsets,props);
   
   boolequation_set_value(condition->cond,condition->cond_value);
-  condition->cond->font = condition->cond_font;
+  dia_font_unref(condition->cond->font);
+  condition->cond->font = dia_font_ref(condition->cond_font);
   condition->cond->fontheight = condition->cond_fontheight;
   condition->cond->color = condition->cond_color;
   condition_update_data(condition);
@@ -191,14 +191,15 @@ condition_distance_from(Condition *condition, Point *point)
 
 static void
 condition_select(Condition *condition, Point *clicked_point,
-		  Renderer *interactive_renderer)
+		  DiaRenderer *interactive_renderer)
 {
   condition_update_data(condition);
 }
 
-static void
+static ObjectChange*
 condition_move_handle(Condition *condition, Handle *handle,
-		       Point *to, HandleMoveReason reason, ModifierKeys modifiers)
+		       Point *to, ConnectionPoint *cp,
+		      HandleMoveReason reason, ModifierKeys modifiers)
 {
   Point s,e,v;
   int horiz;
@@ -225,7 +226,7 @@ condition_move_handle(Condition *condition, Handle *handle,
     point_sub(&s,&v);
     /* XXX: fix e to make it look good (what's good ?) V is a good hint ? */
     connection_move_handle(&condition->connection, HANDLE_MOVE_STARTPOINT,
-			   &s,reason);    
+			   &s, cp, reason, modifiers);    
     break;
   case HANDLE_MOVE_ENDPOINT:
     point_copy(&s,&condition->connection.endpoints[0]);
@@ -233,20 +234,22 @@ condition_move_handle(Condition *condition, Handle *handle,
     point_copy(&v,&e);
     point_sub(&v,&s);
     connection_move_handle(&condition->connection, HANDLE_MOVE_ENDPOINT,
-			   to,reason);
+			   to, cp, reason, modifiers);
     point_copy(&s,to);
     point_sub(&s,&v);
     connection_move_handle(&condition->connection, HANDLE_MOVE_STARTPOINT,
-			   &s,reason);    
+			   &s, NULL, reason, 0);    
     break;
   default:
     g_assert_not_reached();
   }
   condition_update_data(condition);
+
+  return NULL;
 }
 
 
-static void
+static ObjectChange*
 condition_move(Condition *condition, Point *to)
 {
   Point start_to_end;
@@ -259,6 +262,8 @@ condition_move(Condition *condition, Point *to)
   point_add(&endpoints[1], &start_to_end);
 
   condition_update_data(condition);
+
+  return NULL;
 }
 
 static void
@@ -272,7 +277,7 @@ condition_update_data(Condition *condition)
 
   /* compute the label's width and bounding box */
   condition->cond->pos.x = conn->endpoints[0].x + 
-    (.5 * font_string_width("a", condition->cond->font,
+    (.5 * dia_font_string_width("a", condition->cond->font,
 			    condition->cond->fontheight));
   condition->cond->pos.y = conn->endpoints[0].y + condition->cond->fontheight;
   
@@ -284,23 +289,29 @@ condition_update_data(Condition *condition)
 
 
 static void 
-condition_draw(Condition *condition, Renderer *renderer)
+condition_draw(Condition *condition, DiaRenderer *renderer)
 {
+  DiaRendererClass *renderer_ops = DIA_RENDERER_GET_CLASS (renderer);
   Connection *conn = &condition->connection;
 
-  renderer->ops->set_linewidth(renderer, CONDITION_LINE_WIDTH);
-  renderer->ops->set_linestyle(renderer, LINESTYLE_SOLID);
-  renderer->ops->set_linecaps(renderer, LINECAPS_BUTT);
+  renderer_ops->set_linewidth(renderer, CONDITION_LINE_WIDTH);
+  renderer_ops->set_linestyle(renderer, LINESTYLE_SOLID);
+  renderer_ops->set_linecaps(renderer, LINECAPS_BUTT);
 
-  renderer->ops->draw_line(renderer,
-			   &conn->endpoints[0],&conn->endpoints[1],
-			   &color_black);
   if (CONDITION_ARROW_SIZE > (CONDITION_LINE_WIDTH/2.0)) {
-    arrow_draw(renderer, ARROW_FILLED_TRIANGLE,
-               &conn->endpoints[1],&conn->endpoints[0],
-               CONDITION_ARROW_SIZE,CONDITION_ARROW_SIZE/2,
-               CONDITION_LINE_WIDTH,
-               &color_black,&color_white);
+    Arrow arrow;
+    arrow.type = ARROW_FILLED_TRIANGLE;
+    arrow.width = CONDITION_ARROW_SIZE;
+    arrow.length = CONDITION_ARROW_SIZE/2;
+    renderer_ops->draw_line_with_arrows(renderer,
+					 &conn->endpoints[0],&conn->endpoints[1],
+					 CONDITION_LINE_WIDTH,
+					 &color_black,
+					 &arrow, NULL);
+  } else {
+    renderer_ops->draw_line(renderer,
+			     &conn->endpoints[0],&conn->endpoints[1],
+			     &color_black);
   }
 
   boolequation_draw(condition->cond,renderer);
@@ -338,13 +349,14 @@ condition_create(Point *startpoint,
 
   pos = conn->endpoints[1];
 
+  default_font = NULL;
   attributes_get_default_font(&default_font,&default_fontheight);
   fg_color = attributes_get_foreground();
   
   condition->cond = boolequation_create("",default_font,default_fontheight,
 				 &fg_color);
   condition->cond_value = g_strdup("");
-  condition->cond_font = default_font;
+  condition->cond_font = dia_font_ref(default_font);
   condition->cond_fontheight = default_fontheight;
   condition->cond_color = fg_color;
 
@@ -359,12 +371,15 @@ condition_create(Point *startpoint,
   *handle1 = &conn->endpoint_handles[0];
   *handle2 = &conn->endpoint_handles[1];
 
+  dia_font_unref(default_font);
+
   return &condition->connection.object;
 }
 
 static void
 condition_destroy(Condition *condition)
 {
+  dia_font_unref(condition->cond_font);  
   boolequation_destroy(condition->cond);
   g_free(condition->cond_value);
   connection_destroy(&condition->connection);
