@@ -38,8 +38,6 @@
 #include "pixmaps/activity.xpm"
 
 typedef struct _State State;
-//typedef struct _StateState StateState;
-
 
 struct _State {
   Element element;
@@ -61,9 +59,10 @@ struct _State {
 static real state_distance_from(State *state, Point *point);
 static void state_select(State *state, Point *clicked_point,
 			DiaRenderer *interactive_renderer);
-static void state_move_handle(State *state, Handle *handle,
-			     Point *to, HandleMoveReason reason, ModifierKeys modifiers);
-static void state_move(State *state, Point *to);
+static ObjectChange* state_move_handle(State *state, Handle *handle,
+				       Point *to, ConnectionPoint *cp,
+				       HandleMoveReason reason, ModifierKeys modifiers);
+static ObjectChange* state_move(State *state, Point *to);
 static void state_draw(State *state, DiaRenderer *renderer);
 static Object *state_create_activity(Point *startpoint,
 			   void *user_data,
@@ -128,6 +127,9 @@ static PropDescription activity_props[] = {
 static PropDescription *
 state_describe_props(State *state)
 {
+  if (activity_props[0].quark == 0) {
+    prop_desc_list_calculate_quarks(activity_props);
+  }
    return activity_props;
 }
 
@@ -173,22 +175,27 @@ state_select(State *state, Point *clicked_point,
   element_update_handles(&state->element);
 }
 
-static void
+static ObjectChange*
 state_move_handle(State *state, Handle *handle,
-		 Point *to, HandleMoveReason reason, ModifierKeys modifiers)
+		  Point *to, ConnectionPoint *cp,
+		  HandleMoveReason reason, ModifierKeys modifiers)
 {
   assert(state!=NULL);
   assert(handle!=NULL);
   assert(to!=NULL);
 
   assert(handle->id < 8);
+
+  return NULL;
 }
 
-static void
+static ObjectChange*
 state_move(State *state, Point *to)
 {
   state->element.corner = *to;
   state_update_data(state);
+
+  return NULL;
 }
 
 static void
@@ -263,6 +270,15 @@ state_update_data(State *state)
   state->connections[7].pos.x = elem->corner.x + elem->width;
   state->connections[7].pos.y = elem->corner.y + elem->height;
   
+  state->connections[0].directions = DIR_NORTH|DIR_WEST;
+  state->connections[1].directions = DIR_NORTH;
+  state->connections[2].directions = DIR_NORTH|DIR_EAST;
+  state->connections[3].directions = DIR_WEST;
+  state->connections[4].directions = DIR_EAST;
+  state->connections[5].directions = DIR_SOUTH|DIR_WEST;
+  state->connections[6].directions = DIR_SOUTH;
+  state->connections[7].directions = DIR_SOUTH|DIR_EAST;
+                                                                                          
   element_update_boundingbox(elem);
 
   obj->position = elem->corner;
