@@ -21,14 +21,13 @@
 #endif
 
 #include <assert.h>
-#include <gtk/gtk.h>
 #include <math.h>
 #include <string.h>
 
 #include "intl.h"
 #include "object.h"
 #include "orth_conn.h"
-#include "render.h"
+#include "diarenderer.h"
 #include "attributes.h"
 #include "arrows.h"
 #include "properties.h"
@@ -47,39 +46,40 @@ struct _Generalization {
   Alignment text_align;
   real text_width;
   
-  char *name;
-  char *stereotype; /* including << and >> */
+  Color text_color;
+  Color line_color;
 
+  char *name;
+  char *stereotype; /* excluding << and >> */
+  char *st_stereotype; /* including << and >> */
 };
 
 #define GENERALIZATION_WIDTH 0.1
 #define GENERALIZATION_TRIANGLESIZE 0.8
 #define GENERALIZATION_FONTHEIGHT 0.8
 
-static Font *genlz_font = NULL;
+static DiaFont *genlz_font = NULL;
 
 static real generalization_distance_from(Generalization *genlz, Point *point);
 static void generalization_select(Generalization *genlz, Point *clicked_point,
-			      Renderer *interactive_renderer);
-static void generalization_move_handle(Generalization *genlz, Handle *handle,
-				   Point *to, HandleMoveReason reason, ModifierKeys modifiers);
-static void generalization_move(Generalization *genlz, Point *to);
-static void generalization_draw(Generalization *genlz, Renderer *renderer);
+			      DiaRenderer *interactive_renderer);
+static ObjectChange* generalization_move_handle(Generalization *genlz, Handle *handle,
+						Point *to, ConnectionPoint *cp,
+						HandleMoveReason reason, ModifierKeys modifiers);
+static ObjectChange* generalization_move(Generalization *genlz, Point *to);
+static void generalization_draw(Generalization *genlz, DiaRenderer *renderer);
 static Object *generalization_create(Point *startpoint,
 				 void *user_data,
 				 Handle **handle1,
 				 Handle **handle2);
 static void generalization_destroy(Generalization *genlz);
-static Object *generalization_copy(Generalization *genlz);
 static DiaMenu *generalization_get_object_menu(Generalization *genlz,
 						Point *clickedpoint);
 
 static PropDescription *generalization_describe_props(Generalization *generalization);
-static void generalization_get_props(Generalization * generalization, Property *props, guint nprops);
-static void generalization_set_props(Generalization * generalization, Property *props, guint nprops);
+static void generalization_get_props(Generalization * generalization, GPtrArray *props);
+static void generalization_set_props(Generalization * generalization, GPtrArray *props);
 
-static void generalization_save(Generalization *genlz, ObjectNode obj_node,
-				const char *filename);
 static Object *generalization_load(ObjectNode obj_node, int version,
 				   const char *filename);
 
@@ -88,8 +88,10 @@ static void generalization_update_data(Generalization *genlz);
 static ObjectTypeOps generalization_type_ops =
 {
   (CreateFunc) generalization_create,
-  (LoadFunc)   generalization_load,
-  (SaveFunc)   generalization_save
+  (LoadFunc)   generalization_load,/*using_properties*/     /* load */
+  (SaveFunc)   object_save_using_properties,      /* save */
+  (GetDefaultsFunc)   NULL, 
+  (ApplyDefaultsFunc) NULL
 };
 
 ObjectType generalization_type =
@@ -98,7 +100,9 @@ ObjectType generalization_type =
   0,                      /* version */
   (char **) generalization_xpm,  /* pixmap */
   
-  &generalization_type_ops       /* ops */
+  &generalization_type_ops,      /* ops */
+  NULL,                 /* pixmap_file */
+  0                     /* default_user_data */
 };
 
 static ObjectOps generalization_ops = {
@@ -106,7 +110,7 @@ static ObjectOps generalization_ops = {
   (DrawFunc)            generalization_draw,
   (DistanceFunc)        generalization_distance_from,
   (SelectFunc)          generalization_select,
-  (CopyFunc)            generalization_copy,
+  (CopyFunc)            object_copy_using_properties,
   (MoveFunc)            generalization_move,
   (MoveHandleFunc)      generalization_move_handle,
   (GetPropertiesFunc)   object_create_props_dialog,
@@ -118,7 +122,9 @@ static ObjectOps generalization_ops = {
 };
 
 static PropDescription generalization_props[] = {
-  OBJECT_COMMON_PROPERTIES,
+  ORTHCONN_COMMON_PROPERTIES,
+  PROP_STD_TEXT_COLOUR_OPTIONAL,
+  PROP_STD_LINE_COLOUR_OPTIONAL, 
   { "name", PROP_TYPE_STRING, PROP_FLAG_VISIBLE,
     N_("Name:"), NULL, NULL },
   { "stereotype", PROP_TYPE_STRING, PROP_FLAG_VISIBLE,
@@ -129,75 +135,35 @@ static PropDescription generalization_props[] = {
 static PropDescription *
 generalization_describe_props(Generalization *generalization)
 {
-  if (generalization_props[0].quark == 0)
+  if (generalization_props[0].quark == 0) {
     prop_desc_list_calculate_quarks(generalization_props);
+  }
   return generalization_props;
 }
 
 static PropOffset generalization_offsets[] = {
-  OBJECT_COMMON_PROPERTIES_OFFSETS,
+  ORTHCONN_COMMON_PROPERTIES_OFFSETS,
+  {"text_colour",PROP_TYPE_COLOUR,offsetof(Generalization,text_color)},
+  {"line_colour",PROP_TYPE_COLOUR,offsetof(Generalization,line_color)},
   { "name", PROP_TYPE_STRING, offsetof(Generalization, name) },
-  /*{ "stereotype", PROP_TYPE_STRING, offsetof(Generalization, stereotype) },*/
+  { "stereotype", PROP_TYPE_STRING, offsetof(Generalization, stereotype) },
   { NULL, 0, 0 }
 };
 
-static struct { const gchar *name; GQuark q; } quarks[] = {
-  { "stereotype" }
-};
-
 static void
-generalization_get_props(Generalization * generalization, Property *props, guint nprops)
+generalization_get_props(Generalization * generalization, GPtrArray *props)
 {
-  guint i;
-
-  if (object_get_props_from_offsets(&generalization->orth.object, 
-                                    generalization_offsets, props, nprops))
-    return;
-  /* these props can't be handled as easily */
-  if (quarks[0].q == 0)
-    for (i = 0; i < sizeof(quarks)/sizeof(*quarks); i++)
-      quarks[i].q = g_quark_from_static_string(quarks[i].name);
-  for (i = 0; i < nprops; i++) {
-    GQuark pquark = g_quark_from_string(props[i].name);
-
-    if (pquark == quarks[0].q) {
-      props[i].type = PROP_TYPE_STRING;
-      g_free(PROP_VALUE_STRING(props[i]));
-      if (generalization->stereotype != NULL &&
-	  generalization->stereotype[0] != '\0')
-	PROP_VALUE_STRING(props[i]) =
-	  stereotype_to_string(generalization->stereotype);
-      else
-	PROP_VALUE_STRING(props[i]) = NULL;
-    }
-  }
-
+  object_get_props_from_offsets(&generalization->orth.object,
+                                generalization_offsets,props);
 }
 
 static void
-generalization_set_props(Generalization *generalization, Property *props, guint nprops)
+generalization_set_props(Generalization *generalization, GPtrArray *props)
 {
-  if (!object_set_props_from_offsets(&generalization->orth.object, 
-                                     generalization_offsets, props, nprops)) {
-    guint i;
-
-    if (quarks[0].q == 0)
-      for (i = 0; i < sizeof(quarks)/sizeof(*quarks); i++)
-	quarks[i].q = g_quark_from_static_string(quarks[i].name);
-    
-    for (i = 0; i < nprops; i++) {
-      GQuark pquark = g_quark_from_string(props[i].name);
-      if (pquark == quarks[0].q && props[i].type == PROP_TYPE_STRING) {
-	g_free(generalization->stereotype);
-	if (PROP_VALUE_STRING(props[i]) != NULL &&
-	    PROP_VALUE_STRING(props[i])[0] != '\0')
-	  generalization->stereotype =
-	    string_to_stereotype(PROP_VALUE_STRING(props[i]));
-	else 
-	  generalization->stereotype = NULL;
-      }
-    }
-  }
+  object_set_props_from_offsets(&generalization->orth.object, 
+                                generalization_offsets, props);
+  g_free(generalization->st_stereotype);
+  generalization->st_stereotype = NULL;
   generalization_update_data(generalization);
 }
 
@@ -210,71 +176,83 @@ generalization_distance_from(Generalization *genlz, Point *point)
 
 static void
 generalization_select(Generalization *genlz, Point *clicked_point,
-		  Renderer *interactive_renderer)
+		  DiaRenderer *interactive_renderer)
 {
   orthconn_update_data(&genlz->orth);
 }
 
-static void
+static ObjectChange*
 generalization_move_handle(Generalization *genlz, Handle *handle,
-		       Point *to, HandleMoveReason reason, ModifierKeys modifiers)
+			   Point *to, ConnectionPoint *cp,
+			   HandleMoveReason reason, ModifierKeys modifiers)
 {
+  ObjectChange *change;
   assert(genlz!=NULL);
   assert(handle!=NULL);
   assert(to!=NULL);
   
-  orthconn_move_handle(&genlz->orth, handle, to, reason);
+  change = orthconn_move_handle(&genlz->orth, handle, to, cp, reason, modifiers);
   generalization_update_data(genlz);
+
+  return change;
 }
 
-static void
+static ObjectChange*
 generalization_move(Generalization *genlz, Point *to)
 {
-  orthconn_move(&genlz->orth, to);
+  ObjectChange *change;
+
+  change = orthconn_move(&genlz->orth, to);
   generalization_update_data(genlz);
+
+  return change;
 }
 
 static void
-generalization_draw(Generalization *genlz, Renderer *renderer)
+generalization_draw(Generalization *genlz, DiaRenderer *renderer)
 {
+  DiaRendererClass *renderer_ops = DIA_RENDERER_GET_CLASS (renderer);
   OrthConn *orth = &genlz->orth;
   Point *points;
   int n;
   Point pos;
+  Arrow arrow;
   
   points = &orth->points[0];
   n = orth->numpoints;
   
-  renderer->ops->set_linewidth(renderer, GENERALIZATION_WIDTH);
-  renderer->ops->set_linestyle(renderer, LINESTYLE_SOLID);
-  renderer->ops->set_linejoin(renderer, LINEJOIN_MITER);
-  renderer->ops->set_linecaps(renderer, LINECAPS_BUTT);
+  renderer_ops->set_linewidth(renderer, GENERALIZATION_WIDTH);
+  renderer_ops->set_linestyle(renderer, LINESTYLE_SOLID);
+  renderer_ops->set_linejoin(renderer, LINEJOIN_MITER);
+  renderer_ops->set_linecaps(renderer, LINECAPS_BUTT);
 
-  renderer->ops->draw_polyline(renderer, points, n, &color_black);
+  arrow.type = ARROW_HOLLOW_TRIANGLE;
+  arrow.length = GENERALIZATION_TRIANGLESIZE;
+  arrow.width = GENERALIZATION_TRIANGLESIZE;
 
-  arrow_draw(renderer, ARROW_HOLLOW_TRIANGLE,
-	     &points[0], &points[1],
-	     GENERALIZATION_TRIANGLESIZE, GENERALIZATION_TRIANGLESIZE,
-	     GENERALIZATION_WIDTH,
-	     &color_black, &color_white);
+  renderer_ops->draw_polyline_with_arrows(renderer,
+					   points, n,
+					   GENERALIZATION_WIDTH,
+					   &genlz->line_color,
+					   &arrow, NULL);
 
-  renderer->ops->set_font(renderer, genlz_font, GENERALIZATION_FONTHEIGHT);
+  renderer_ops->set_font(renderer, genlz_font, GENERALIZATION_FONTHEIGHT);
   pos = genlz->text_pos;
   
-  if (genlz->stereotype != NULL && genlz->stereotype[0] != '\0') {
-    renderer->ops->draw_string(renderer,
-			       genlz->stereotype,
+  if (genlz->st_stereotype != NULL && genlz->st_stereotype[0] != '\0') {
+    renderer_ops->draw_string(renderer,
+			       genlz->st_stereotype,
 			       &pos, genlz->text_align,
-			       &color_black);
+			       &genlz->text_color);
 
     pos.y += GENERALIZATION_FONTHEIGHT;
   }
   
   if (genlz->name != NULL && genlz->name[0] != '\0') {
-    renderer->ops->draw_string(renderer,
+    renderer_ops->draw_string(renderer,
 			       genlz->name,
 			       &pos, genlz->text_align,
-			       &color_black);
+			       &genlz->text_color);
   }
   
 }
@@ -287,20 +265,42 @@ generalization_update_data(Generalization *genlz)
   int num_segm, i;
   Point *points;
   Rectangle rect;
-  OrthConnBBExtras *extra;
+  PolyBBExtras *extra;
+  real descent;
+  real ascent;
   
   orthconn_update_data(orth);
   
+  genlz->stereotype = remove_stereotype_from_string(genlz->stereotype);
+  if (!genlz->st_stereotype) {
+    genlz->st_stereotype =  string_to_stereotype(genlz->stereotype);
+  }
+
   genlz->text_width = 0.0;
-
-  if (genlz->name)
-    genlz->text_width = font_string_width(genlz->name, genlz_font,
-					  GENERALIZATION_FONTHEIGHT);
-  if (genlz->stereotype)
+  descent = 0.0;
+  ascent = 0.0;
+  
+  if (genlz->name) {      
+    genlz->text_width = dia_font_string_width(genlz->name, genlz_font,
+                                              GENERALIZATION_FONTHEIGHT);
+    descent = dia_font_descent(genlz->name,
+                               genlz_font,GENERALIZATION_FONTHEIGHT);
+    ascent = dia_font_ascent(genlz->name,
+                             genlz_font,GENERALIZATION_FONTHEIGHT);
+  }
+  if (genlz->stereotype) {
     genlz->text_width = MAX(genlz->text_width,
-			    font_string_width(genlz->stereotype, genlz_font,
-					      GENERALIZATION_FONTHEIGHT));
-
+                            dia_font_string_width(genlz->stereotype,
+                                                  genlz_font,
+                                                  GENERALIZATION_FONTHEIGHT));
+    if (!genlz->name) {
+      descent = dia_font_descent(genlz->stereotype,
+                                genlz_font,GENERALIZATION_FONTHEIGHT);
+    }
+    ascent = dia_font_ascent(genlz->stereotype,
+                             genlz_font,GENERALIZATION_FONTHEIGHT);
+  }
+  
   extra = &orth->extra_spacing;
   
   extra->start_trans = GENERALIZATION_WIDTH/2.0 + GENERALIZATION_TRIANGLESIZE;
@@ -325,13 +325,12 @@ generalization_update_data(Generalization *genlz)
   case HORIZONTAL:
     genlz->text_align = ALIGN_CENTER;
     genlz->text_pos.x = 0.5*(points[i].x+points[i+1].x);
-    genlz->text_pos.y = points[i].y - font_descent(genlz_font, GENERALIZATION_FONTHEIGHT);
+    genlz->text_pos.y = points[i].y - descent;
     break;
   case VERTICAL:
     genlz->text_align = ALIGN_LEFT;
     genlz->text_pos.x = points[i].x + 0.1;
-    genlz->text_pos.y =
-      0.5*(points[i].y+points[i+1].y) - font_descent(genlz_font, GENERALIZATION_FONTHEIGHT);
+    genlz->text_pos.y = 0.5*(points[i].y+points[i+1].y) - descent;
     break;
   }
 
@@ -340,7 +339,7 @@ generalization_update_data(Generalization *genlz)
   if (genlz->text_align == ALIGN_CENTER)
     rect.left -= genlz->text_width/2.0;
   rect.right = rect.left + genlz->text_width;
-  rect.top = genlz->text_pos.y - font_ascent(genlz_font, GENERALIZATION_FONTHEIGHT);
+  rect.top = genlz->text_pos.y - ascent;
   rect.bottom = rect.top + 2*GENERALIZATION_FONTHEIGHT;
 
   rectangle_union(&obj->bounding_box, &rect);
@@ -368,6 +367,7 @@ generalization_delete_segment_callback(Object *obj, Point *clicked, gpointer dat
 static DiaMenuItem object_menu_items[] = {
   { N_("Add segment"), generalization_add_segment_callback, NULL, 1 },
   { N_("Delete segment"), generalization_delete_segment_callback, NULL, 1 },
+  ORTHCONN_COMMON_MENUS,
 };
 
 static DiaMenu object_menu = {
@@ -383,9 +383,12 @@ generalization_get_object_menu(Generalization *genlz, Point *clickedpoint)
   OrthConn *orth;
 
   orth = &genlz->orth;
+  
   /* Set entries sensitive/selected etc here */
   object_menu_items[0].active = orthconn_can_add_segment(orth, clickedpoint);
   object_menu_items[1].active = orthconn_can_delete_segment(orth, clickedpoint);
+  orthconn_update_object_menu(orth, clickedpoint, &object_menu_items[2]);
+
   return &object_menu;
 }
 
@@ -398,10 +401,10 @@ generalization_create(Point *startpoint,
   Generalization *genlz;
   OrthConn *orth;
   Object *obj;
-  OrthConnBBExtras *extra;
+  PolyBBExtras *extra;
 
   if (genlz_font == NULL) {
-    genlz_font = font_getfont("Courier");
+    genlz_font = dia_font_new_from_style(DIA_FONT_MONOSPACE, GENERALIZATION_FONTHEIGHT);
   }
   
   genlz = g_new0(Generalization, 1);
@@ -415,8 +418,11 @@ generalization_create(Point *startpoint,
 
   orthconn_init(orth, startpoint);
 
+  genlz->text_color = color_black;
+  genlz->line_color = attributes_get_foreground();
   genlz->name = NULL;
   genlz->stereotype = NULL;
+  genlz->st_stereotype = NULL;
 
   generalization_update_data(genlz);
   
@@ -431,82 +437,15 @@ generalization_destroy(Generalization *genlz)
 {
   g_free(genlz->name);
   g_free(genlz->stereotype);
+  g_free(genlz->st_stereotype);
 
   orthconn_destroy(&genlz->orth);
 }
 
-static Object *
-generalization_copy(Generalization *genlz)
-{
-  Generalization *newgenlz;
-  OrthConn *orth, *neworth;
-  Object *newobj;
-  
-  orth = &genlz->orth;
-  
-  newgenlz = g_malloc0(sizeof(Generalization));
-  neworth = &newgenlz->orth;
-  newobj = (Object *)newgenlz;
-
-  orthconn_copy(orth, neworth);
-
-  newgenlz->name = genlz->name ? strdup(genlz->name) : NULL;
-  newgenlz->stereotype = genlz->stereotype ? strdup(genlz->stereotype) : NULL;
-  newgenlz->text_width = genlz->text_width;
-  
-  generalization_update_data(newgenlz);
-  
-  return (Object *)newgenlz;
-}
-
-static void
-generalization_save(Generalization *genlz, ObjectNode obj_node,
-		    const char *filename)
-{
-  orthconn_save(&genlz->orth, obj_node);
-
-  data_add_string(new_attribute(obj_node, "name"),
-		  genlz->name);
-  data_add_string(new_attribute(obj_node, "stereotype"),
-		  genlz->stereotype);
-}
-
 static Object *
 generalization_load(ObjectNode obj_node, int version,
 		    const char *filename)
 {
-  Generalization *genlz;
-  AttributeNode attr;
-  OrthConn *orth;
-  Object *obj;
-  OrthConnBBExtras *extra;
-
-  if (genlz_font == NULL) {
-    genlz_font = font_getfont("Courier");
-  }
-
-  genlz = g_new0(Generalization, 1);
-
-  orth = &genlz->orth;
-  obj = (Object *)genlz;
-  extra = &orth->extra_spacing;
-
-  obj->type = &generalization_type;
-  obj->ops = &generalization_ops;
-
-  orthconn_load(orth, obj_node);
-
-  genlz->name = NULL;
-  attr = object_find_attribute(obj_node, "name");
-  if (attr != NULL)
-    genlz->name = data_string(attribute_first_data(attr));
-
-  genlz->stereotype = NULL;
-  attr = object_find_attribute(obj_node, "stereotype");
-  if (attr != NULL)
-    genlz->stereotype = data_string(attribute_first_data(attr));
-
-  generalization_update_data(genlz);
-
-  return &genlz->orth.object;
+  return object_load_using_properties(&generalization_type,
+                                      obj_node,version,filename);
 }
