@@ -34,6 +34,7 @@
 #include "handle.h"
 #include "arrows.h"
 #include "properties.h"
+#include "attributes.h"
 
 #include "pixmaps/message.xpm"
 
@@ -60,6 +61,9 @@ struct _Message {
   Point text_pos;
   real text_width;
     
+  Color text_color;
+  Color line_color;
+
   MessageType type;
 };
 
@@ -73,9 +77,10 @@ struct _Message {
 
 static DiaFont *message_font = NULL;
 
-static void message_move_handle(Message *message, Handle *handle,
-				   Point *to, HandleMoveReason reason, ModifierKeys modifiers);
-static void message_move(Message *message, Point *to);
+static ObjectChange* message_move_handle(Message *message, Handle *handle,
+					 Point *to, ConnectionPoint *cp,
+					 HandleMoveReason reason, ModifierKeys modifiers);
+static ObjectChange* message_move(Message *message, Point *to);
 static void message_select(Message *message, Point *clicked_point,
 			      DiaRenderer *interactive_renderer);
 static void message_draw(Message *message, DiaRenderer *renderer);
@@ -139,6 +144,8 @@ static PropEnumData prop_message_type_data[] = {
 
 static PropDescription message_props[] = {
   CONNECTION_COMMON_PROPERTIES,
+  PROP_STD_TEXT_COLOUR_OPTIONAL,
+  PROP_STD_LINE_COLOUR_OPTIONAL, 
   { "text", PROP_TYPE_STRING, PROP_FLAG_VISIBLE,
     N_("Message:"), NULL, NULL },  
   { "type", PROP_TYPE_ENUM, PROP_FLAG_VISIBLE,
@@ -159,6 +166,8 @@ message_describe_props(Message *mes)
 
 static PropOffset message_offsets[] = {
   CONNECTION_COMMON_PROPERTIES_OFFSETS,
+  { "line_colour",PROP_TYPE_COLOUR,offsetof(Message,line_color) },
+  { "text_colour",PROP_TYPE_COLOUR,offsetof(Message,text_color) },
   { "text", PROP_TYPE_STRING, offsetof(Message, text) },
   { "type", PROP_TYPE_ENUM, offsetof(Message, type) },
   { "text_pos", PROP_TYPE_POINT, offsetof(Message,text_pos) }, 
@@ -201,9 +210,10 @@ message_select(Message *message, Point *clicked_point,
   connection_update_handles(&message->connection);
 }
 
-static void
+static ObjectChange*
 message_move_handle(Message *message, Handle *handle,
-		 Point *to, HandleMoveReason reason, ModifierKeys modifiers)
+		    Point *to, ConnectionPoint *cp,
+		    HandleMoveReason reason, ModifierKeys modifiers)
 {
   Point p1, p2;
   Point *endpoints;
@@ -218,7 +228,7 @@ message_move_handle(Message *message, Handle *handle,
     endpoints = &message->connection.endpoints[0]; 
     p1.x = 0.5*(endpoints[0].x + endpoints[1].x);
     p1.y = 0.5*(endpoints[0].y + endpoints[1].y);
-    connection_move_handle(&message->connection, handle->id, to, reason);
+    connection_move_handle(&message->connection, handle->id, to, cp, reason, modifiers);
     p2.x = 0.5*(endpoints[0].x + endpoints[1].x);
     p2.y = 0.5*(endpoints[0].y + endpoints[1].y);
     point_sub(&p2, &p1);
@@ -226,9 +236,11 @@ message_move_handle(Message *message, Handle *handle,
   }
 
   message_update_data(message);
+
+  return NULL;
 }
 
-static void
+static ObjectChange*
 message_move(Message *message, Point *to)
 {
   Point start_to_end;
@@ -247,6 +259,8 @@ message_move(Message *message, Point *to)
   point_add(&message->text_pos, &delta);
   
   message_update_data(message);
+
+  return NULL;
 }
 
 static void
@@ -297,18 +311,18 @@ message_draw(Message *message, DiaRenderer *renderer)
       px.y = p1.y;
       renderer_ops->draw_line(renderer,
 			       &p1, &px,
-			       &color_black);
+			       &message->line_color);
 
       renderer_ops->draw_line(renderer,
 			   &px, &p2,
-			   &color_black);
+			   &message->line_color);
       p1.y = p2.y;
   } 
 
   renderer_ops->draw_line_with_arrows(renderer,
 				       &p1, &p2,
 				       MESSAGE_WIDTH,
-				       &color_black,
+				       &message->line_color,
 				       &arrow, NULL); 
 
   renderer_ops->set_font(renderer, message_font,
@@ -325,7 +339,7 @@ message_draw(Message *message, DiaRenderer *renderer)
       renderer_ops->draw_string(renderer,
 				 mname, /*message->text,*/
 				 &message->text_pos, ALIGN_CENTER,
-				 &color_black);
+				 &message->text_color);
   if (message->type == MESSAGE_CREATE || message->type == MESSAGE_DESTROY)
   {
 	  g_free(mname);
@@ -365,6 +379,8 @@ message_create(Point *startpoint,
   
   connection_init(conn, 3, 0);
 
+  message->text_color = color_black;
+  message->line_color = attributes_get_foreground();
   message->text = g_strdup("");
   message->text_width = 0.0;
   message->text_pos.x = 0.5*(conn->endpoints[0].x + conn->endpoints[1].x);
