@@ -15,18 +15,22 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
+
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <assert.h>
-#include <gtk/gtk.h>
 #include <math.h>
 #include <string.h>
 
-#include "config.h"
 #include "intl.h"
 #include "object.h"
 #include "element.h"
-#include "render.h"
+#include "diarenderer.h"
 #include "attributes.h"
 #include "text.h"
+#include "properties.h"
 
 #include "pixmaps/case.xpm"
 
@@ -34,13 +38,6 @@ typedef struct _UsecasePropertiesDialog UsecasePropertiesDialog;
 typedef struct _Usecase Usecase;
 typedef struct _UsecaseState UsecaseState;
 
-struct _UsecaseState {
-  ObjectState obj_state;
-  
-  int text_outside;
-  int collaboration;
-};
-
 struct _Usecase {
   Element element;
 
@@ -49,6 +46,10 @@ struct _Usecase {
   Text *text;
   int text_outside;
   int collaboration;
+  TextAttributes attrs;
+
+  Color line_color;
+  Color fill_color;
 };
 
 
@@ -67,36 +68,33 @@ struct _UsecasePropertiesDialog {
 #define USECASE_LINEWIDTH 0.1
 #define USECASE_MARGIN_Y 0.3
 
-static UsecasePropertiesDialog *properties_dialog;
-
 static real usecase_distance_from(Usecase *usecase, Point *point);
 static void usecase_select(Usecase *usecase, Point *clicked_point,
-			   Renderer *interactive_renderer);
-static void usecase_move_handle(Usecase *usecase, Handle *handle,
-				Point *to, HandleMoveReason reason, ModifierKeys modifiers);
-static void usecase_move(Usecase *usecase, Point *to);
-static void usecase_draw(Usecase *usecase, Renderer *renderer);
+			   DiaRenderer *interactive_renderer);
+static ObjectChange* usecase_move_handle(Usecase *usecase, Handle *handle,
+					 Point *to, ConnectionPoint *cp,
+					 HandleMoveReason reason, ModifierKeys modifiers);
+static ObjectChange* usecase_move(Usecase *usecase, Point *to);
+static void usecase_draw(Usecase *usecase, DiaRenderer *renderer);
 static Object *usecase_create(Point *startpoint,
 			      void *user_data,
 			      Handle **handle1,
 			      Handle **handle2);
 static void usecase_destroy(Usecase *usecase);
-static Object *usecase_copy(Usecase *usecase);
-
-static void usecase_save(Usecase *usecase, ObjectNode obj_node,
-			 const char *filename);
 static Object *usecase_load(ObjectNode obj_node, int version,
 			    const char *filename);
 static void usecase_update_data(Usecase *usecase);
-static ObjectChange *usecase_apply_properties(Usecase *usecase);
-static GtkWidget *usecase_get_properties(Usecase *dep);
-
+static PropDescription *usecase_describe_props(Usecase *usecase);
+static void usecase_get_props(Usecase *usecase, GPtrArray *props);
+static void usecase_set_props(Usecase *usecase, GPtrArray *props);
 
 static ObjectTypeOps usecase_type_ops =
 {
   (CreateFunc) usecase_create,
-  (LoadFunc)   usecase_load,
-  (SaveFunc)   usecase_save
+  (LoadFunc)   usecase_load,/*using_properties*/     /* load */
+  (SaveFunc)   object_save_using_properties,      /* save */
+  (GetDefaultsFunc)   NULL, 
+  (ApplyDefaultsFunc) NULL
 };
 
 ObjectType usecase_type =
@@ -113,14 +111,72 @@ static ObjectOps usecase_ops = {
   (DrawFunc)            usecase_draw,
   (DistanceFunc)        usecase_distance_from,
   (SelectFunc)          usecase_select,
-  (CopyFunc)            usecase_copy,
+  (CopyFunc)            object_copy_using_properties,
   (MoveFunc)            usecase_move,
   (MoveHandleFunc)      usecase_move_handle,
-  (GetPropertiesFunc)   usecase_get_properties,
-  (ApplyPropertiesFunc) usecase_apply_properties,
-  (ObjectMenuFunc)      NULL
+  (GetPropertiesFunc)   object_create_props_dialog,
+  (ApplyPropertiesFunc) object_apply_props_from_dialog,
+  (ObjectMenuFunc)      NULL,
+  (DescribePropsFunc)   usecase_describe_props,
+  (GetPropsFunc)        usecase_get_props,
+  (SetPropsFunc)        usecase_set_props
 };
 
+static PropDescription usecase_props[] = {
+  ELEMENT_COMMON_PROPERTIES,
+  PROP_STD_LINE_COLOUR_OPTIONAL, 
+  PROP_STD_FILL_COLOUR_OPTIONAL, 
+  { "text_outside", PROP_TYPE_BOOL, PROP_FLAG_VISIBLE,
+  N_("Text outside"), NULL, NULL },
+  { "collaboration", PROP_TYPE_BOOL, PROP_FLAG_VISIBLE,
+  N_("Collaboration"), NULL, NULL },
+  PROP_STD_TEXT_FONT,
+  PROP_STD_TEXT_HEIGHT,
+  PROP_STD_TEXT_COLOUR_OPTIONAL,
+  { "text", PROP_TYPE_TEXT, 0, N_("Text"), NULL, NULL }, 
+  
+  PROP_DESC_END
+};
+
+static PropDescription *
+usecase_describe_props(Usecase *usecase)
+{
+  if (usecase_props[0].quark == 0) {
+    prop_desc_list_calculate_quarks(usecase_props);
+  }
+  return usecase_props;
+}
+
+static PropOffset usecase_offsets[] = {
+  ELEMENT_COMMON_PROPERTIES_OFFSETS,
+  {"line_colour", PROP_TYPE_COLOUR, offsetof(Usecase, line_color) },
+  {"fill_colour", PROP_TYPE_COLOUR, offsetof(Usecase, fill_color) },
+  {"text_outside", PROP_TYPE_BOOL, offsetof(Usecase, text_outside) },
+  {"collaboration", PROP_TYPE_BOOL, offsetof(Usecase, collaboration) },
+  {"text",PROP_TYPE_TEXT,offsetof(Usecase,text)},
+  {"text_font",PROP_TYPE_FONT,offsetof(Usecase,attrs.font)},
+  {"text_height",PROP_TYPE_REAL,offsetof(Usecase,attrs.height)},
+  {"text_colour",PROP_TYPE_COLOUR,offsetof(Usecase,attrs.color)},
+  { NULL, 0, 0 },
+};
+
+static void
+usecase_get_props(Usecase * usecase, GPtrArray *props)
+{
+  text_get_attributes(usecase->text,&usecase->attrs);
+  object_get_props_from_offsets(&usecase->element.object,
+                                usecase_offsets,props);
+}
+
+static void
+usecase_set_props(Usecase *usecase, GPtrArray *props)
+{
+  object_set_props_from_offsets(&usecase->element.object,
+                                usecase_offsets,props);
+  apply_textattr_properties(props,usecase->text,"text",&usecase->attrs);
+  usecase_update_data(usecase);
+}
+
 static real
 usecase_distance_from(Usecase *usecase, Point *point)
 {
@@ -130,25 +186,28 @@ usecase_distance_from(Usecase *usecase, Point *point)
 
 static void
 usecase_select(Usecase *usecase, Point *clicked_point,
-	       Renderer *interactive_renderer)
+	       DiaRenderer *interactive_renderer)
 {
   text_set_cursor(usecase->text, clicked_point, interactive_renderer);
-  text_grab_focus(usecase->text, (Object *)usecase);
+  text_grab_focus(usecase->text, &usecase->element.object);
   element_update_handles(&usecase->element);
 }
 
-static void
+static ObjectChange*
 usecase_move_handle(Usecase *usecase, Handle *handle,
-		 Point *to, HandleMoveReason reason, ModifierKeys modifiers)
+		    Point *to, ConnectionPoint *cp,
+		    HandleMoveReason reason, ModifierKeys modifiers)
 {
   assert(usecase!=NULL);
   assert(handle!=NULL);
   assert(to!=NULL);
 
   assert(handle->id < 8);
+
+  return NULL;
 }
 
-static void
+static ObjectChange*
 usecase_move(Usecase *usecase, Point *to)
 {
   real h;
@@ -166,11 +225,14 @@ usecase_move(Usecase *usecase, Point *to)
   }
   text_set_position(usecase->text, &p);
   usecase_update_data(usecase);
+
+  return NULL;
 }
 
 static void
-usecase_draw(Usecase *usecase, Renderer *renderer)
+usecase_draw(Usecase *usecase, DiaRenderer *renderer)
 {
+  DiaRendererClass *renderer_ops = DIA_RENDERER_GET_CLASS (renderer);
   Element *elem;
   real x, y, w, h;
   Point c;
@@ -196,22 +258,22 @@ usecase_draw(Usecase *usecase, Renderer *renderer)
   }
 
 
-  renderer->ops->set_fillstyle(renderer, FILLSTYLE_SOLID);
-  renderer->ops->set_linewidth(renderer, USECASE_LINEWIDTH);
+  renderer_ops->set_fillstyle(renderer, FILLSTYLE_SOLID);
+  renderer_ops->set_linewidth(renderer, USECASE_LINEWIDTH);
 
   if (usecase->collaboration)
-	  renderer->ops->set_linestyle(renderer, LINESTYLE_DASHED);
+	  renderer_ops->set_linestyle(renderer, LINESTYLE_DASHED);
   else 
-	  renderer->ops->set_linestyle(renderer, LINESTYLE_SOLID);
+	  renderer_ops->set_linestyle(renderer, LINESTYLE_SOLID);
 
-  renderer->ops->fill_ellipse(renderer, 
+  renderer_ops->fill_ellipse(renderer, 
 			     &c,
 			     w, h,
-			     &color_white);
-  renderer->ops->draw_ellipse(renderer, 
+			     &usecase->fill_color);
+  renderer_ops->draw_ellipse(renderer, 
 			     &c,
 			     w, h,
-			     &color_black);  
+			     &usecase->line_color);  
   
   text_draw(usecase->text, renderer);
 }
@@ -221,11 +283,12 @@ static void
 usecase_update_data(Usecase *usecase)
 {
   real w, h, ratio;
-  Point c, half, r;
+  Point c, half, r,p;
   
   Element *elem = &usecase->element;
-  Object *obj = (Object *) usecase;
+  Object *obj = &elem->object;
   
+  text_calc_boundingbox(usecase->text, NULL);
   w = usecase->text->max_width;
   h = usecase->text->height*usecase->text->numlines;
 
@@ -278,7 +341,7 @@ usecase_update_data(Usecase *usecase)
   usecase->connections[3].pos.y = c.y;
   usecase->connections[4].pos.x = c.x + r.x;
   usecase->connections[4].pos.y = c.y;
-  
+                                                                                            
   if (usecase->text_outside) { 
       usecase->connections[5].pos.x = elem->corner.x;
       usecase->connections[5].pos.y = elem->corner.y + elem->height;
@@ -295,11 +358,31 @@ usecase_update_data(Usecase *usecase)
       usecase->connections[7].pos.y = c.y + half.y;
   }
 
+  usecase->connections[0].directions = DIR_NORTH|DIR_WEST;
+  usecase->connections[1].directions = DIR_NORTH;
+  usecase->connections[2].directions = DIR_NORTH|DIR_EAST;
+  usecase->connections[3].directions = DIR_WEST;
+  usecase->connections[4].directions = DIR_EAST;
+  usecase->connections[5].directions = DIR_SOUTH|DIR_WEST;
+  usecase->connections[6].directions = DIR_SOUTH;
+  usecase->connections[7].directions = DIR_SOUTH|DIR_EAST;
+
+  h = usecase->text->height*usecase->text->numlines;
+  p = usecase->element.corner;
+  p.x += usecase->element.width/2.0;
+  if (usecase->text_outside) {
+      p.y += usecase->element.height - h + usecase->text->ascent;
+  } else {
+      p.y += (usecase->element.height - h)/2.0 + usecase->text->ascent;
+  }
+  text_set_position(usecase->text, &p);
+
   element_update_boundingbox(elem);
 
   obj->position = elem->corner;
 
   element_update_handles(elem);
+
 }
 
 static Object *
@@ -312,12 +395,12 @@ usecase_create(Point *startpoint,
   Element *elem;
   Object *obj;
   Point p;
-  Font *font;
+  DiaFont *font;
   int i;
   
-  usecase = g_malloc(sizeof(Usecase));
+  usecase = g_malloc0(sizeof(Usecase));
   elem = &usecase->element;
-  obj = (Object *) usecase;
+  obj = &elem->object;
   
   obj->type = &usecase_type;
   obj->ops = &usecase_ops;
@@ -325,12 +408,17 @@ usecase_create(Point *startpoint,
   elem->width = USECASE_WIDTH;
   elem->height = USECASE_HEIGHT;
 
-  font = font_getfont("Helvetica");
+  usecase->line_color = attributes_get_foreground();
+  usecase->fill_color = attributes_get_background();
+
+  font = dia_font_new_from_style(DIA_FONT_SANS, 0.8);
   p = *startpoint;
   p.x += USECASE_WIDTH/2.0;
   p.y += USECASE_HEIGHT/2.0;
   
   usecase->text = new_text("", font, 0.8, &p, &color_black, ALIGN_CENTER);
+  dia_font_unref(font);  
+  text_get_attributes(usecase->text,&usecase->attrs);
   usecase->text_outside = 0;
   usecase->collaboration = 0;
   element_init(elem, 8, 8);
@@ -340,6 +428,7 @@ usecase_create(Point *startpoint,
     usecase->connections[i].object = obj;
     usecase->connections[i].connected = NULL;
   }
+  elem->extra_spacing.border_trans = 0.0;
   usecase_update_data(usecase);
 
   for (i=0;i<8;i++) {
@@ -348,7 +437,7 @@ usecase_create(Point *startpoint,
 
   *handle1 = NULL;
   *handle2 = NULL;
-  return (Object *)usecase;
+  return &usecase->element.object;
 }
 
 static void
@@ -359,201 +448,11 @@ usecase_destroy(Usecase *usecase)
   element_destroy(&usecase->element);
 }
 
-static Object *
-usecase_copy(Usecase *usecase)
-{
-  int i;
-  Usecase *newusecase;
-  Element *elem, *newelem;
-  Object *newobj;
-  
-  elem = &usecase->element;
-  
-  newusecase = g_malloc(sizeof(Usecase));
-  newelem = &newusecase->element;
-  newobj = (Object *) newusecase;
-
-  element_copy(elem, newelem);
-
-  newusecase->text = text_copy(usecase->text);
-  newusecase->text_outside = usecase->text_outside;
-  newusecase->collaboration = usecase->collaboration;
-  
-  for (i=0;i<8;i++) {
-    newobj->connections[i] = &newusecase->connections[i];
-    newusecase->connections[i].object = newobj;
-    newusecase->connections[i].connected = NULL;
-    newusecase->connections[i].pos = usecase->connections[i].pos;
-    newusecase->connections[i].last_pos = usecase->connections[i].last_pos;
-  }
-
-  newusecase->text_outside = usecase->text_outside;
-  newusecase->collaboration = usecase->collaboration;
-  usecase_update_data(newusecase);
-  
-  return (Object *)newusecase;
-}
-
-static UsecaseState *
-usecase_get_state(Usecase *usecase)
-{
-  UsecaseState *state = g_new(UsecaseState, 1);
-
-  state->obj_state.free = NULL;
-
-  state->text_outside = usecase->text_outside;
-  state->collaboration = usecase->collaboration;
-
-  return state;
-}
-
-static void
-usecase_set_state(Usecase *usecase, UsecaseState *state)
-{
-  usecase->text_outside = state->text_outside;
-  usecase->collaboration = state->collaboration;
-  
-  g_free(state);
-  
-  usecase_update_data(usecase);
-}
-
-static void
-usecase_save(Usecase *usecase, ObjectNode obj_node, const char *filename)
-{
-  element_save(&usecase->element, obj_node);
-
-  data_add_text(new_attribute(obj_node, "text"),
-		usecase->text);
-
-  data_add_boolean(new_attribute(obj_node, "textout"),
-		   usecase->text_outside);
-
-  data_add_boolean(new_attribute(obj_node, "collaboration"),
-		   usecase->collaboration);
-}
-
 static Object *
 usecase_load(ObjectNode obj_node, int version, const char *filename)
 {
-  Usecase *usecase;
-  Element *elem;
-  Object *obj;
-  int i;
-  AttributeNode attr;
-
-  usecase = g_malloc(sizeof(Usecase));
-  elem = &usecase->element;
-  obj = (Object *) usecase;
-  
-  obj->type = &usecase_type;
-  obj->ops = &usecase_ops;
-
-  element_load(elem, obj_node);
-  attr = object_find_attribute(obj_node, "text");
-  if (attr != NULL)
-      usecase->text = data_text(attribute_first_data(attr));
-
-
-  attr = object_find_attribute(obj_node, "textout");
-  if (attr != NULL)
-    usecase->text_outside = data_boolean(attribute_first_data(attr));
-  else
-    usecase->text_outside = 0;
-
-  attr = object_find_attribute(obj_node, "collaboration");
-  if (attr != NULL)
-    usecase->collaboration = data_boolean(attribute_first_data(attr));
-  else
-    usecase->collaboration = 0;
-  
-  element_init(elem, 8, 8);
-
-  for (i=0;i<8;i++) {
-    obj->connections[i] = &usecase->connections[i];
-    usecase->connections[i].object = obj;
-    usecase->connections[i].connected = NULL;
-  }
-  usecase_update_data(usecase);
-
-  for (i=0;i<8;i++) {
-    obj->handles[i]->type = HANDLE_NON_MOVABLE;
-  }
-
-  return (Object *)usecase;
+  return object_load_using_properties(&usecase_type,
+                                      obj_node,version,filename);
 }
 
-static ObjectChange *
-usecase_apply_properties(Usecase *usecase)
-{
-  UsecasePropertiesDialog *prop_dialog;
-  ObjectState *old_state;
-  real h;
-  Point p;
 
-  prop_dialog = properties_dialog;
-
-  old_state = (ObjectState *)usecase_get_state(usecase);
-
-  usecase->text_outside = prop_dialog->text_out->active;
-  usecase->collaboration = prop_dialog->collaboration->active;
-  usecase_update_data(usecase);
-
-  h = usecase->text->height*usecase->text->numlines;
-  p = usecase->element.corner;
-  p.x += usecase->element.width/2.0;
-  if (usecase->text_outside) {
-      p.y += usecase->element.height - h + usecase->text->ascent;
-  } else {
-      p.y += (usecase->element.height - h)/2.0 + usecase->text->ascent;
-  }
-  text_set_position(usecase->text, &p);
-  return new_object_state_change((Object *)usecase, old_state, 
-				 (GetStateFunc)usecase_get_state,
-				 (SetStateFunc)usecase_set_state);
-}
-
-static void
-fill_in_dialog(Usecase *usecase)
-{
-  UsecasePropertiesDialog *prop_dialog;
-  prop_dialog = properties_dialog;
-  gtk_toggle_button_set_active(prop_dialog->text_out, usecase->text_outside);
-  gtk_toggle_button_set_active(prop_dialog->collaboration, usecase->collaboration);
-
-
-}
-
-static GtkWidget *
-usecase_get_properties(Usecase *dep)
-{
-  UsecasePropertiesDialog *prop_dialog;
-  GtkWidget *dialog;
-  GtkWidget *checkbox;
-
-  if (properties_dialog == NULL) {
-
-    prop_dialog = g_new(UsecasePropertiesDialog, 1);
-    properties_dialog = prop_dialog;
-
-    dialog = gtk_vbox_new(FALSE, 0);
-    gtk_object_ref(GTK_OBJECT(dialog));
-    gtk_object_sink(GTK_OBJECT(dialog));
-    prop_dialog->dialog = dialog;
-    
-    checkbox = gtk_check_button_new_with_label(_("Text outside:"));
-    prop_dialog->text_out = GTK_TOGGLE_BUTTON( checkbox );
-    gtk_widget_show(checkbox);
-    gtk_box_pack_start (GTK_BOX (dialog), checkbox, TRUE, TRUE, 0);
-
-    checkbox = gtk_check_button_new_with_label(_("Collaboration"));
-    prop_dialog->collaboration = GTK_TOGGLE_BUTTON( checkbox );
-    gtk_widget_show(checkbox);
-    gtk_box_pack_start (GTK_BOX (dialog), checkbox, TRUE, TRUE, 0);
-  }
-  
-  fill_in_dialog(dep);
-  gtk_widget_show (properties_dialog->dialog);
-
-  return properties_dialog->dialog;
-}
