@@ -80,10 +80,11 @@ static EllipseProperties default_properties;
 static real ellipse_distance_from(Ellipse *ellipse, Point *point);
 static void ellipse_select(Ellipse *ellipse, Point *clicked_point,
 		       DiaRenderer *interactive_renderer);
-static void ellipse_move_handle(Ellipse *ellipse, Handle *handle,
-			    Point *to, HandleMoveReason reason, 
+static ObjectChange* ellipse_move_handle(Ellipse *ellipse, Handle *handle,
+					 Point *to, ConnectionPoint *cp,
+					 HandleMoveReason reason, 
 			    ModifierKeys modifiers);
-static void ellipse_move(Ellipse *ellipse, Point *to);
+static ObjectChange* ellipse_move(Ellipse *ellipse, Point *to);
 static void ellipse_draw(Ellipse *ellipse, DiaRenderer *renderer);
 static void ellipse_update_data(Ellipse *ellipse, AnchorShape h,AnchorShape v);
 static Object *ellipse_create(Point *startpoint,
@@ -147,6 +148,7 @@ static PropDescription ellipse_props[] = {
   PROP_STD_TEXT_FONT,
   PROP_STD_TEXT_HEIGHT,
   PROP_STD_TEXT_COLOUR,
+  PROP_STD_TEXT_ALIGNMENT,
   PROP_STD_SAVED_TEXT,
   
   { NULL, 0, 0, NULL, NULL, NULL, 0}
@@ -173,6 +175,7 @@ static PropOffset ellipse_offsets[] = {
   {"text_font",PROP_TYPE_FONT,offsetof(Ellipse,attrs.font)},
   {"text_height",PROP_TYPE_REAL,offsetof(Ellipse,attrs.height)},
   {"text_colour",PROP_TYPE_COLOUR,offsetof(Ellipse,attrs.color)},
+  {"text_alignment",PROP_TYPE_ENUM,offsetof(Ellipse,attrs.alignment)},
   { NULL, 0, 0 },
 };
 
@@ -256,9 +259,10 @@ ellipse_select(Ellipse *ellipse, Point *clicked_point,
   element_update_handles(&ellipse->element);
 }
 
-static void
+static ObjectChange*
 ellipse_move_handle(Ellipse *ellipse, Handle *handle,
-		Point *to, HandleMoveReason reason, ModifierKeys modifiers)
+		    Point *to, ConnectionPoint *cp,
+		    HandleMoveReason reason, ModifierKeys modifiers)
 {
   AnchorShape horiz = ANCHOR_MIDDLE, vert = ANCHOR_MIDDLE;
 
@@ -266,7 +270,8 @@ ellipse_move_handle(Ellipse *ellipse, Handle *handle,
   assert(handle!=NULL);
   assert(to!=NULL);
 
-  element_move_handle(&ellipse->element, handle->id, to, reason);
+  element_move_handle(&ellipse->element, handle->id, to, cp, 
+		      reason, modifiers);
 
   switch (handle->id) {
   case HANDLE_RESIZE_NW:
@@ -289,14 +294,18 @@ ellipse_move_handle(Ellipse *ellipse, Handle *handle,
     break;
   }
   ellipse_update_data(ellipse, horiz, vert);
+
+  return NULL;
 }
 
-static void
+static ObjectChange*
 ellipse_move(Ellipse *ellipse, Point *to)
 {
   ellipse->element.corner = *to;
   
   ellipse_update_data(ellipse, ANCHOR_MIDDLE, ANCHOR_MIDDLE);
+
+  return NULL;
 }
 
 static void
@@ -401,6 +410,16 @@ ellipse_update_data(Ellipse *ellipse, AnchorShape horiz, AnchorShape vert)
   p.x += elem->width / 2.0;
   p.y += elem->height / 2.0 - ellipse->text->height*ellipse->text->numlines/2 +
     ellipse->text->ascent;
+  switch (ellipse->text->alignment) {
+  case ALIGN_LEFT:
+    p.x -= (elem->width - 2*(ellipse->padding + ellipse->border_width))/2;
+    break;
+  case ALIGN_RIGHT:
+    p.x += (elem->width - 2*(ellipse->padding + ellipse->border_width))/2;
+    break;
+  case ALIGN_CENTER:
+    break;
+  }
   text_set_position(ellipse->text, &p);
 
   /* Update connections: */
@@ -410,8 +429,13 @@ ellipse_update_data(Ellipse *ellipse, AnchorShape horiz, AnchorShape vert)
   dh = elem->height / 2.0;
   for (i = 0; i < 16; i++) {
     real theta = M_PI / 8.0 * i;
-    ellipse->connections[i].pos.x = c.x + dw * cos(theta);
-    ellipse->connections[i].pos.y = c.y - dh * sin(theta);
+    real costheta = cos(theta);
+    real sintheta = sin(theta);
+    connpoint_update(&ellipse->connections[i],
+		      c.x + dw * costheta,
+		      c.y - dh * sintheta,
+		      (costheta > .5?DIR_EAST:(costheta < -.5?DIR_WEST:0))|
+		      (sintheta > .5?DIR_NORTH:(sintheta < -.5?DIR_SOUTH:0)));
   }
 
   extra->border_trans = ellipse->border_width / 2.0;
@@ -448,7 +472,7 @@ ellipse_create(Point *startpoint,
 
   elem->corner = *startpoint;
   elem->width = DEFAULT_WIDTH;
-  elem->height = DEFAULT_WIDTH;
+  elem->height = DEFAULT_HEIGHT;
 
   ellipse->border_width =  attributes_get_default_linewidth();
   ellipse->border_color = attributes_get_foreground();
