@@ -18,21 +18,25 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
+
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <assert.h>
-#include <gtk/gtk.h>
 #include <math.h>
 
-#include "config.h"
 #include "intl.h"
 #include "object.h"
 #include "bezier_conn.h"
 #include "connectionpoint.h"
-#include "render.h"
+#include "diarenderer.h"
 #include "attributes.h"
 #include "widgets.h"
 #include "diamenu.h"
 #include "message.h"
 #include "properties.h"
+#include "create.h"
 
 #include "pixmaps/bezier.xpm"
 
@@ -51,12 +55,13 @@ struct _Bezierline {
 };
 
 
-static void bezierline_move_handle(Bezierline *bezierline, Handle *handle,
-				   Point *to, HandleMoveReason reason, ModifierKeys modifiers);
-static void bezierline_move(Bezierline *bezierline, Point *to);
+static ObjectChange* bezierline_move_handle(Bezierline *bezierline, Handle *handle,
+					    Point *to, ConnectionPoint *cp,
+					    HandleMoveReason reason, ModifierKeys modifiers);
+static ObjectChange* bezierline_move(Bezierline *bezierline, Point *to);
 static void bezierline_select(Bezierline *bezierline, Point *clicked_point,
-			      Renderer *interactive_renderer);
-static void bezierline_draw(Bezierline *bezierline, Renderer *renderer);
+			      DiaRenderer *interactive_renderer);
+static void bezierline_draw(Bezierline *bezierline, DiaRenderer *renderer);
 static Object *bezierline_create(Point *startpoint,
 				 void *user_data,
 				 Handle **handle1,
@@ -67,26 +72,22 @@ static void bezierline_destroy(Bezierline *bezierline);
 static Object *bezierline_copy(Bezierline *bezierline);
 
 static PropDescription *bezierline_describe_props(Bezierline *bezierline);
-static void bezierline_get_props(Bezierline *bezierline,
-				 Property *props, guint nprops);
-static void bezierline_set_props(Bezierline *bezierline,
-				 Property *props, guint nprops);
+static void bezierline_get_props(Bezierline *bezierline, GPtrArray *props);
+static void bezierline_set_props(Bezierline *bezierline, GPtrArray *props);
 
 static void bezierline_save(Bezierline *bezierline, ObjectNode obj_node,
 			  const char *filename);
 static Object *bezierline_load(ObjectNode obj_node, int version,
 			     const char *filename);
 static DiaMenu *bezierline_get_object_menu(Bezierline *bezierline, Point *clickedpoint);
-/* static GtkWidget *bezierline_get_defaults();
-   static void bezierline_apply_defaults(); */
 
 static ObjectTypeOps bezierline_type_ops =
 {
   (CreateFunc)bezierline_create,   /* create */
   (LoadFunc)  bezierline_load,     /* load */
   (SaveFunc)  bezierline_save,      /* save */
-  (GetDefaultsFunc)   NULL /*bezierline_get_defaults*/,
-  (ApplyDefaultsFunc) NULL /*bezierline_apply_defaults*/
+  (GetDefaultsFunc)   NULL,
+  (ApplyDefaultsFunc) NULL
 };
 
 static ObjectType bezierline_type =
@@ -95,7 +96,9 @@ static ObjectType bezierline_type =
   0,                         /* version */
   (char **) bezier_xpm,      /* pixmap */
   
-  &bezierline_type_ops       /* ops */
+  &bezierline_type_ops,      /* ops */
+  NULL,                      /* pixmap_file */
+  0                          /* default_user_data */
 };
 
 ObjectType *_bezierline_type = (ObjectType *) &bezierline_type;
@@ -118,7 +121,7 @@ static ObjectOps bezierline_ops = {
 };
 
 static PropDescription bezierline_props[] = {
-  OBJECT_COMMON_PROPERTIES,
+  BEZCONN_COMMON_PROPERTIES,
   PROP_STD_LINE_WIDTH,
   PROP_STD_LINE_COLOUR,
   PROP_STD_LINE_STYLE,
@@ -136,7 +139,7 @@ bezierline_describe_props(Bezierline *bezierline)
 }
 
 static PropOffset bezierline_offsets[] = {
-  OBJECT_COMMON_PROPERTIES_OFFSETS,
+  BEZCONN_COMMON_PROPERTIES_OFFSETS,
   { "line_width", PROP_TYPE_REAL, offsetof(Bezierline, line_width) },
   { "line_colour", PROP_TYPE_COLOUR, offsetof(Bezierline, line_color) },
   { "line_style", PROP_TYPE_LINESTYLE,
@@ -147,17 +150,17 @@ static PropOffset bezierline_offsets[] = {
 };
 
 static void
-bezierline_get_props(Bezierline *bezierline, Property *props, guint nprops)
+bezierline_get_props(Bezierline *bezierline, GPtrArray *props)
 {
-  object_get_props_from_offsets((Object *)bezierline, bezierline_offsets,
-				props, nprops);
+  object_get_props_from_offsets(&bezierline->bez.object, bezierline_offsets,
+				props);
 }
 
 static void
-bezierline_set_props(Bezierline *bezierline, Property *props, guint nprops)
+bezierline_set_props(Bezierline *bezierline, GPtrArray *props)
 {
-  object_set_props_from_offsets((Object *)bezierline, bezierline_offsets,
-				props, nprops);
+  object_set_props_from_offsets(&bezierline->bez.object, bezierline_offsets,
+				props);
   bezierline_update_data(bezierline);
 }
 
@@ -179,14 +182,15 @@ static int bezierline_closest_segment(Bezierline *bezierline, Point *point) {
 
 static void
 bezierline_select(Bezierline *bezierline, Point *clicked_point,
-		  Renderer *interactive_renderer)
+		  DiaRenderer *interactive_renderer)
 {
   bezierconn_update_data(&bezierline->bez);
 }
 
-static void
+static ObjectChange*
 bezierline_move_handle(Bezierline *bezierline, Handle *handle,
-		       Point *to, HandleMoveReason reason, ModifierKeys modifiers)
+		       Point *to, ConnectionPoint *cp,
+		       HandleMoveReason reason, ModifierKeys modifiers)
 {
   assert(bezierline!=NULL);
   assert(handle!=NULL);
@@ -201,39 +205,51 @@ bezierline_move_handle(Bezierline *bezierline, Handle *handle,
     dist.y = 0;
     point_scale(&dist, 0.332);
 
-    bezierconn_move_handle(bez, handle, to, reason);
+    bezierconn_move_handle(bez, handle, to, cp, reason, modifiers);
     
     bez->points[1].p1 = bez->points[0].p1;
     point_sub(&bez->points[1].p1, &dist);
     bez->points[1].p2 = *to;
     point_add(&bez->points[1].p2, &dist);
   } else {
-    bezierconn_move_handle(&bezierline->bez, handle, to, reason);
+    bezierconn_move_handle(&bezierline->bez, handle, to, cp, reason, modifiers);
   }
 
   bezierline_update_data(bezierline);
+
+  return NULL;
 }
 
 
-static void
+static ObjectChange*
 bezierline_move(Bezierline *bezierline, Point *to)
 {
   bezierconn_move(&bezierline->bez, to);
   bezierline_update_data(bezierline);
+
+  return NULL;
 }
 
 static void
-bezierline_draw(Bezierline *bezierline, Renderer *renderer)
+bezierline_draw(Bezierline *bezierline, DiaRenderer *renderer)
 {
   BezierConn *bez = &bezierline->bez;
+  DiaRendererClass *renderer_ops = DIA_RENDERER_GET_CLASS (renderer);
   
-  renderer->ops->set_linewidth(renderer, bezierline->line_width);
-  renderer->ops->set_linestyle(renderer, bezierline->line_style);
-  renderer->ops->set_dashlength(renderer, bezierline->dashlength);
-  renderer->ops->set_linejoin(renderer, LINEJOIN_MITER);
-  renderer->ops->set_linecaps(renderer, LINECAPS_BUTT);
+  renderer_ops->set_linewidth(renderer, bezierline->line_width);
+  renderer_ops->set_linestyle(renderer, bezierline->line_style);
+  renderer_ops->set_dashlength(renderer, bezierline->dashlength);
+  renderer_ops->set_linejoin(renderer, LINEJOIN_MITER);
+  renderer_ops->set_linecaps(renderer, LINECAPS_BUTT);
 
-  renderer->ops->draw_bezier(renderer, bez->points, bez->numpoints,
+  renderer_ops->draw_bezier_with_arrows(renderer, bez->points, bez->numpoints,
+					 bezierline->line_width,
+					 &bezierline->line_color,
+					 &bezierline->start_arrow,
+					 &bezierline->end_arrow);
+
+#if 0
+  renderer_ops->draw_bezier(renderer, bez->points, bez->numpoints,
 			     &bezierline->line_color);
 
   if (bezierline->start_arrow.type != ARROW_NONE) {
@@ -251,11 +267,15 @@ bezierline_draw(Bezierline *bezierline, Renderer *renderer)
 	       bezierline->line_width,
 	       &bezierline->line_color, &color_white);
   }
+#endif
 
-  /* these lines should only be displayed when object is selected.
-   * Unfortunately the draw function is not aware of the selected
-   * state.  This is a compromise until I fix this properly. */
-  if (renderer->is_interactive) {
+  /* Only display lines while selected.  Calling dia_object_is_selected()
+   * every time may take a little long.  Some time can be saved by storing
+   * whether the object is currently selected in bezierline_select, and
+   * only checking while selected.  But we'll do that if needed.
+   */
+  if (renderer->is_interactive &&
+      dia_object_is_selected((Object*)bezierline)) {
     bezierconn_draw_control_lines(&bezierline->bez, renderer);
   }
 }
@@ -271,23 +291,29 @@ bezierline_create(Point *startpoint,
   Object *obj;
   Point defaultlen = { .3, .3 };
 
-  /*bezierline_init_defaults();*/
-  bezierline = g_new(Bezierline, 1);
+  bezierline = g_new0(Bezierline, 1);
   bez = &bezierline->bez;
-  obj = (Object *) bezierline;
+  obj = &bez->object;
   
   obj->type = &bezierline_type;
   obj->ops = &bezierline_ops;
 
-  bezierconn_init(bez);
-  
-  bez->points[0].p1 = *startpoint;
-  bez->points[1].p1 = *startpoint;
-  point_add(&bez->points[1].p1, &defaultlen);
-  bez->points[1].p2 = bez->points[1].p1;
-  point_add(&bez->points[1].p2, &defaultlen);
-  bez->points[1].p3 = bez->points[1].p2;
-  point_add(&bez->points[1].p3, &defaultlen);
+  if (user_data == NULL) {
+    bezierconn_init(bez, 2);
+    
+    bez->points[0].p1 = *startpoint;
+    bez->points[1].p1 = *startpoint;
+    point_add(&bez->points[1].p1, &defaultlen);
+    bez->points[1].p2 = bez->points[1].p1;
+    point_add(&bez->points[1].p2, &defaultlen);
+    bez->points[1].p3 = bez->points[1].p2;
+    point_add(&bez->points[1].p3, &defaultlen);
+  } else {
+    BezierCreateData *bcd = (BezierCreateData*)user_data;
+
+    bezierconn_init(bez, bcd->num_points);
+    bezierconn_set_points(bez, bcd->num_points, bcd->points);
+  }
 
   bezierline_update_data(bezierline);
 
@@ -300,7 +326,7 @@ bezierline_create(Point *startpoint,
 
   *handle1 = bez->object.handles[0];
   *handle2 = bez->object.handles[3];
-  return (Object *)bezierline;
+  return &bezierline->bez.object;
 }
 
 static void
@@ -318,9 +344,9 @@ bezierline_copy(Bezierline *bezierline)
   
   bez = &bezierline->bez;
  
-  newbezierline = g_new(Bezierline, 1);
+  newbezierline = g_new0(Bezierline, 1);
   newbez = &newbezierline->bez;
-  newobj = (Object *) newbezierline;
+  newobj = &bez->object;
 
   bezierconn_copy(bez, newbez);
 
@@ -331,7 +357,7 @@ bezierline_copy(Bezierline *bezierline)
   newbezierline->start_arrow = bezierline->start_arrow;
   newbezierline->end_arrow = bezierline->end_arrow;
 
-  return (Object *)newbezierline;
+  return &newbezierline->bez.object;
 }
 
 
@@ -339,31 +365,24 @@ static void
 bezierline_update_data(Bezierline *bezierline)
 {
   BezierConn *bez = &bezierline->bez;
-  Object *obj = (Object *) bezierline;
+  Object *obj = &bez->object;
+  PolyBBExtras *extra = &bez->extra_spacing;
 
   bezierconn_update_data(bez);
     
+  extra->start_trans =  (bezierline->line_width / 2.0);
+  extra->end_trans =     (bezierline->line_width / 2.0);
+  extra->middle_trans = (bezierline->line_width / 2.0);
+    
+  if (bezierline->start_arrow.type != ARROW_NONE) 
+    extra->start_trans = MAX(extra->start_trans,bezierline->start_arrow.width);
+  if (bezierline->end_arrow.type != ARROW_NONE) 
+    extra->end_trans = MAX(extra->end_trans,bezierline->end_arrow.width);
+
+  extra->start_long = (bezierline->line_width / 2.0);
+  extra->end_long   = (bezierline->line_width / 2.0);
+
   bezierconn_update_boundingbox(bez);
-  /* fix boundingbox for line_width: */
-  obj->bounding_box.top -= bezierline->line_width/2;
-  obj->bounding_box.left -= bezierline->line_width/2;
-  obj->bounding_box.bottom += bezierline->line_width/2;
-  obj->bounding_box.right += bezierline->line_width/2;
-
-  /* Fix boundingbox for arrowheads */
-  if (bezierline->start_arrow.type != ARROW_NONE ||
-      bezierline->end_arrow.type != ARROW_NONE) {
-    real arrow_width = 0.0;
-    if (bezierline->start_arrow.type != ARROW_NONE)
-      arrow_width = bezierline->start_arrow.width;
-    if (bezierline->end_arrow.type != ARROW_NONE)
-      arrow_width = MAX(arrow_width, bezierline->start_arrow.width);
-
-    obj->bounding_box.top -= arrow_width;
-    obj->bounding_box.left -= arrow_width;
-    obj->bounding_box.bottom += arrow_width;
-    obj->bounding_box.right += arrow_width;
-  }
 
   obj->position = bez->points[0].p1;
 }
@@ -418,10 +437,10 @@ bezierline_load(ObjectNode obj_node, int version, const char *filename)
   Object *obj;
   AttributeNode attr;
 
-  bezierline = g_new(Bezierline, 1);
+  bezierline = g_new0(Bezierline, 1);
 
   bez = &bezierline->bez;
-  obj = (Object *) bezierline;
+  obj = &bez->object;
   
   obj->type = &bezierline_type;
   obj->ops = &bezierline_ops;
@@ -449,8 +468,8 @@ bezierline_load(ObjectNode obj_node, int version, const char *filename)
     bezierline->dashlength = data_real(attribute_first_data(attr));
 
   bezierline->start_arrow.type = ARROW_NONE;
-  bezierline->start_arrow.length = 0.8;
-  bezierline->start_arrow.width = 0.8;
+  bezierline->start_arrow.length = DEFAULT_ARROW_LENGTH;
+  bezierline->start_arrow.width = DEFAULT_ARROW_WIDTH;
   attr = object_find_attribute(obj_node, "start_arrow");
   if (attr != NULL)
     bezierline->start_arrow.type = data_enum(attribute_first_data(attr));
@@ -476,7 +495,7 @@ bezierline_load(ObjectNode obj_node, int version, const char *filename)
 
   bezierline_update_data(bezierline);
 
-  return (Object *)bezierline;
+  return &bezierline->bez.object;
 }
 
 static ObjectChange *
