@@ -21,18 +21,18 @@
 #endif
 
 #include <assert.h>
-#include <gtk/gtk.h>
 #include <math.h>
 
 #include "intl.h"
 #include "object.h"
 #include "orth_conn.h"
 #include "connectionpoint.h"
-#include "render.h"
+#include "diarenderer.h"
 #include "attributes.h"
 #include "widgets.h"
 #include "message.h"
 #include "properties.h"
+#include "autoroute.h"
 
 #include "pixmaps/zigzag.xpm"
 
@@ -51,12 +51,13 @@ typedef struct _Zigzagline {
 } Zigzagline;
 
 
-static void zigzagline_move_handle(Zigzagline *zigzagline, Handle *handle,
-				   Point *to, HandleMoveReason reason, ModifierKeys modifiers);
-static void zigzagline_move(Zigzagline *zigzagline, Point *to);
+static ObjectChange* zigzagline_move_handle(Zigzagline *zigzagline, Handle *handle,
+					    Point *to, ConnectionPoint *cp,
+					    HandleMoveReason reason, ModifierKeys modifiers);
+static ObjectChange* zigzagline_move(Zigzagline *zigzagline, Point *to);
 static void zigzagline_select(Zigzagline *zigzagline, Point *clicked_point,
-			      Renderer *interactive_renderer);
-static void zigzagline_draw(Zigzagline *zigzagline, Renderer *renderer);
+			      DiaRenderer *interactive_renderer);
+static void zigzagline_draw(Zigzagline *zigzagline, DiaRenderer *renderer);
 static Object *zigzagline_create(Point *startpoint,
 				 void *user_data,
 				 Handle **handle1,
@@ -69,17 +70,13 @@ static DiaMenu *zigzagline_get_object_menu(Zigzagline *zigzagline,
 					   Point *clickedpoint);
 
 static PropDescription *zigzagline_describe_props(Zigzagline *zigzagline);
-static void zigzagline_get_props(Zigzagline *zigzagline, Property *props,
-				 guint nprops);
-static void zigzagline_set_props(Zigzagline *zigzagline, Property *props,
-				 guint nprops);
+static void zigzagline_get_props(Zigzagline *zigzagline, GPtrArray *props);
+static void zigzagline_set_props(Zigzagline *zigzagline, GPtrArray *props);
 
 static void zigzagline_save(Zigzagline *zigzagline, ObjectNode obj_node,
 			    const char *filename);
 static Object *zigzagline_load(ObjectNode obj_node, int version,
 			       const char *filename);
-/* static GtkWidget *zigzagline_get_defaults();
-   static void zigzagline_apply_defaults(); */
 
 static ObjectTypeOps zigzagline_type_ops =
 {
@@ -119,7 +116,7 @@ static ObjectOps zigzagline_ops = {
 };
 
 static PropDescription zigzagline_props[] = {
-  OBJECT_COMMON_PROPERTIES,
+  ORTHCONN_COMMON_PROPERTIES,
   PROP_STD_LINE_WIDTH,
   PROP_STD_LINE_COLOUR,
   PROP_STD_LINE_STYLE,
@@ -137,7 +134,7 @@ zigzagline_describe_props(Zigzagline *zigzagline)
 }
 
 static PropOffset zigzagline_offsets[] = {
-  OBJECT_COMMON_PROPERTIES_OFFSETS,
+  ORTHCONN_COMMON_PROPERTIES_OFFSETS,
   { "line_width", PROP_TYPE_REAL, offsetof(Zigzagline, line_width) },
   { "line_colour", PROP_TYPE_COLOUR, offsetof(Zigzagline, line_color) },
   { "line_style", PROP_TYPE_LINESTYLE,
@@ -148,17 +145,17 @@ static PropOffset zigzagline_offsets[] = {
 };
 
 static void
-zigzagline_get_props(Zigzagline *zigzagline, Property *props, guint nprops)
+zigzagline_get_props(Zigzagline *zigzagline, GPtrArray *props)
 {
   object_get_props_from_offsets(&zigzagline->orth.object, zigzagline_offsets,
-				props, nprops);
+				props);
 }
 
 static void
-zigzagline_set_props(Zigzagline *zigzagline, Property *props, guint nprops)
+zigzagline_set_props(Zigzagline *zigzagline, GPtrArray *props)
 {
   object_set_props_from_offsets(&zigzagline->orth.object, zigzagline_offsets,
-				props, nprops);
+				props);
   zigzagline_update_data(zigzagline);
 }
 
@@ -171,34 +168,43 @@ zigzagline_distance_from(Zigzagline *zigzagline, Point *point)
 
 static void
 zigzagline_select(Zigzagline *zigzagline, Point *clicked_point,
-		  Renderer *interactive_renderer)
+		  DiaRenderer *interactive_renderer)
 {
   orthconn_update_data(&zigzagline->orth);
 }
 
-static void
+static ObjectChange*
 zigzagline_move_handle(Zigzagline *zigzagline, Handle *handle,
-		       Point *to, HandleMoveReason reason, ModifierKeys modifiers)
+		       Point *to, ConnectionPoint *cp,
+		       HandleMoveReason reason, ModifierKeys modifiers)
 {
+  ObjectChange *change;
   assert(zigzagline!=NULL);
   assert(handle!=NULL);
   assert(to!=NULL);
 
-  orthconn_move_handle(&zigzagline->orth, handle, to, reason);
+  change = orthconn_move_handle((OrthConn*)zigzagline, handle, to, cp, 
+				reason, modifiers);
+
   zigzagline_update_data(zigzagline);
+
+  return change;
 }
 
 
-static void
+static ObjectChange*
 zigzagline_move(Zigzagline *zigzagline, Point *to)
 {
   orthconn_move(&zigzagline->orth, to);
   zigzagline_update_data(zigzagline);
+
+  return NULL;
 }
 
 static void
-zigzagline_draw(Zigzagline *zigzagline, Renderer *renderer)
+zigzagline_draw(Zigzagline *zigzagline, DiaRenderer *renderer)
 {
+  DiaRendererClass *renderer_ops = DIA_RENDERER_GET_CLASS (renderer);
   OrthConn *orth = &zigzagline->orth;
   Point *points;
   int n;
@@ -206,28 +212,18 @@ zigzagline_draw(Zigzagline *zigzagline, Renderer *renderer)
   points = &orth->points[0];
   n = orth->numpoints;
   
-  renderer->ops->set_linewidth(renderer, zigzagline->line_width);
-  renderer->ops->set_linestyle(renderer, zigzagline->line_style);
-  renderer->ops->set_dashlength(renderer, zigzagline->dashlength);
-  renderer->ops->set_linejoin(renderer, LINEJOIN_MITER);
-  renderer->ops->set_linecaps(renderer, LINECAPS_BUTT);
+  renderer_ops->set_linewidth(renderer, zigzagline->line_width);
+  renderer_ops->set_linestyle(renderer, zigzagline->line_style);
+  renderer_ops->set_dashlength(renderer, zigzagline->dashlength);
+  renderer_ops->set_linejoin(renderer, LINEJOIN_MITER);
+  renderer_ops->set_linecaps(renderer, LINECAPS_BUTT);
 
-  renderer->ops->draw_polyline(renderer, points, n, &zigzagline->line_color);
-  
-  if (zigzagline->start_arrow.type != ARROW_NONE) {
-    arrow_draw(renderer, zigzagline->start_arrow.type,
-	       &points[0], &points[1],
-	       zigzagline->start_arrow.length, zigzagline->start_arrow.width, 
-	       zigzagline->line_width,
-	       &zigzagline->line_color, &color_white);
-  }
-  if (zigzagline->end_arrow.type != ARROW_NONE) {
-    arrow_draw(renderer, zigzagline->end_arrow.type,
-	       &points[n-1], &points[n-2],
-	       zigzagline->end_arrow.length, zigzagline->end_arrow.width, 
-	       zigzagline->line_width,
-	       &zigzagline->line_color, &color_white);
-  }
+  renderer_ops->draw_polyline_with_arrows(renderer,
+					   points, n,
+					   zigzagline->line_width,
+					   &zigzagline->line_color,
+					   &zigzagline->start_arrow,
+					   &zigzagline->end_arrow);
 }
 
 static Object *
@@ -292,6 +288,8 @@ zigzagline_copy(Zigzagline *zigzagline)
   newzigzagline->start_arrow = zigzagline->start_arrow;
   newzigzagline->end_arrow = zigzagline->end_arrow;
 
+  zigzagline_update_data(newzigzagline);
+
   return &newzigzagline->orth.object;
 }
 
@@ -299,7 +297,7 @@ static void
 zigzagline_update_data(Zigzagline *zigzagline)
 {
   OrthConn *orth = &zigzagline->orth;
-  OrthConnBBExtras *extra = &orth->extra_spacing;
+  PolyBBExtras *extra = &orth->extra_spacing;
 
   orthconn_update_data(&zigzagline->orth);
     
@@ -338,6 +336,7 @@ zigzagline_delete_segment_callback(Object *obj, Point *clicked, gpointer data)
 static DiaMenuItem object_menu_items[] = {
   { N_("Add segment"), zigzagline_add_segment_callback, NULL, 1 },
   { N_("Delete segment"), zigzagline_delete_segment_callback, NULL, 1 },
+  ORTHCONN_COMMON_MENUS,
 };
 
 static DiaMenu object_menu = {
@@ -356,6 +355,8 @@ zigzagline_get_object_menu(Zigzagline *zigzagline, Point *clickedpoint)
   /* Set entries sensitive/selected etc here */
   object_menu_items[0].active = orthconn_can_add_segment(orth, clickedpoint);
   object_menu_items[1].active = orthconn_can_delete_segment(orth, clickedpoint);
+  orthconn_update_object_menu(orth, clickedpoint, &object_menu_items[2]);
+
   return &object_menu;
 }
 
@@ -436,8 +437,8 @@ zigzagline_load(ObjectNode obj_node, int version, const char *filename)
     zigzagline->line_style = data_enum(attribute_first_data(attr));
 
   zigzagline->start_arrow.type = ARROW_NONE;
-  zigzagline->start_arrow.length = 0.8;
-  zigzagline->start_arrow.width = 0.8;
+  zigzagline->start_arrow.length = DEFAULT_ARROW_LENGTH;
+  zigzagline->start_arrow.width = DEFAULT_ARROW_WIDTH;
   attr = object_find_attribute(obj_node, "start_arrow");
   if (attr != NULL)
     zigzagline->start_arrow.type = data_enum(attribute_first_data(attr));
@@ -449,8 +450,8 @@ zigzagline_load(ObjectNode obj_node, int version, const char *filename)
     zigzagline->start_arrow.width = data_real(attribute_first_data(attr));
 
   zigzagline->end_arrow.type = ARROW_NONE;
-  zigzagline->end_arrow.length = 0.8;
-  zigzagline->end_arrow.width = 0.8;
+  zigzagline->end_arrow.length = DEFAULT_ARROW_LENGTH;
+  zigzagline->end_arrow.width = DEFAULT_ARROW_WIDTH;
   attr = object_find_attribute(obj_node, "end_arrow");
   if (attr != NULL)
     zigzagline->end_arrow.type = data_enum(attribute_first_data(attr));
