@@ -35,16 +35,55 @@
 #include "pydia-image.h"
 #include "pydia-properties.h"
 #include "pydia-error.h"
+#include "pydia-text.h"
 
 #include "object.h"
+#include "group.h"
 #include "app/diagram.h"
 #include "app/display.h"
 #include "app/load_save.h"
 
-#ifdef G_OS_WIN32
-#pragma message("FIXME: open_diagrams")
-#define open_diagrams NULL
-#endif
+#include "lib/message.h"
+
+static PyObject *
+PyDia_GroupCreate(PyObject *self, PyObject *args)
+{
+    int i, len;
+    GList *list = NULL;
+    PyObject *lst;
+    PyObject *ret;
+
+    if (!PyArg_ParseTuple(args, "O!:dia.group_create",
+                          &PyList_Type, &lst))
+	return NULL;
+
+    len = PyList_Size(lst);
+    for (i = 0; i < len; i++)
+      {
+        PyObject *o = PyList_GetItem(lst, i);
+
+        if (0 && !PyDiaObject_Check(o))
+          {
+            PyErr_SetString(PyExc_TypeError, "Only DiaObjects can be grouped.");
+            g_list_free(list);
+            return NULL;
+          }
+
+        list = g_list_append(list, ((PyDiaObject *)o)->object);
+      }
+
+    if (list)
+      ret = PyDiaObject_New(group_create(list));
+    else
+      {
+        Py_INCREF(Py_None);
+        ret = Py_None;
+      }
+    //Urgh : group_create() eats list
+    //g_list_free(list);
+
+    return ret;
+}
 
 static PyObject *
 PyDia_Diagrams(PyObject *self, PyObject *args)
@@ -55,7 +94,7 @@ PyDia_Diagrams(PyObject *self, PyObject *args)
     if (!PyArg_ParseTuple(args, ":dia.diagrams"))
 	return NULL;
     ret = PyList_New(0);
-    for (tmp = open_diagrams; tmp; tmp = tmp->next)
+    for (tmp = dia_open_diagrams(); tmp; tmp = tmp->next)
 	PyList_Append(ret, PyDiaDiagram_New((Diagram *)tmp->data));
     return ret;
 }
@@ -75,6 +114,22 @@ PyDia_Load(PyObject *self, PyObject *args)
     return NULL;
 }
 
+static PyObject *
+PyDia_New(PyObject *self, PyObject *args)
+{
+    Diagram *dia;
+    gchar *filename;
+
+    if (!PyArg_ParseTuple(args, "s:dia.new", &filename))
+	return NULL;
+
+    dia = new_diagram(filename);
+    if (dia)
+	return PyDiaDiagram_New(dia);
+    PyErr_SetString(PyExc_IOError, "could not create diagram");
+    return NULL;
+}
+
 static PyObject *
 PyDia_GetObjectType(PyObject *self, PyObject *args)
 {
@@ -90,6 +145,42 @@ PyDia_GetObjectType(PyObject *self, PyObject *args)
     return NULL;
 }
 
+/*
+ * A dictionary interface to all registered object(-types)
+ */
+static void
+_ot_item (gpointer key,
+          gpointer value,
+          gpointer user_data)
+{
+    gchar *name = (gchar *)key;
+    ObjectType *type = (ObjectType *)value;
+    PyObject *dict = (PyObject *)user_data;
+    PyObject *k, *v;
+
+    k = PyString_FromString(name);
+    v = PyDiaObjectType_New(type);
+    if (k && v)
+        PyDict_SetItem(dict, k, v);
+    Py_XDECREF(k);
+    Py_XDECREF(v);
+}
+
+static PyObject *
+PyDia_RegisteredTypes(PyObject *self, PyObject *args)
+{
+    PyObject *dict;
+
+    if (!PyArg_ParseTuple(args, ":dia.registered_types"))
+	return NULL;
+
+    dict = PyDict_New();
+
+    object_registry_foreach(_ot_item, dict);
+
+    return dict;
+}
+
 static PyObject *
 PyDia_ActiveDisplay(PyObject *self, PyObject *args)
 {
@@ -111,7 +202,7 @@ PyDia_UpdateAll(PyObject *self, PyObject *args)
 
     if (!PyArg_ParseTuple(args, ":dia.update_all"))
 	return NULL;
-    for (tmp = open_diagrams; tmp; tmp = tmp->next)
+    for (tmp = dia_open_diagrams(); tmp; tmp = tmp->next)
 	diagram_add_update_all((Diagram *)tmp->data);
     Py_INCREF(Py_None);
     return Py_None;
@@ -146,6 +237,70 @@ PyDia_RegisterExport(PyObject *self, PyObject *args)
     return obj;
 }
 
+
+/*
+ * This function gets called by Dia as a reaction to file import.
+ * It needs to be registered before via Python function 
+ * dia.register_import
+ */
+gboolean
+PyDia_import_data (const gchar* filename, DiagramData *dia, void *user_data)
+{
+    PyObject *diaobj, *res, *arg, *func = user_data;
+    if (!func || !PyCallable_Check (func)) {
+        message_error ("Import called without valid callback function.");
+        return FALSE;
+    }
+    if (dia)
+        diaobj = PyDiaDiagramData_New (dia);
+    else {
+        diaobj = Py_None;
+        Py_INCREF (diaobj);
+    }
+      
+    Py_INCREF(func);
+
+    arg = Py_BuildValue ("(sO)", filename, diaobj);
+    if (arg) {
+      res = PyEval_CallObject (func, arg);
+      ON_RES(res);
+    }
+    Py_XDECREF (arg);
+
+    Py_DECREF(func);
+    Py_XDECREF(diaobj);
+
+    return !!res;
+}
+
+static PyObject *
+PyDia_RegisterImport(PyObject *self, PyObject *args)
+{
+    gchar *name;
+    gchar *ext;
+    DiaImportFilter *filter;
+    PyObject* func;
+
+    if (!PyArg_ParseTuple(args, "ssO:dia.register_import",
+			  &name, &ext, &func))
+	return NULL;
+
+    Py_INCREF(func); /* stay alive, where to kill ?? */
+
+    filter = g_new (DiaImportFilter, 1);
+    filter->description = g_strdup (name);
+    filter->extensions = g_new (gchar*, 2);
+    filter->extensions[0] = g_strdup (ext);
+    filter->extensions[1] = NULL;
+    filter->import = &PyDia_import_data;
+    filter->user_data = func;
+
+    filter_register_import(filter);
+
+    Py_INCREF(Py_None);
+    return Py_None;
+}
+
 /*
  * This function gets called by Dia as a reaction to a menu item.
  * It needs to be registered before via Python function 
@@ -211,13 +366,39 @@ PyDia_RegisterCallback(PyObject *self, PyObject *args)
     return Py_None;
 }
 
+static PyObject *
+PyDia_Message (PyObject *self, PyObject *args)
+{
+    int type = 0;
+    char *text = "Huh?";
+
+    if (!PyArg_ParseTuple(args, "is:dia.message",
+			  &type, &text))
+	return NULL;
+
+    if (0 == type)
+	message_notice (text);
+    else if (1 == type)
+	message_warning (text);
+    else
+	message_error (text);
+
+    Py_INCREF(Py_None);
+    return Py_None;
+}
+
 static PyMethodDef dia_methods[] = {
+    { "group_create", PyDia_GroupCreate, 1 },
     { "diagrams", PyDia_Diagrams, 1 },
     { "load", PyDia_Load, 1 },
+    { "message", PyDia_Message, 1 },
+    { "new", PyDia_New, 1 },
     { "get_object_type", PyDia_GetObjectType, 1 },
+    { "registered_types", PyDia_RegisteredTypes, 1 },
     { "active_display", PyDia_ActiveDisplay, 1 },
     { "update_all", PyDia_UpdateAll, 1 },
     { "register_export", PyDia_RegisterExport, 1 },
+    { "register_import", PyDia_RegisterImport, 1 },
     { "register_callback", PyDia_RegisterCallback, 1},
     { NULL, NULL }
 };
@@ -252,6 +433,7 @@ initdia(void)
     PyDiaProperties_Type.ob_type = &PyType_Type;
     PyDiaError_Type.ob_type = &PyType_Type;
     PyDiaArrow_Type.ob_type = &PyType_Type;
+    PyDiaText_Type.ob_type = &PyType_Type;
 #endif
 
     m = Py_InitModule("dia", dia_methods);
@@ -295,6 +477,8 @@ initdia(void)
 			 (PyObject *)&PyDiaError_Type);
     PyDict_SetItemString(d, "DiaArrow",
 			 (PyObject *)&PyDiaArrow_Type);
+    PyDict_SetItemString(d, "DiaText",
+			 (PyObject *)&PyDiaText_Type);
 
     if (PyErr_Occurred())
 	Py_FatalError("can't initialise module dia");
